///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.apps.clnews;

import java.io.File;
import java.io.IOException;
import java.awt.Frame;
import org.cooldevtools.hdl.*;
import org.cooldevtools.utility.*;
import org.cooldevtools.utility.logging.*;
import org.cooldevtools.coffee_server.config.*;
import org.cooldevtools.coffee_server.framework.*;
import org.cooldevtools.coffee_server.services.nntp.pub.*;
import org.cooldevtools.coffee_server.services.nntp.protocol.*;
import org.cooldevtools.apps.clnews.configure.Constants;
import org.cooldevtools.apps.clnews.repository.*;

public class NntpServiceManager
extends BasicCoffeeServiceManager
{
   public NntpServiceManager()
   {
   }

   public void shutdown()
   throws CoffeeServiceException
   {
      Logger.s().logSilently(MessagePriority.LOG_NOTICE, SHUTDOWN_MESSAGE);
      try {
         m_poster.shutdown();
      } catch (InterruptedException ex) {
      }
      m_NewsFactory.shutdown();
   }

   private void createServiceInstance()
   {
      if (Assert.DEBUG) Assert.pre(m_poster == null);
      if (Assert.DEBUG) Assert.pre(m_service == null);

      m_poster = new NntpService_POST();

      m_service = new NntpService();
      m_service.setNewsFactory(m_NewsFactory);
      m_service.addCommand("article", new NntpService_ARTICLE(NntpService_ARTICLE.MODE_ARTICLE));
      m_service.addCommand("authinfo", new NntpService_AUTHINFO());
      m_service.addCommand("body", new NntpService_ARTICLE(NntpService_ARTICLE.MODE_BODY));
      m_service.addCommand("date", new NntpService_DATE());
      m_service.addCommand("group", new NntpService_GROUP());
      m_service.addCommand("head", new NntpService_ARTICLE(NntpService_ARTICLE.MODE_HEADER));
      m_service.addCommand("help", new NntpService_HELP());
      m_service.addCommand("ihave", new NntpService_IHAVE(m_poster));
      m_service.addCommand("last", new NntpService_LAST());
      m_service.addCommand("list", new NntpService_LIST());
      m_service.addCommand("listgroup", new NntpService_LISTGROUP());
      m_service.addCommand("mode", new NntpService_MODE_READER());
      m_service.addCommand("newgroups", new NntpService_NEWGROUPS());
      m_service.addCommand("newnews", new NntpService_NEWNEWS());
      m_service.addCommand("next", new NntpService_NEXT());
      m_service.addCommand("over", new NntpService_XOVER());
      m_service.addCommand("pat", new NntpService_PAT());
      m_service.addCommand("post", m_poster);
      m_service.addCommand("quit", new NntpService_QUIT());
      m_service.addCommand("stat", new NntpService_ARTICLE(NntpService_ARTICLE.MODE_STAT));
      m_service.addCommand("xhdr", new NntpService_XHDR());
      m_service.addCommand("xlistgroup", new NntpService_LISTGROUP());
      m_service.addCommand("xmode", new NntpService_MODE_READER());
      m_service.addCommand("xover", new NntpService_XOVER());
      m_service.addCommand("bcc_define_group", new NntpService_BCC_DEFINE_GROUP());
      m_service.addCommand("bcc_delete_group", new NntpService_BCC_DELETE_GROUP());
      m_service.addCommand("bcc_list_group", new NntpService_BCC_LIST_GROUP());
      m_service.addCommand("bcc_define_role", new NntpService_BCC_DEFINE_ROLE());
      m_service.addCommand("bcc_delete_role", new NntpService_BCC_DELETE_ROLE());
      m_service.addCommand("bcc_list_role", new NntpService_BCC_LIST_ROLE());
      m_service.addCommand("bcc_list_roles", new NntpService_BCC_LIST_ROLES());
      m_service.addCommand("bcc_define_user", new NntpService_BCC_DEFINE_USER());
      m_service.addCommand("bcc_delete_user", new NntpService_BCC_DELETE_USER());
      m_service.addCommand("bcc_list_user", new NntpService_BCC_LIST_USER());
      m_service.addCommand("bcc_list_users", new NntpService_BCC_LIST_USERS());
      m_service.addCommand("bcc_set_max_connect", new NntpService_BCC_SET_MAX_CONNECT());
      m_service.addCommand("bcc_set_max_message_size", new NntpService_BCC_SET_MAX_MESSAGE_SIZE());
      m_service.addCommand("bcc_set_max_message_days", new NntpService_BCC_SET_MAX_MESSAGE_DAYS());
      m_service.addCommand("bcc_set_ports", new NntpService_BCC_SET_PORTS());
      m_service.addCommand("bcc_set_logging", new NntpService_BCC_SET_LOGGING());
      m_service.addCommand("bcc_set_log_level", new NntpService_BCC_SET_LOG_LEVEL());
      m_service.addCommand("bcc_set_unix_identity", new NntpService_BCC_SET_UNIX_IDENTITY());
      m_service.addCommand("bcc_get_max_connect", new NntpService_BCC_GET_MAX_CONNECT());
      m_service.addCommand("bcc_get_max_message_size", new NntpService_BCC_GET_MAX_MESSAGE_SIZE());
      m_service.addCommand("bcc_get_max_message_days", new NntpService_BCC_GET_MAX_MESSAGE_DAYS());
      m_service.addCommand("bcc_get_ports", new NntpService_BCC_GET_PORTS());
      m_service.addCommand("bcc_get_logging", new NntpService_BCC_GET_LOGGING());
      m_service.addCommand("bcc_get_log_level", new NntpService_BCC_GET_LOG_LEVEL());
      m_service.addCommand("bcc_get_unix_identity", new NntpService_BCC_GET_UNIX_IDENTITY());
      m_service.addCommand("bcc_change_user_password", new NntpService_BCC_CHANGE_USER_PASSWORD());
      m_service.addCommand("bcc_shutdown", new NntpService_BCC_SHUTDOWN());
      m_service.addCommand("bcc_save_config", new NntpService_BCC_SAVE_CONFIG());
   }

   public CoffeeService newServiceInstance()
   {
      return m_service;
   }

   public CoffeeServiceConfigBean newConfigBeanInstance()
   {
      return null;
   }

   private Statement loadSchema(String filename)
   throws HDLException, IOException
   {
      return new Parser().parse(filename);
   }

   public void configure(ServiceConfigInfo config)
   throws CoffeeServiceException
   {
      for (int i = 0; i < STARTUP_MESSAGE.length; i++) {
         Logger.s().logSilently(MessagePriority.LOG_NOTICE, STARTUP_MESSAGE[i]);
      }
      super.configure(config);
      Statement root = config.getConfiguration();
      try {
         m_installRoot = Misc.getInstallRoot();

         new SyntaxChecker(HDL_SYNTAX).checkSyntax(root);
         m_dbDirectory    = root.child(TOK_DATABASE_DIR).argument(0).strValue();
         m_dbSchemaFile   = root.child(TOK_SCHEMA_FILE).argument(0).strValue();
         m_dbSchema       = loadSchema(m_installRoot + m_dbSchemaFile);

         m_NewsFactory    = new FileNewsRepositoryFactory(config,
                                                          m_installRoot + m_dbDirectory,
                                                          m_dbSchema);
         createServiceInstance();
      } catch (HDLException ex) {
         throw new CoffeeServiceException(ex);
      } catch (IOException ex) {
         throw new CoffeeServiceException(ex);
      }

      FileRoleRepository roles = (FileRoleRepository)m_NewsFactory.getRoleRepository();
      boolean first_run = (roles.getRoleNames().length == 0);
      if (first_run) {
         String[] grp_patts  = new String[1];
         String[] grp_perms  = new String[1];
         String[] role_patts = new String[0];

         grp_patts[0] = "public.*";
         grp_perms[0] = "R";
         roles.defineRole("ReadPublic", "X", role_patts, grp_patts, grp_perms);
         grp_perms[0] = "C";
         roles.defineRole("default",    "X", role_patts, grp_patts, grp_perms);
         roles.defineRole("PostPublic", "X", role_patts, grp_patts, grp_perms);

         grp_patts[0] = "private.*";
         grp_perms[0] = "R";
         roles.defineRole("ReadPrivate", "X", role_patts, grp_patts, grp_perms);
         grp_perms[0] = "C";
         roles.defineRole("PostPrivate", "X", role_patts, grp_patts, grp_perms);

         grp_patts[0] = "*";
         grp_perms[0] = "R";
         roles.defineRole("ReadAll", "X", role_patts, grp_patts, grp_perms);
         grp_perms[0] = "C";
         roles.defineRole("PostAll", "X", role_patts, grp_patts, grp_perms);

         grp_patts[0] = "*";
         grp_perms[0] = "A";
         role_patts = new String[1];
         role_patts[0] = "*";
         roles.defineRole("admin", "R", role_patts, grp_patts, grp_perms);

         grp_patts[0] = "*";
         grp_perms[0] = "X";
         role_patts = new String[0];
         roles.defineRole("None", "X", role_patts, grp_patts, grp_perms);

         grp_patts = new String[2];
         grp_patts[0] = "private.*";
         grp_patts[1] = "*";
         grp_perms = new String[2];
         grp_perms[0] = "X";
         grp_perms[1] = "R";
         role_patts = new String[0];
         roles.defineRole("ReadNonPrivate", "X", role_patts, grp_patts, grp_perms);
         grp_perms[1] = "C";
         roles.defineRole("PostNonPrivate", "X", role_patts, grp_patts, grp_perms);

         UserRepository users = m_NewsFactory.getUserRepository();
         if (users.getUserNames().length == 0) {
            String[] role_names = { "default" };
            users.defineUser("default", "", "", role_names);
            role_names[0] = "admin";
            users.defineUser("admin", "admin", "", role_names);
         }

         NewsGroupRepository groups = m_NewsFactory.getNewsGroupRepository();
         if (groups.getNewsGroups().length == 0) {
            groups.createNewsGroup(users.getUser("admin","admin"), "public.discuss","", "");
         }
      }
      // check for and add None role if not found (part of 1.1 update)
      if (roles.getRoleDefinition("None") == null) {
         String[] role_patts = new String[0];
         String[] grp_patts  = new String[]{ "*" };
         String[] grp_perms  = new String[]{ "X" };
         roles.defineRole("None", "X", role_patts, grp_patts, grp_perms);
      }
   }

   public void updateConfiguration()
   throws CoffeeServiceException
   {
      Statement root = new Statement(TOK_CONFIG, true);

      Statement stmt = new Statement(TOK_DATABASE_DIR, false);
      stmt.addArgument(Token.strToken(m_dbDirectory, "", 0));
      root.addChild(stmt);

      stmt = new Statement(TOK_SCHEMA_FILE, false);
      stmt.addArgument(Token.strToken(m_dbSchemaFile));
      root.addChild(stmt);

      getConfiguration().setConfiguration(root);
   }

   private FileNewsRepositoryFactory m_NewsFactory;
   private String m_dbDirectory;
   private String m_dbSchemaFile;
   private Statement m_dbSchema;
   private String m_installRoot;
   private NntpService m_service;
   private NntpService_POST m_poster;

   private static final String HDL_SYNTAX =
      "begin language configuration;\n" +
      "  begin statement database_dirname;\n" +
      "    args 1 1 string;\n" +
      "  end;\n" +
      "  begin statement schema_filename;\n" +
      "    args 1 1 string;\n" +
      "  end;\n" +
      "end;\n";

   private static Token TOK_CONFIG       = Token.idToken("configuration", "", 0);
   private static Token TOK_DATABASE_DIR = Token.idToken("database_dirname", "", 0);
   private static Token TOK_MAX_MSG_SIZE = Token.idToken("max_message_size", "", 0);
   private static Token TOK_SCHEMA_FILE  = Token.idToken("schema_filename", "", 0);

   private static String[] STARTUP_MESSAGE = {
      Constants.PRODUCT_NAME_AND_VERSION + " (build " + Constants.BUILD_NUMBER + ")",
      "Copyright (C) 1998,1999,2000,2001 Burton Computer Corporation, All rights reserved"
   };
   private static String SHUTDOWN_MESSAGE  = Constants.PRODUCT_NAME + " shutting down";
}
