///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//


package org.cooldevtools.apps.clnews.configure;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.text.*;
import org.cooldevtools.utility.*;
import org.cooldevtools.awt.*;

public class LoginDialog extends JDialog
{
   public LoginDialog(Frame parent)
   {
      super(parent, "Login To Server", true);
      initDisplay();
   }

   private void initDisplay()
   {
      m_host = "localhost";
      m_port = 119;
      m_username = "admin";
      m_password = "";

      JPanel pane = new JPanel();
      pane.setBorder(new EmptyBorder(15, 15, 10, 15));
      pane.setLayout(new BorderLayout(15, 15));

      m_fields = new BasicFieldPanel();
      pane.add(m_fields, "Center");

      m_fields.addField(HOSTNAME, "Host: ", "", 20);
      m_fields.addField(PORT,     "Port: ", "", 5);
      m_fields.addField(USERNAME, "Username: ", "", 20);
      m_fields.addField(PASSWORD, "Password: ", "", 20);
      m_fields.getField(PORT).setDocument(new BasicIntegerDocument(false));
      
      m_fields.setFieldValue(HOSTNAME, m_host);
      m_fields.setIntFieldValue(PORT,  m_port);
      m_fields.setFieldValue(USERNAME, m_username);
      m_fields.setFieldValue(PASSWORD, m_password);

      m_fields.setMnemonic(HOSTNAME, 'H');
      m_fields.setMnemonic(PORT,     'P');
      m_fields.setMnemonic(USERNAME, 'U');
      m_fields.setMnemonic(PASSWORD, 'W');
      
      m_ok = new JButton("OK");
      m_cancel = new JButton("Cancel");
      
      ActionListener oker = new OkListener();
      ActionListener canceller = new CancelListener();
      m_ok.addActionListener(oker);
      m_cancel.addActionListener(canceller);

      m_ok.setMnemonic('O');
      m_cancel.setMnemonic('C');

      JPanel p = new JPanel();
      p.setLayout(new EqualSizeLayout(5));
      p.add(new Gap());
      p.add(m_ok);
      p.add(m_cancel);
      p.add(new Gap());
      pane.add(p, "South");

      getContentPane().add(pane);
      pack();
      setLocationRelativeTo(getParent());

      addWindowListener(new AutoRequestFocus(this, m_fields.getField(0)));
      DialogUtil.setKeyStrokeListener(this, KeyEvent.VK_ESCAPE, canceller);
      DialogUtil.setKeyStrokeListener(this, KeyEvent.VK_ENTER, oker);
   }

   public class OkListener implements ActionListener
   {
      public void actionPerformed(ActionEvent ev)
      {
	 m_host = m_fields.getFieldValue(HOSTNAME);
	 try {
	    m_port = m_fields.getIntFieldValue(PORT);
	 } catch (NumberFormatException ex) {
	    m_port = 0;
	 }
	 m_username = m_fields.getFieldValue(USERNAME);
	 m_password = m_fields.getFieldValue(PASSWORD);
	 m_cancelled = false;
         setVisible(false);
      }
   }

   public class CancelListener implements ActionListener
   {
      public void actionPerformed(ActionEvent ev) {
	 m_cancelled = true;
	 setVisible(false);
      }
   }

   public void setVisible(boolean visible)
   {
      if (visible) {
         m_fields.setFieldValue(HOSTNAME, m_host);
         m_fields.setIntFieldValue(PORT,  m_port);
         m_fields.setFieldValue(USERNAME, m_username);
         m_fields.setFieldValue(PASSWORD, m_password);
      }
      super.setVisible(visible);
   }

   public String getHost()
   {
      return m_host;
   }

   public int getPort()
   {
      return m_port;
   }

   public String getUser()
   {
      return m_username;
   }

   public String getPassword()
   {
      return m_password;
   }

   public boolean isCancelled()
   {
      return m_cancelled;
   }

   public static void main(String[] args)
   {
      LoginDialog r = new LoginDialog(null);
      r.show();
   }

   private BasicFieldPanel m_fields;
   private JButton m_ok;
   private JButton m_cancel;

   private String m_host;
   private int    m_port;
   private String m_username;
   private String m_password;
   private boolean m_cancelled;

   private static final int HOSTNAME = 0;
   private static final int PORT     = 1;
   private static final int USERNAME = 2;
   private static final int PASSWORD = 3;
}

