///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.apps.clnews.configure;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import org.cooldevtools.awt.*;
import org.cooldevtools.utility.*;
import org.cooldevtools.coffee_server.services.nntp.pub.UnexpectedNntpResponseException;

public class Main extends JFrame implements ErrorHandler
{
   public Main()
   {
      super(TITLE);
      initDisplay();
   }

   private void initDisplay()
   {
      addWindowListener(new WindowHandler());

      m_main = new MainConfigPanel(this);
      getContentPane().add(m_main);

      JMenuBar mb = new JMenuBar();

      JMenu menu = new JMenu("Server");
      menu.setMnemonic('S');
      JMenuItem item = new JMenuItem("Login To Server...");
      item.setMnemonic('L');
      item.addActionListener(new LoginItemListener());
      menu.add(item);
      item = new JMenuItem("Cancel Message On Server...");
      item.setMnemonic('C');
      item.addActionListener(new CancelMessageItemListener());
      menu.add(item);
      item = new JMenuItem("Shutdown Server");
      item.setMnemonic('S');
      item.addActionListener(new ShutdownItemListener());
      menu.add(item);
      menu.addSeparator();
      item = new JMenuItem("Exit");
      item.setMnemonic('X');
      item.addActionListener(new QuitItemListener());
      menu.add(item);
      mb.add(menu);

      menu = new JMenu("Help");
      menu.setMnemonic('H');
      item = new JMenuItem("About " + Constants.PRODUCT_NAME + "...");
      item.setMnemonic('A');
      menu.add(item);
      item.addActionListener(new AboutItemListener());
      mb.add(menu);

      setJMenuBar(mb);
      setTitle(TITLE);
   }

   private boolean login()
   {
      if (m_login == null) {
         m_login = new LoginDialog(this);
      }

      m_login.setVisible(true);
      if (!m_login.isCancelled()) {
         try {
            m_main.setServerParams(m_login.getHost(),
                                   m_login.getPort(),
                                   m_login.getUser(),
                                   m_login.getPassword());

            if (m_login.getUser().equals("admin") && m_login.getPassword().equals("admin")) {
               String message =
                  "Please change the \"admin\" user's password to something other " +
                  "than \"admin\" as soon as possible.";
               JOptionPane.showMessageDialog(Main.this,
                                             Misc.wrapString(message, DIALOG_WIDTH, 0, 0),
                                             "Security Warning",
                                             JOptionPane.WARNING_MESSAGE);
            }
            return true;
         } catch (IOException ex) {
            handleIOException(ex);
         }
      }
      return false;
   }

   private void shutdownRequested()
   {
      System.exit(0);
   }

   private void handleInvalidResponse(UnexpectedNntpResponseException ex)
   {
      String[] lines = null;
      if (ex.code() == PERMISSION_DENIED) {
         String tmp =
            "The current userid (" + m_main.getServer().getUserName() + ") does not have" +
            " permission to perform the requested operation." +
            " No changes have been made at the server.";
         lines = Misc.wrapString(tmp, DIALOG_WIDTH, 0, 0);
      } else {
         String tmp =
            "Unexpected response from the server: \n  " +
            "  \"" + ex.getMessage() + "\"\n" +
            "Check to ensure that the hostname and port used when logging" +
            " in refer to an operational " + Constants.PRODUCT_NAME + ".";
         lines = Misc.wrapString(tmp, DIALOG_WIDTH, 0, 0);
      }
      JOptionPane.showMessageDialog(Main.this,
                                    lines,
                                    "Operation Failure",
                                    JOptionPane.ERROR_MESSAGE);
   }

   public void handleIOException(IOException ex)
   {
      if (ex instanceof UnexpectedNntpResponseException) {
         handleInvalidResponse((UnexpectedNntpResponseException)ex);
      } else {
         if (Assert.DEBUG) ex.printStackTrace();
         String[] lines = Misc.wrapString(
            "Error encountered while communicating with the server:\n" +
            "  \"" + ex.getMessage() + "\"\n"+
            "\nPlease check to ensure that the server is running and" +
            " that your network connection to the server is operating" +
            " correctly.", DIALOG_WIDTH, 0, 0);
         JOptionPane.showMessageDialog(Main.this,
                                       lines,
                                       "Operation Failure",
                                       JOptionPane.ERROR_MESSAGE);
      }
   }

   public class LoginItemListener implements ActionListener
   {
      public void actionPerformed(ActionEvent e)
      {
         login();
      }
   }

   public class WindowHandler extends WindowAdapter
   {
      public void windowOpened(WindowEvent ev)
      {
         if (!login()) {
            System.exit(0);
         }
      }

      public void windowClosing(WindowEvent ev)
      {
         shutdownRequested();
      }
   }

   public class QuitItemListener implements ActionListener
   {
      public void actionPerformed(ActionEvent e)
      {
         shutdownRequested();
      }
   }

   public class AboutItemListener implements ActionListener
   {
      public void actionPerformed(ActionEvent e)
      {
         String[] lines = {
            Constants.PRODUCT_NAME_AND_VERSION + "\n",
            "Copyright (C) 1998,1999,2000,2001 Burton Computer Corporation\n",
            "All Rights Reserved\n",
            "http://www.burton-computer.com\n",
            " \n",
            "WARNING: This program is protected by copyright law and ",
            "international treaties.",
         };
         JOptionPane.showMessageDialog(Main.this,
                                       Misc.wrapStrings(lines, DIALOG_WIDTH, 0, 0),
                                       "About " + Constants.PRODUCT_NAME,
                                       JOptionPane.INFORMATION_MESSAGE);
      }
   }

   public class ShutdownItemListener implements ActionListener
   {
      public void actionPerformed(ActionEvent e)
      {
         String[] lines = {
            "Once shut down, the server must be",
            "restarted manually on the server host.",
            "Really shut down the server?",
         };
         int rc = JOptionPane.showConfirmDialog(Main.this,
                                                lines,
                                                "Confirm Server Shutdown",
                                                JOptionPane.YES_NO_OPTION);
         if (rc == JOptionPane.YES_OPTION) {
            try {
               m_main.getServer().shutdown();
            } catch (IOException ex) {
               handleIOException(ex);
            }
         }
      }
   }

   public class CancelMessageItemListener implements ActionListener
   {
      public void actionPerformed(ActionEvent e)
      {
         String message_id = JOptionPane.showInputDialog(m_main,
                                                         "Message ID of message to cancel:",
                                                         "Cancel Message",
                                                         JOptionPane.QUESTION_MESSAGE);
         if (message_id == null) {
            return;
         }

         int open_index = message_id.indexOf('<');
         int close_index = message_id.lastIndexOf('>');
         if (open_index != 0 || close_index < 1 || open_index > close_index) {
            String[] lines = {
               "The value you entered: ",
               "  \"" + message_id + "\"",
               "is not a valid message-ID.  Message-IDs always begin with",
               "a '<' character and end with a '>' character.  An article's",
               "message-ID can be found near the beginning of the article on",
               "a line beginning with \"Message-ID: \""
            };
            JOptionPane.showMessageDialog(Main.this,
                                          lines,
                                          "Invalid Message-ID",
                                          JOptionPane.ERROR_MESSAGE);
            return;
         }

         try {
            m_main.getServer().cancel(message_id);
         } catch (IOException ex) {
            handleIOException(ex);
         }
      }
   }

   public static void main(String[] args)
   {
      Main r = new Main();
      r.pack();

      Dimension paneSize = r.getSize();
      Dimension screenSize = r.getToolkit().getScreenSize();
      r.setLocation((screenSize.width - paneSize.width) / 2,
                    (screenSize.height - paneSize.height) / 2);

      r.show();
   }

   private LoginDialog m_login;
   private MainConfigPanel m_main;

   private static final int PERMISSION_DENIED = 502;
   private static final int DIALOG_WIDTH = 60;
   private static final String TITLE = Constants.PRODUCT_NAME + " Control Panel";
}

