///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.apps.clnews.configure;

import java.awt.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import javax.swing.border.*;
import org.cooldevtools.awt.*;
import org.cooldevtools.utility.*;

public class OverviewTreePanel
extends JPanel
{
   public OverviewTreePanel()
   {
      m_listeners = new SuperVector(2);
      initDisplay();
   }

   public static class SelectionChangedListener
   {
      public void optionSelected(String name)
      {
      }

      public void groupSelected(String name)
      {
      }

      public void userSelected(String name)
      {
      }

      public void roleSelected(String name)
      {
      }

      public boolean canChangeSelection()
      {
         return true;
      }
   }

   private class Selector implements TreeSelectionListener
   {
      public void valueChanged(TreeSelectionEvent ev)
      {
         if (m_ignore) {
            m_ignore = false;
            return;
         }

         SelectionChangedListener[] listeners = new SelectionChangedListener[m_listeners.size()];
         m_listeners.copyInto(listeners);

         TreePath old_path = ev.getOldLeadSelectionPath();
         if (old_path != null) {
            for (int i = 0; i < listeners.length; i++) {
               if (!listeners[i].canChangeSelection()) {
                  getToolkit().beep();
                  m_ignore = true;
                  m_tree.setSelectionPath(old_path);
                  return;
               }
            }
         }

         TreePath path = ev.getPath();
         if (path.getPathCount() != 3) {
            return;
         }

         String selection = (String)(((DefaultMutableTreeNode)path.getPathComponent(2)).getUserObject());
         if (path.getPathComponent(1) == m_groups) {
            for (int i = 0; i < listeners.length; i++) {
               listeners[i].groupSelected(selection);
            }
         } else if (path.getPathComponent(1) == m_users) {
            for (int i = 0; i < listeners.length; i++) {
               listeners[i].userSelected(selection);
            }
         } else if (path.getPathComponent(1) == m_roles) {
            for (int i = 0; i < listeners.length; i++) {
               listeners[i].roleSelected(selection);
            }
         } else {
            for (int i = 0; i < listeners.length; i++) {
               listeners[i].optionSelected(selection);
            }
         }
      }

      boolean m_ignore;
   }

   public Dimension getPreferredSize()
   {
      Dimension d = super.getPreferredSize();
      if (getGraphics() != null) {
        d.width = getGraphics().getFontMetrics().stringWidth("XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX");
      }
      return d;
   }

   public Dimension getMinimumSize()
   {
      Dimension d = super.getMinimumSize();
      if (getGraphics() != null) {
        d.width = getGraphics().getFontMetrics().stringWidth("XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX");
      }
      return d;
   }

   private void selectChild(MutableTreeNode parent,
                            String name)
   {
      Enumeration en = parent.children();
      while (en.hasMoreElements()) {
         DefaultMutableTreeNode child = (DefaultMutableTreeNode)en.nextElement();
         String value = (String)child.getUserObject();
         if (value.equals(name)) {
            TreePath path = new TreePath(child.getPath());
            m_tree.setSelectionPath(path);
            m_tree.expandPath(path);
            return;
         }
      }
   }

   public void selectRole(String name)
   {
      selectChild(m_roles, name);
   }

   public void selectUser(String name)
   {
      selectChild(m_users, name);
   }

   public void selectGroup(String name)
   {
      selectChild(m_groups, name);
   }

   private void initDisplay()
   {
      m_root = new DefaultMutableTreeNode("", true);
      m_model = new DefaultTreeModel(m_root, true);

      m_tree = new JTree(m_model);
      m_tree.setRootVisible(false);
      m_tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
      m_tree.setShowsRootHandles(true);

      String[] empty = { };
      setChildren(makeNode(OPTIONS_LABEL, empty),
                  makeNode(GROUPS_LABEL, empty),
                  makeNode(USERS_LABEL, empty),
                  makeNode(ROLES_LABEL, empty));

      GridBagHelper gb = new GridBagHelper(this);
      gb.setFill(GridBagConstraints.BOTH);
      gb.setInsets(5, 5, 5, 5);
      gb.setWeights(1, 1);
      gb.setBounds(0, 0, 1, 1);
      gb.addComponent(new JScrollPane(m_tree));

      m_tree.addTreeSelectionListener(new Selector());
   }

   public void addSelectionChangedListener(SelectionChangedListener listener)
   {
      m_listeners.addElement(listener);
   }

   private DefaultMutableTreeNode makeNode(String name,
                                           String[] values)
   {
      DefaultMutableTreeNode root = new DefaultMutableTreeNode(name);
      root.setAllowsChildren(true);

      for (int i = 0; i < values.length; i++) {
         DefaultMutableTreeNode child = new DefaultMutableTreeNode(values[i]);
         child.setAllowsChildren(false);
         root.insert(child, i);
      }

      return root;
   }

   private void setChildren(DefaultMutableTreeNode options,
                            DefaultMutableTreeNode groups,
                            DefaultMutableTreeNode roles,
                            DefaultMutableTreeNode users)
   {
      m_root.removeAllChildren();
      m_root.insert(options, OPTIONS_BRANCH);
      m_root.insert(groups, GROUPS_BRANCH);
      m_root.insert(users, USERS_BRANCH);
      m_root.insert(roles, ROLES_BRANCH);
      m_model.reload();

      m_options = options;
      m_users = users;
      m_groups = groups;
      m_roles = roles;
   }

   public void reset(String[] options,
                     String[] roles,
                     String[] users,
                     String[] groups)
   {
      boolean expand_roles = m_tree.isExpanded(new TreePath(m_roles.getPath()));
      boolean expand_users = m_tree.isExpanded(new TreePath(m_users.getPath()));
      boolean expand_groups = m_tree.isExpanded(new TreePath(m_groups.getPath()));
      DefaultMutableTreeNode options_node  = makeNode(OPTIONS_LABEL, options);
      DefaultMutableTreeNode groups_node = makeNode(GROUPS_LABEL, groups);
      DefaultMutableTreeNode roles_node  = makeNode(ROLES_LABEL, roles);
      DefaultMutableTreeNode users_node  = makeNode(USERS_LABEL, users);
      setChildren(options_node, groups_node, roles_node, users_node);
      if (expand_roles) {
         m_tree.expandPath(new TreePath(m_roles.getPath()));
      }
      if (expand_users) {
         m_tree.expandPath(new TreePath(m_users.getPath()));
      }
      if (expand_groups) {
         m_tree.expandPath(new TreePath(m_groups.getPath()));
      }
   }

   public void setNewsGroups(String[] names)
   {
      DefaultMutableTreeNode node = makeNode(GROUPS_LABEL, names);
      setChildren(m_options, node, m_roles, m_users);
   }

   public void setUsers(String[] names)
   {
      DefaultMutableTreeNode node = makeNode(USERS_LABEL, names);
      setChildren(m_options, m_groups, m_roles, node);
   }

   public static void main(String[] args)
   {
      OverviewTreePanel t = new OverviewTreePanel();

      JFrame f = new JFrame();
      f.getContentPane().setLayout(new BorderLayout());
      f.getContentPane().add(t, "Center");
      f.pack();

      String[] users  = { "admin", "brian", "default" };
      String[] groups = { "clnews.misc", "mygroup", "coffee_server.discuss" };

      t.setNewsGroups(groups);
      t.setUsers(users);

      f.show();
   }

   private JTree m_tree;
   private DefaultMutableTreeNode m_root;
   private DefaultMutableTreeNode m_options;
   private DefaultMutableTreeNode m_groups;
   private DefaultMutableTreeNode m_users;
   private DefaultMutableTreeNode m_roles;
   private DefaultTreeModel m_model;
   private SuperVector m_listeners;

   private static final int OPTIONS_BRANCH = 0;
   private static final int GROUPS_BRANCH  = 1;
   private static final int USERS_BRANCH   = 2;
   private static final int ROLES_BRANCH   = 3;

   private static final String OPTIONS_LABEL = "Options";
   private static final String GROUPS_LABEL  = "Groups";
   private static final String ROLES_LABEL   = "Roles";
   private static final String USERS_LABEL   = "Users";
}

