///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.apps.clnews.configure;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import org.cooldevtools.awt.*;
import org.cooldevtools.utility.*;
import org.cooldevtools.coffee_server.services.nntp.protocol.ServerConnection;

public class RoleDetailPanel
extends JPanel
{
   public RoleDetailPanel()
   {
      initDisplay(new ModifiedEventHelper());
   }

   public RoleDetailPanel(ModifiedEventHelper modified)
   {
      initDisplay(modified);
   }

   private JTextField newTextField(ModifiedEventHelper modified)
   {
      JTextField field = new JTextField(30);
      field.addFocusListener(AutoSelectAll.s());
      field.getDocument().addDocumentListener(modified);
      return field;
   }

   private JComboBox newComboBox(ModifiedEventHelper modified,
				 String[] values)
   {
      JComboBox box = new JComboBox(values);
      box.addActionListener(modified);
      return box;
   }

   private void initDisplay(ModifiedEventHelper modified)
   {
      setBorder(Constants.PANEL_BORDER);
      setLayout(new GapFlowLayout(10, true));

      JPanel p = new JPanel();
      GridBagHelper gb = new GridBagHelper(p);

      p.setBorder(new TitledBorder("Role Information"));
      gb.setFill(GridBagConstraints.NONE);
      gb.setAnchor(GridBagConstraints.EAST);
      gb.setWeights(0, 0);
      gb.setBounds(0, 0, 1, 1);
      gb.addComponent(new JLabel("Role Name:"));
      gb.setBounds(0, 1, 1, 1);
      gb.addComponent(new JLabel("Admin Level:"));

      gb.setFill(GridBagConstraints.HORIZONTAL);
      gb.setAnchor(GridBagConstraints.WEST);
      gb.setWeights(1, 0);
      gb.setBounds(1, 0, 1, 1);
      gb.addComponent(m_name = newTextField(modified));
      gb.setBounds(1, 1, 1, 1);
      gb.addComponent(m_admin = newComboBox(modified, ADMIN_LEVEL_DESCRIPTIONS));
      add(p);

      p = new JPanel();
      gb = new GridBagHelper(p);
      p.setBorder(new TitledBorder("Group Authority"));
      gb.setAnchor(GridBagConstraints.CENTER);
      gb.setFill(GridBagConstraints.HORIZONTAL);
      gb.setWeights(1, 0);
      gb.setBounds(0, 0, 1, 1);
      gb.addComponent(new JLabel("Name"));
      gb.setBounds(1, 0, 1, 1);
      gb.addComponent(new JLabel("Access Level"));

      gb.setAnchor(GridBagConstraints.WEST);
      m_groups = new JTextField[NUM_GROUPS];
      m_access = new JComboBox[NUM_GROUPS];
      for (int i = 0; i < NUM_GROUPS; ++i) {
	 m_groups[i] = newTextField(modified);
	 m_access[i] = newComboBox(modified, ACCESS_LEVEL_DESCRIPTIONS);
	 gb.setWeights(2, 0);
	 gb.setBounds(0, i + 2, 1, 1);
	 gb.addComponent(m_groups[i]);
	 gb.setWeights(1, 0);
	 gb.setBounds(1, i + 2, 1, 1);
	 gb.addComponent(m_access[i]);
      }
      add(p);

      p = new JPanel();
      gb = new GridBagHelper(p);
      p.setBorder(new TitledBorder("Role Membership Authority"));
      gb.setAnchor(GridBagConstraints.CENTER);
      gb.setFill(GridBagConstraints.HORIZONTAL);
      gb.setWeights(1, 0);
      gb.setBounds(0, 0, 1, 1);
      gb.addComponent(new JLabel("Role Name"));
      gb.setAnchor(GridBagConstraints.WEST);
      m_roles = new JTextField[NUM_ROLES];
      for (int i = 0; i < NUM_ROLES; ++i) {
	 m_roles[i] = newTextField(modified);
	 gb.setBounds(0, i + 2, 1, 1);
	 gb.addComponent(m_roles[i]);
      }
      add(p);
   }

   private void setComboValue(JComboBox box,
			      String value,
			      String[] table,
			      int default_item)
   {
      int item = default_item;
      for (int i = 0; i < table.length; ++i) {
	 if (table[i].equals(value)) {
	    item = i;
	    break;
	 }
      }
      box.setSelectedIndex(item);
   }

   public void clear()
   {
      m_name.setText("");
      m_admin.setSelectedIndex(0);
      for (int i = 0; i < NUM_GROUPS; ++i) {
	 m_groups[i].setText("");
	 m_access[i].setSelectedIndex(0);
      }
      for (int i = 0; i < NUM_ROLES; ++i) {
	 m_roles[i].setText("");
      }
   }

   private int countValues(JTextField[] array)
   {
      int count = 0;
      for (int i = 0; i < array.length; ++i) {
	 if (array[i].getText().length() > 0) {
	    ++count;
	 }
      }
      return count;
   }

   public void loadFromServer(ServerConnection server,
			      String role_name)
      throws IOException
   {
      clear();
      if (role_name == null) {
	 return;
      }

      ServerConnection.RoleInfo role = server.getRole(role_name);
      if (role == null) {
	 return;
      }

      m_name.setText(role.name);
      setComboValue(m_admin, role.admin_level, ADMIN_LEVEL_CODES, 0);
      for (int i = 0; i < role.groups.length && i < NUM_GROUPS; ++i) {
	 m_groups[i].setText(role.groups[i]);
	 setComboValue(m_access[i], role.group_perms[i], ACCESS_LEVEL_CODES, 0);
      }

      for (int i = 0; i < role.user_roles.length && i < NUM_ROLES; ++i) {
	 m_roles[i].setText(role.user_roles[i]);
      }
   }

   public String getName()
   {
      return m_name.getText();
   }

   public void sendToServer(ServerConnection server)
      throws IOException
   {
      ServerConnection.RoleInfo role = new ServerConnection.RoleInfo();

      role.name = m_name.getText();
      role.admin_level = ADMIN_LEVEL_CODES[m_admin.getSelectedIndex()];

      role.groups = new String[countValues(m_groups)];
      role.group_perms = new String[role.groups.length];
      int index = 0;
      for (int i = 0; i < NUM_GROUPS; ++i) {
	 String value = m_groups[i].getText();
	 if (value.length() > 0) {
	    role.groups[index] = value;
	    role.group_perms[index] = ACCESS_LEVEL_CODES[m_access[i].getSelectedIndex()];
	    ++index;
	 }
      }

      role.user_roles = new String[countValues(m_roles)];
      index = 0;
      for (int i = 0; i < NUM_ROLES; ++i) {
	 String value = m_roles[i].getText();
	 if (value.length() > 0) {
	    role.user_roles[index] = value;
	    ++index;
	 }
      }

      server.defineRole(role);
      server.saveConfig();
   }

   public static void main(String[] args)
   {
      JFrame f = new JFrame();
      f.getContentPane().add(new RoleDetailPanel());
      f.pack();
      f.show();
   }

   private JTextField m_name;
   private JComboBox m_admin;
   private JTextField[] m_groups;
   private JComboBox[] m_access;
   private JTextField[] m_roles;

   private static final int NUM_GROUPS = 4;
   private static final int NUM_ROLES = 4;

   private static final String[] ADMIN_LEVEL_DESCRIPTIONS = {
      "No administrative authority",
      "Administer users only",
      "Administer users and roles"
   };
   private static final String[] ADMIN_LEVEL_CODES = {
      "X",
      "U",
      "R"
   };
   private static final String[] ACCESS_LEVEL_DESCRIPTIONS = {
      "No access allowed",
      "Read only",
      "Read, post",
      "Read, post, cancel",
      "Read, post, cancel, modify",
   };
   private static final String[] ACCESS_LEVEL_CODES = {
      "X",
      "R",
      "P",
      "C",
      "A",
   };
}

