///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.apps.clnews.configure;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.text.*;
import org.cooldevtools.utility.*;
import org.cooldevtools.awt.*;
import org.cooldevtools.coffee_server.services.nntp.protocol.ServerConnection;

public class ServerGeneralDetailPanel
extends JPanel
implements ControlPanelInterface
{
   public ServerGeneralDetailPanel(ModifiedEventHelper modified)
   {
      initDisplay(modified);
   }

   private void initDisplay(ModifiedEventHelper modified)
   {
      setLayout(new GapFlowLayout(6, true));
      setBorder(Constants.PANEL_BORDER);

      m_fields = new BasicFieldPanel();
      m_fields.addField(HOST_NAME,        "Host/IP Address: ", "", 20);
      m_fields.addField(PRIMARY_PORT,     "Preferred Port: ", "", 8);
      m_fields.addField(ALTERNATE_PORT,   "Alternate Port: ", "", 8);
      m_fields.addField(MAX_MESSAGE_SIZE, "Max Message Size (kb): ", "", 8);
      m_fields.addField(MAX_MESSAGE_DAYS, "Max Message Age (days): ", "", 8);
      m_fields.addField(MAX_CONNECTIONS,  "Max Connections: ", "", 8);
      m_fields.getField(PRIMARY_PORT).setDocument(new BasicIntegerDocument(false));
      m_fields.getField(ALTERNATE_PORT).setDocument(new BasicIntegerDocument(false));
      m_fields.getField(MAX_MESSAGE_SIZE).setDocument(new BasicIntegerDocument(false));
      m_fields.getField(MAX_MESSAGE_DAYS).setDocument(new BasicIntegerDocument(false));
      m_fields.getField(MAX_CONNECTIONS).setDocument(new BasicIntegerDocument(false));
      m_fields.addDocumentListener(modified);

      m_fields.setMnemonic(HOST_NAME,        'H');
      m_fields.setMnemonic(PRIMARY_PORT,     'P');
      m_fields.setMnemonic(ALTERNATE_PORT,   'A');
      m_fields.setMnemonic(MAX_MESSAGE_SIZE, 'M');
      m_fields.setMnemonic(MAX_MESSAGE_DAYS, 'D');
      m_fields.setMnemonic(MAX_CONNECTIONS,  'C');

      add(m_fields);
   }

   public void loadFromServer(ServerConnection server)
      throws IOException
   {
      ServerConnection.AddressInfo addr = server.getAddressInfo();
      m_fields.setFieldValue(HOST_NAME, addr.getAddress());
      m_fields.setIntFieldValue(PRIMARY_PORT, addr.getPrimaryPort());
      m_fields.setIntFieldValue(ALTERNATE_PORT, addr.getAlternatePort());
      m_fields.setIntFieldValue(MAX_MESSAGE_SIZE, server.getMaxMessageSize());
      m_fields.setIntFieldValue(MAX_MESSAGE_DAYS, server.getMaxMessageDays());
      m_fields.setIntFieldValue(MAX_CONNECTIONS, server.getMaxConnections());
   }

   public void sendToServer(ServerConnection server)
      throws IOException
   {
      ServerConnection.AddressInfo
	 addr = new ServerConnection.AddressInfo(m_fields.getFieldValue(HOST_NAME),
						 m_fields.getIntFieldValue(PRIMARY_PORT),
						 m_fields.getIntFieldValue(ALTERNATE_PORT));
      server.setAddressInfo(addr);
      server.setMaxMessageSize(m_fields.getIntFieldValue(MAX_MESSAGE_SIZE));
      server.setMaxMessageDays(m_fields.getIntFieldValue(MAX_MESSAGE_DAYS));
      server.setMaxConnections(m_fields.getIntFieldValue(MAX_CONNECTIONS));
      server.saveConfig();
   }

   private BasicFieldPanel m_fields;

   public static final int HOST_NAME        = 0;
   public static final int PRIMARY_PORT     = 1;
   public static final int ALTERNATE_PORT   = 2;
   public static final int MAX_MESSAGE_SIZE = 3;
   public static final int MAX_MESSAGE_DAYS = 4;
   public static final int MAX_CONNECTIONS  = 5;
}

