///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.apps.clnews.configure;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.text.*;
import org.cooldevtools.utility.*;
import org.cooldevtools.awt.*;
import org.cooldevtools.coffee_server.services.nntp.protocol.ServerConnection;

public class ServerLoggingDetailPanel
extends JPanel
implements ControlPanelInterface
{
   public ServerLoggingDetailPanel(ModifiedEventHelper modified)
   {
      initDisplay(modified);
   }

   private void initDisplay(ModifiedEventHelper modified)
   {
      m_logToConsole = new JRadioButton("Log To Java Console");
      m_logToFile    = new JRadioButton("Log To File");
      m_logToSyslog  = new JRadioButton("Log To Unix Syslog Server");

      m_logToConsole.addActionListener(modified);
      m_logToFile.addActionListener(modified);
      m_logToSyslog.addActionListener(modified);

      m_logToConsole.setMnemonic('C');
      m_logToFile.setMnemonic('F');
      m_logToSyslog.setMnemonic('S');

      ButtonGroup group = new ButtonGroup();
      group.add(m_logToConsole);
      group.add(m_logToFile);
      group.add(m_logToSyslog);

      m_logLevel = new JComboBox(LOG_LEVEL_DESCRIPTIONS);
      m_logLevel.setEditable(false);
      m_logLevel.addActionListener(modified);

      setBorder(Constants.PANEL_BORDER);
      setLayout(new GapFlowLayout(10, true));

      JPanel p = new JPanel();
      p.setLayout(new GapFlowLayout(5, true));
      p.add(m_logToConsole);

      p.add(m_logToFile);
      BasicFieldPanel fields = new BasicFieldPanel();
      fields.addField(0, "File Name: ", "", 20);
      m_filename = fields.getField(0);
      m_filename.addFocusListener(AutoSelectAll.s());
      fields.addDocumentListener(modified);
      p.add(fields);

      p.add(m_logToSyslog);
      fields = new BasicFieldPanel();
      fields.addField(0, "Host Name: ", "", 20);
      m_hostname = fields.getField(0);
      m_hostname.addFocusListener(AutoSelectAll.s());
      m_hostname.setDocument(new RejectChars(RejectChars.HOST_CHARS));
      fields.addDocumentListener(modified);
      p.add(fields);
      add(p);

      p = new JPanel();
      p.setBorder(new TitledBorder("Level Of Detail"));
      p.add(m_logLevel);

      add(new Gap());
      add(p);
      add(new Gap());
   }

   public void loadFromServer(ServerConnection server)
   throws IOException
   {
      String[] log_param = new String[1];
      int log_type = server.getLogging(log_param);
      if (log_type == ServerConnection.LOG_TO_SYSLOG) {
         m_logToSyslog.setSelected(true);
         m_filename.setText("");
         m_hostname.setText(log_param[0]);
      } else if (log_type == ServerConnection.LOG_TO_FILE && !log_param[0].equals("-")) {
         m_logToFile.setSelected(true);
         m_filename.setText(log_param[0]);
         m_hostname.setText("");
      } else {
         m_logToConsole.setSelected(true);
         m_filename.setText("");
         m_hostname.setText("");
      }

      String log_level_code = server.getLogLevel();
      int log_level_index = logLevelCodeToIndex(log_level_code);
      m_logLevel.setSelectedIndex(log_level_index);
   }

   public void sendToServer(ServerConnection server)
   throws IOException
   {
      if (m_logToSyslog.isSelected()) {
         server.setLogging(ServerConnection.LOG_TO_SYSLOG, m_hostname.getText());
         m_filename.setText("");
      } else if (m_logToFile.isSelected()) {
         server.setLogging(ServerConnection.LOG_TO_FILE, m_filename.getText());
         m_hostname.setText("");
      } else {
         m_logToConsole.setSelected(true);
         server.setLogging(ServerConnection.LOG_TO_FILE, "-");
         m_filename.setText("");
         m_hostname.setText("");
      }
      server.setLogLevel(LOG_LEVEL_CODES[m_logLevel.getSelectedIndex()]);
      server.saveConfig();
   }

   private int logLevelCodeToIndex(String code)
   {
      return Misc.findStringInArray(code, LOG_LEVEL_CODES, DEFAULT_LOG_LEVEL);
   }

   private JRadioButton m_logToConsole;
   private JRadioButton m_logToFile;
   private JRadioButton m_logToSyslog;
   private JTextField   m_filename;
   private JTextField   m_hostname;
   private JComboBox    m_logLevel;

   private static final String[] LOG_LEVEL_DESCRIPTIONS = {
      "Verbose: Maximum amount of information.",
      "Normal: Notices, warnings and errors.",
      "Quiet: Only warnings and errors."
   };
   private static final String[] LOG_LEVEL_CODES = {
      "verbose",
      "normal",
      "quiet"
   };
   private static final int DEFAULT_LOG_LEVEL = 1;
}
