///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.apps.clnews.configure;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.text.*;
import org.cooldevtools.utility.*;
import org.cooldevtools.awt.*;
import org.cooldevtools.coffee_server.services.nntp.protocol.ServerConnection;

public class ServerUnixDetailPanel
extends JPanel
implements ControlPanelInterface
{
   public ServerUnixDetailPanel(ModifiedEventHelper modified)
   {
      initDisplay(modified);
   }

   private void initDisplay(ModifiedEventHelper modified)
   {
      m_setuidButton = new JCheckBox("Change Identity On Startup");
      m_setuidButton.addActionListener(modified);
      m_setuidButton.setMnemonic('C');

      m_fields = new BasicFieldPanel();
      m_fields.addField(USER_NAME,  "Unix User ID: ", "", 20);
      m_fields.addField(GROUP_NAME, "Unix Group ID: ", "", 20);
      m_fields.addDocumentListener(modified);

      setBorder(Constants.PANEL_BORDER);
      setLayout(new GapFlowLayout(5, true));
      add(m_setuidButton);
      add(m_fields);
   }

   public void loadFromServer(ServerConnection server)
   throws IOException
   {
      String[] ids = server.getUnixIdentity();

      m_setuidButton.setSelected(ids[0].length() > 0);
      m_fields.setFieldValue(USER_NAME, ids[0]);
      m_fields.setFieldValue(GROUP_NAME, ids[1]);
   }

   public void sendToServer(ServerConnection server)
   throws IOException
   {
      String user_id = m_fields.getFieldValue(USER_NAME);
      String group_id = m_fields.getFieldValue(GROUP_NAME);
      boolean both_filled = (user_id.length() > 0) && (group_id.length() > 0);
      if (both_filled && m_setuidButton.isSelected()) {
         server.setUnixIdentity(user_id, group_id);
      } else {
         server.setUnixIdentity("", "");
         m_setuidButton.setSelected(false);
         m_fields.setFieldValue(USER_NAME, "");
         m_fields.setFieldValue(GROUP_NAME, "");
      }
      server.saveConfig();
   }

   private JCheckBox m_setuidButton;
   private BasicFieldPanel m_fields;

   public static final int USER_NAME  = 0;
   public static final int GROUP_NAME = 1;
}

