///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.apps.clnews.configure;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import org.cooldevtools.utility.*;
import org.cooldevtools.awt.*;

public class UserDetailPanel
extends JPanel
{
   public UserDetailPanel()
   {
      initDisplay(new ModifiedEventHelper());
   }

   public UserDetailPanel(ModifiedEventHelper modified)
   {
      initDisplay(modified);
   }

   private void initDisplay(ModifiedEventHelper modified)
   {
      setBorder(Constants.PANEL_BORDER);

      m_fields = new BasicFieldPanel();
      m_fields.addField(USERNAME, "User Name: ", "", 20);
      m_fields.addField(PASSWORD, "Password: ", "", 20);
      m_fields.addField(EMAIL, "Email Address: ", "", 20);
      m_fields.getField(USERNAME).setDocument(new RejectChars(RejectChars.NAME_CHARS));
      m_fields.addDocumentListener(modified);

      m_fields.setMnemonic(USERNAME, 'U');
      m_fields.setMnemonic(PASSWORD, 'P');
      m_fields.setMnemonic(EMAIL,    'E');

      JLabel roles_label = new JLabel("Roles:");
      m_roles = new JList();
      m_roles.addListSelectionListener(modified);
      roles_label.setLabelFor(m_roles);
      roles_label.setDisplayedMnemonic('R');

      GridBagHelper gb = new GridBagHelper(this);
      gb.setWeights(1, 0);
      gb.setInsets(5, 5, 5, 5);
      gb.setAnchor(GridBagConstraints.NORTHWEST);
      gb.setBounds(0, 0, 1, 2);
      gb.setFill(GridBagConstraints.HORIZONTAL);
      gb.addComponent(m_fields);
      gb.setBounds(0, 2, 1, 1);
      gb.setFill(GridBagConstraints.NONE);
      gb.addComponent(new JLabel(""));
      gb.setBounds(0, 3, 1, 1);
      gb.setFill(GridBagConstraints.NONE);
      gb.addComponent(roles_label);
      gb.setBounds(0, 4, 1, 4);
      gb.setWeights(1, 1);
      gb.setFill(GridBagConstraints.BOTH);
      gb.addComponent(new JScrollPane(m_roles));
   }

   public static void main(String[] args)
   {
      JFrame f = new JFrame();
      UserDetailPanel r = new UserDetailPanel(null);

      r.setUsername("brian");
      r.setPassword("secret");
      r.setEmail("brian@nowhere.com");

      String[] all_roles = { "admin", "default", "bozo" };
      String[] user_roles = { "admin", "bozo" };
      r.setRoles(all_roles, user_roles);

      f.getContentPane().add(r);
      f.pack();
      f.show();
   }

   public void setUsername(String value)
   {
      m_fields.getField(USERNAME).setText(value);
   }

   public void setPassword(String value)
   {
      m_fields.getField(PASSWORD).setText(value);
   }

   public void setEmail(String value)
   {
      m_fields.getField(EMAIL).setText(value);
   }

   public void setRoles(String[] all_roles,
                        String[] user_roles)
   {
      SuperVector vec = new SuperVector(user_roles.length);

      m_roles.setListData(all_roles);
      for (int i = 0; i < user_roles.length; i++) {
         for (int k = 0; k < all_roles.length; k++) {
            if (all_roles[k].equals(user_roles[i])) {
               vec.addElement(new Integer(k));
               break;
            }
         }
      }

      int[] selections = new int[vec.size()];
      Integer[] values = vec.getIntegers();
      for (int i = 0; i < values.length; i++) {
         selections[i] = values[i].intValue();
      }
      m_roles.setSelectedIndices(selections);
   }

   public String getUsername()
   {
      return m_fields.getField(USERNAME).getText();
   }

   public String getPassword()
   {
      return m_fields.getField(PASSWORD).getText().trim();
   }

   public String getEmail()
   {
      return m_fields.getField(EMAIL).getText().trim();
   }

   public String[] getRoles()
   {
      Object[] values = m_roles.getSelectedValues();
      String[] answer = new String[values.length];
      System.arraycopy(values, 0, answer, 0, values.length);
      return answer;
   }

   private BasicFieldPanel m_fields;
   private JList m_roles;

   private static final int USERNAME = 0;
   private static final int PASSWORD = 1;
   private static final int EMAIL    = 2;
}

