///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.apps.clnews.repository;

import java.io.*;
import org.cooldevtools.utility.*;
import org.cooldevtools.utility.storage.*;
import org.cooldevtools.coffee_server.framework.*;
import org.cooldevtools.coffee_server.services.nntp.pub.*;
import org.cooldevtools.coffee_server.services.nntp.protocol.WildMat;

public class DBRoles
{
   public DBRoles(String directory,
		  FileArchive archive,
		  Schema schema,
		  CommitManager commit_manager)
   throws IOException, StorageException
   {
      m_db = new SimpleDatabase(schema, directory, DB_NAME);
      commit_manager.addCommitable(m_db);
      m_archive  = archive;
   }

   public boolean hasGroupPerm(String role_name,
                               String[] perms)
   throws CoffeeServiceException
   {
      try {
         try {
            Record key = m_db.getBlank();
            key.setString(COL_Name, role_name);

            Record r = m_db.find(NDX_ByName, key);

            String[] tmp = Misc.splitString(r.getBlobAsString(COL_GroupPerms), SEP_OUTER);
            for (int i = 0; i < tmp.length; i++) {
               String[] entry = Misc.splitString(tmp[i], SEP_INNER);
               if (Misc.stringInArray(entry[1], perms)) {
                  return true;
               }
            }
         } catch (UnknownKeyException ex) {
            
         }

         return false;
      } catch (IOException ex) {
         throw new CoffeeServiceException(ex);
      }
   }

   public String getGroupPerm(String role_name,
                              String group_name)
   throws CoffeeServiceException
   {
      try {
         try {
            Record key = m_db.getBlank();
            key.setString(COL_Name, role_name);

            Record r = m_db.find(NDX_ByName, key);

            String[] tmp = Misc.splitString(r.getBlobAsString(COL_GroupPerms), SEP_OUTER);
            for (int i = 0; i < tmp.length; i++) {
               String[] entry = Misc.splitString(tmp[i], SEP_INNER);
               WildMat wild = new WildMat(entry[0]);
               if (wild.match(group_name)) {
                  return entry[1];
               }
            }
         } catch (UnknownKeyException ex) {
            
         }

         return RoleRepository.PERM_NONE;
      } catch (IOException ex) {
         throw new CoffeeServiceException(ex);
      }
   }

   /**
      Returns true iff this role can use ALL of the named roles.
    */
   public boolean canUseRoles(String role,
                              String[] role_names)
   throws CoffeeServiceException
   {
      if (Assert.DEBUG) System.out.println("canUseRoles: " + Misc.joinStrings(role_names, '\n'));
      try {
         try {
            Record key = m_db.getBlank();
            key.setString(COL_Name, role);

            Record r = m_db.find(NDX_ByName, key);

            String[] tmp = Misc.splitString(r.getBlobAsString(COL_RolePerms), SEP_OUTER);
            if (tmp.length == 0) {
               if (Assert.DEBUG) System.out.println("canUseRoles = false");
               return false;
            }

            for (int i = 0; i < tmp.length; i++) {
               WildMat wild = new WildMat(tmp[i]);
               if (Assert.DEBUG) System.out.println("canUseRoles tesing wild = " + wild.getExpression());
               for (int k = 0; k < role_names.length; k++) {
                  if (!wild.match(role_names[k])) {
                     if (Assert.DEBUG) System.out.println("canUseRoles = false");
                     return false;
                  }
               }
            }
         } catch (UnknownKeyException ex) {
            
         }

         if (Assert.DEBUG) System.out.println("canUseRoles = true");
         return true;
      } catch (IOException ex) {
         throw new CoffeeServiceException(ex);
      }
   }

   public String getAdminPerm(String role_name)
   throws CoffeeServiceException
   {
      try {
         try {
            Record key = m_db.getBlank();
            key.setString(COL_Name, role_name);

            Record r = m_db.find(NDX_ByName, key);
            return r.getString(COL_Admin);
         } catch (UnknownKeyException ex) {
            return null;
         }
      } catch (IOException ex) {
         throw new CoffeeServiceException(ex);
      }
   }

   public void deleteRole(String role_name)
   throws CoffeeServiceException
   {
      try {
         try {
            Record key = m_db.getBlank();
            key.setString(COL_Name, role_name);

            Record r = m_db.find(NDX_ByName, key);
            m_db.remove(r);
         } catch (UnknownKeyException ex) {
            // ignored
            
         }
      } catch (IOException ex) {
         throw new CoffeeServiceException(ex);
      }
   }

   public String[] getRoleNames()
   throws CoffeeServiceException
   {
      try {
         DatabaseNavigator nav = m_db.navigate(NDX_ByName, "");
         try {
            SuperVector v = new SuperVector(10000,0);

            while (nav.isValid()) {
               String key = nav.getKey();
               v.addElement(key.substring(0, key.length() - 1));
               nav.next();
            }

            return v.getStrings();
         } finally {
            nav.close();
         }
      } catch (IOException ex) {
         throw new CoffeeServiceException(ex);
      }
   }

   public String[] getRoleDefinition(String role_name)
   throws CoffeeServiceException
   {
      try {
         try {
            Record key = m_db.getBlank();
            key.setString(COL_Name, role_name);

            Record r = m_db.find(NDX_ByName, key);

            SuperVector v = new SuperVector(500,0);
            v.addElement("ADMIN " + r.getString(COL_Admin));

            String[] tmp = Misc.splitString(r.getBlobAsString(COL_GroupPerms), SEP_OUTER);
            for (int i = 0; i < tmp.length; i++) {
               String[] tmp2 = Misc.splitString(tmp[i], SEP_INNER);
               v.addElement("GROUP " + tmp2[0] + " " + tmp2[1]);
            }

            tmp = Misc.splitString(r.getBlobAsString(COL_RolePerms), SEP_OUTER);
            for (int i = 0; i < tmp.length; i++) {
               v.addElement("ROLE " + tmp[i]);
            }

            return v.getStrings();
         } catch (UnknownKeyException ex) {
            return null;
         }
      } catch (IOException ex) {
         throw new CoffeeServiceException(ex);
      }
   }

   public void defineRole(String role_name,
                          String admin_perm,
                          String[] role_patterns,
                          String[] group_patterns,
                          String[] group_perms)
   throws CoffeeServiceException
   {
      try {
         String[] tmp = new String[group_patterns.length];
         for (int i = 0; i < tmp.length; i++) {
            tmp[i] = Misc.joinStrings(group_patterns[i], group_perms[i], SEP_INNER);
         }
         String groups_str = Misc.joinStrings(tmp, SEP_OUTER);

         Record r = m_db.getBlank();
         r.setString(COL_Name, role_name);
         r.setLong(COL_DBKey, m_archive.nextInSequence(SEQ_NAME));
         r.setString(COL_Admin, admin_perm);
         r.setBlob(COL_GroupPerms, groups_str);
         r.setBlob(COL_RolePerms, Misc.joinStrings(role_patterns, SEP_OUTER));

         deleteRole(role_name);
         m_db.insert(r);
      } catch (IOException ex) {
         throw new CoffeeServiceException(ex);
      }
   }

   public void sync()
   throws IOException, StorageException
   {
      m_db.sync();
   }

   public void abort()
   {
      m_db.abort();
   }

   private static char SEP_INNER = (char)0;
   private static char SEP_OUTER = (char)1;
   private static char SPACE     = (char)' ';

   private FileArchive m_archive;
   private Database    m_db;

   private static final String COL_Name        = "Name";
   private static final String COL_DBKey       = "DBKey";
   private static final String COL_Admin       = "Admin";
   private static final String COL_GroupPerms  = "GroupPerms";
   private static final String COL_RolePerms   = "RolePerms";

   private static final String NDX_ByName      = "ByName";
   private static final String NDX_ByKey       = "ByKey";

   private static final String DB_NAME         = "roles";
   private static final String SEQ_NAME        = DB_NAME;
}
