///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.apps.clnews.repository;

import java.util.*;
import java.io.*;
import org.cooldevtools.coffee_server.framework.*;
import org.cooldevtools.coffee_server.services.nntp.pub.*;
import org.cooldevtools.utility.*;
import org.cooldevtools.utility.storage.*;

public class FileMessage
extends Message {
   FileMessage(FileArchive archive,
               long number)
   {
      super(number);
      m_archive = archive;
   }

   public String[] getBody()
   throws CoffeeServiceException
   {
      m_archive.acquireReadLock();
      try {
         String[] answer = super.getBody();
         if (answer != null) {
            return answer;
         }

         setBody(m_archive.getDBMessages().loadMessageBody(getNumber()));
         return super.getBody();
      } finally {
         m_archive.releaseReadLock();
      }
   }

   public String[] getHeader()
   throws CoffeeServiceException
   {
      m_archive.acquireReadLock();
      try {
         String[] answer = super.getHeader();
         if (answer != null) {
            return answer;
         }

         setHeader(m_archive.getDBMessages().loadMessageHeader(getNumber()));
         return super.getHeader();
      } finally {
         m_archive.releaseReadLock();
      }
   }

   public NewsGroup[] getNewsGroups()
   throws CoffeeServiceException
   {
      m_archive.acquireReadLock();
      try {
         Long[] keys = m_archive.getDBPosts().getNewsGroupsForMessage(this);
         NewsGroup[] groups = new NewsGroup[keys.length];
         for (int i = 0; i < keys.length; i++) {
            groups[i] = m_archive.getDBGroups().loadGroup(keys[i].longValue());
         }
         return groups;
      } finally {
         m_archive.releaseReadLock();
      }
   }

   public static FileMessage[] toArray(Vector v)
   {
      synchronized (v) {
         FileMessage[] answer = new FileMessage[v.size()];
         v.copyInto(answer);
         return answer;
      }
   }

   private FileArchive m_archive;
}
