///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.apps.clnews.repository;

import java.io.*;
import java.util.*;
import org.cooldevtools.utility.storage.*;
import org.cooldevtools.coffee_server.framework.*;
import org.cooldevtools.coffee_server.services.nntp.pub.*;

public class FileMessageRepository
implements MessageRepository
{
   FileMessageRepository(FileArchive archive)
   {
      m_archive  = archive;
   }

   /**
      Returns an incomplete message object.
    */
   public Message createMessage()
   throws CoffeeServiceException
   {
      boolean commit = false;
      m_archive.acquireWriteLock();
      try {
         Message m = new FileMessage(m_archive, m_archive.nextInSequence(SEQ_NAME));
         commit = true;
         return m;
      } finally {
         m_archive.releaseWriteLock(commit);
      }
   }

   public void storeMessage(User user,
                            Message m)
   throws CoffeeServiceException
   {
      boolean commit = false;
      m_archive.acquireWriteLock();
      try {
         if (m_archive.getDBMessages().messageExists(m.messageId())) {
            throw new CoffeeServiceException("Duplicate Message Ignored");
         }
         m_archive.getDBMessages().insertMessage((FileMessage)m);
         commit = true;
      } finally {
         m_archive.releaseWriteLock(commit);
      }
   }

   public boolean messageExists(String name)
   throws CoffeeServiceException
   {
      m_archive.acquireReadLock();
      try {
         return m_archive.getDBMessages().messageExists(name);
      } finally {
         m_archive.releaseReadLock();
      }
   }

   /**
      Public front end to loadMessage() that guarantees synchronization.
    */
   public Message findMessage(String name)
   throws CoffeeServiceException
   {
      m_archive.acquireReadLock();
      try {
         return m_archive.getDBMessages().loadMessage(name);
      } finally {
         m_archive.releaseReadLock();
      }
   }

   public void removeOneMessage(FileMessage m)
      throws CoffeeServiceException
   {
      Long[] group_keys = m_archive.getDBPosts().cancelMessage(m);
      m_archive.getDBMessages().removeMessage(m);
      for (int i = 0; i < group_keys.length; i++) {
	 FileNewsGroup group = m_archive.getDBGroups().loadGroup(group_keys[i].longValue());
	 group.messageRemoved();
      }
   }

   /**
      Remove the message from the cache and from the database.
    */
   public void removeMessage(User user,
                             Message m)
   throws CoffeeServiceException
   {
      if (m == null) {
         return;
      }

      boolean commit = false;
      m_archive.acquireWriteLock();
      try {
	 removeOneMessage((FileMessage)m);
         commit = true;
      } finally {
         m_archive.releaseWriteLock(commit);
      }
   }

   /**
      Return messages newer than the specified date.
    */
   public Message[] getMessagesNewerThan(Date when)
   throws CoffeeServiceException
   {
      m_archive.acquireReadLock();
      try {
         return m_archive.getDBMessages().loadMessagesNewerThan(DateUtils.dateToString(when));
      } finally {
         m_archive.releaseReadLock();
      }
   }

   private FileArchive m_archive;

   private static final String SEQ_NAME = "messages";
}
