///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.apps.clnews.repository;

import org.cooldevtools.coffee_server.services.nntp.pub.*;
import org.cooldevtools.coffee_server.framework.*;

public class FileRoleRepository
implements RoleRepository
{
  public FileRoleRepository(FileArchive archive)
  {
     m_archive = archive;
  }

  public Role getDefaultRole()
  throws CoffeeServiceException
  {
     return new FileRole(DEFAULT_ROLE, m_archive);
  }

  public Role getRole(String rolename)
  throws CoffeeServiceException, InvalidUserOrPasswordException
  {
     return new FileRole(rolename, m_archive);
  }

  public void defineRole(String rolename,
			 String admin_perm,
                         String[] role_patterns,
                         String[] group_patterns,
                         String[] group_perms)
  throws CoffeeServiceException
  {
     boolean commit = false;
     m_archive.acquireWriteLock();
     try {
        m_archive.getDBRoles().defineRole(rolename, admin_perm, role_patterns, group_patterns, group_perms);
        commit = true;
     } finally {
        m_archive.releaseWriteLock(commit);
     }
  }

  public String[] getRoleNames()
  throws CoffeeServiceException
  {
     m_archive.acquireReadLock();
     try {
        return m_archive.getDBRoles().getRoleNames();
     } finally {
        m_archive.releaseReadLock();
     }
  }

  public void deleteRole(String rolename)
  throws CoffeeServiceException
  {
     boolean commit = false;
     m_archive.acquireWriteLock();
     try {
        m_archive.getDBRoles().deleteRole(rolename);
        commit = true;
     } finally {
        m_archive.releaseWriteLock(commit);
     }
  }

  public String[] getRoleDefinition(String rolename)
  throws CoffeeServiceException
  {
     m_archive.acquireReadLock();
     try {
        return m_archive.getDBRoles().getRoleDefinition(rolename);
     } finally {
        m_archive.releaseReadLock();
     }
  }

  private FileArchive m_archive;

  private static final String DEFAULT_ROLE = "default";
}
