///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.apps.clnews.repository;

import java.io.*;
import org.cooldevtools.coffee_server.framework.*;
import org.cooldevtools.coffee_server.services.nntp.pub.*;

public class FileUser
extends NoneUser
{
   public FileUser(String username,
                   FileArchive archive)
   {
      m_name = username;
      m_archive = archive;
   }

   public String getEmailAddress()
   throws CoffeeServiceException
   {
      m_archive.acquireReadLock();
      try {
         return m_archive.getDBUsers().getEmail(m_name);
      } finally {
         m_archive.releaseReadLock();
      }
   }

   public String getUsername()
   throws CoffeeServiceException
   {
      return m_name;
   }

   public String getPassword()
   throws CoffeeServiceException
   {
      m_archive.acquireReadLock();
      try {
         return m_archive.getDBUsers().getPassword(m_name);
      } finally {
         m_archive.releaseReadLock();
      }
   }

   public Role[] getRoles()
   throws CoffeeServiceException
   {
      m_archive.acquireReadLock();
      try {
         String[] role_names = m_archive.getDBUsers().getRoles(m_name);
         if (role_names == null) {
            return null;
         }

         Role[] roles = new Role[role_names.length];
         for (int i = 0; i <roles.length; i++) {
            roles[i] = new FileRole(role_names[i], m_archive);
         }
         return roles;

      } finally {
         m_archive.releaseReadLock();
      }
   }

   private String m_name;
   private FileArchive m_archive;
}
