///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.apps.clnews.repository;

import java.io.*;
import org.cooldevtools.coffee_server.services.nntp.pub.*;
import org.cooldevtools.coffee_server.framework.*;

public class FileUserRepository
implements UserRepository
{
   public FileUserRepository(FileArchive archive)
   {
      m_archive = archive;
   }

   public void defineUser(String username,
                          String password,
                          String email_address,
                          String[] roles)
   throws CoffeeServiceException
   {
      boolean commit = false;
      m_archive.acquireWriteLock();
      try {
         m_archive.getDBUsers().defineUser(username, password, email_address, roles);
         commit = true;
      } finally {
         m_archive.releaseWriteLock(commit);
      }
   }

   public String[] getUserNames()
   throws CoffeeServiceException
   {
      m_archive.acquireReadLock();
      try {
         return m_archive.getDBUsers().getUserNames();
      } finally {
         m_archive.releaseReadLock();
      }
   }

   public void deleteUser(String username)
   throws CoffeeServiceException
   {
      boolean commit = false;
      m_archive.acquireWriteLock();
      try {
         m_archive.getDBUsers().deleteUser(username);
         commit = true;
      } finally {
         m_archive.releaseWriteLock(commit);
      }
   }
      

   public void setUserPassword(String username,
                               String password)
   throws CoffeeServiceException
   {
      boolean commit = false;
      m_archive.acquireWriteLock();
      try {
         m_archive.getDBUsers().deleteUser(username);
         commit = true;
      } finally {
         m_archive.releaseWriteLock(commit);
      }
   }

   public User getUser(String username,
                       String password)
   throws CoffeeServiceException, InvalidUserOrPasswordException
   {
      m_archive.acquireReadLock();
      try {
         String real_password = m_archive.getDBUsers().getPassword(username);
         if ((real_password == null)|| !real_password.equals(password)) {
            throw new InvalidUserOrPasswordException();
         }
         return new FileUser(username, m_archive);
      } finally {
         m_archive.releaseReadLock();
      }
   }

   public User getDefaultUser()
   throws CoffeeServiceException
   {
      return new FileUser(DEFAULT_USER, m_archive);
   }

   public String[] getUserDefinition(String username)
   throws CoffeeServiceException
   {
      m_archive.acquireReadLock();
      try {
         return m_archive.getDBUsers().getUserDefinition(username);
      } finally {
         m_archive.releaseReadLock();
      }
   }
   
   private FileArchive m_archive;

   private static final String DEFAULT_USER = "default";
}
