///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.awt;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import org.cooldevtools.utility.Tuple;

public class BasicFieldPanel
extends BasicContainerPanel
{
   public BasicFieldPanel()
   {
      m_gb = new GridBagHelper(this);
      m_gb.setInsets(2, 2, 2, 2);
      m_fields = new Vector();
      m_rowIndicies = new Vector();
   }

   public void addDocumentListener(DocumentListener listener)
   {
      Enumeration en = m_fields.elements();
      while (en.hasMoreElements()) {
         Tuple tuple = (Tuple)en.nextElement();
         JTextField field = (JTextField)tuple.getElement(1);
         field.getDocument().addDocumentListener(listener);
      }
   }

   public void removeDocumentListener(DocumentListener listener)
   {
      Enumeration en = m_fields.elements();
      while (en.hasMoreElements()) {
         Tuple tuple = (Tuple)en.nextElement();
         JTextField field = (JTextField)tuple.getElement(1);
         field.getDocument().removeDocumentListener(listener);
      }
   }

   private int nextRowIndexFor(int row)
   {
      int size = m_rowIndicies.size();
      for (int i = size; i <= row; i++) {
         m_rowIndicies.addElement(new Integer(0));
      }

      Integer index_int = (Integer)m_rowIndicies.elementAt(row);
      int index = index_int.intValue();
      m_rowIndicies.setElementAt(new Integer(index + 2), row);

      return index;
   }

   public void setEnabled(boolean value)
   {
      super.setEnabled(value);
      Enumeration en = m_fields.elements();
      while (en.hasMoreElements()) {
         Tuple tuple = (Tuple)en.nextElement();
         JTextField field = (JTextField)tuple.getElement(1);
         field.setEnabled(value);
      }
   }

   public synchronized JTextField addField(int row,
                                           String label_text,
                                           String value,
                                           int width)
   {
      int row_index = nextRowIndexFor(row);

      JLabel label = new JLabel(label_text, SwingConstants.RIGHT);
      JTextField field = new JTextField(value, width);

      label.setLabelFor(field);
      field.addActionListener(this);
      field.addFocusListener(AutoSelectAll.s());

      m_gb.setFill(GridBagConstraints.NONE);
      m_gb.setAnchor(GridBagConstraints.EAST);
      m_gb.setBounds(row_index, row, 1, 1);
      m_gb.setWeights(0, 0);
      m_gb.addComponent(label);

      m_gb.setFill(GridBagConstraints.HORIZONTAL);
      m_gb.setAnchor(GridBagConstraints.WEST);
      m_gb.setBounds(row_index + 1, row, 1, 1);
      m_gb.setWeights(1, 0);
      m_gb.addComponent(field);

      m_fields.addElement(new Tuple(label, field));

      return field;
   }

   public void setMnemonic(int index,
                           char key)
   {
      Tuple tuple  = (Tuple)m_fields.elementAt(index);
      JLabel label = (JLabel)tuple.getElement(0);
      label.setDisplayedMnemonic(key);
   }

   public JTextField getField(int index)
   {
      Tuple tuple  = (Tuple)m_fields.elementAt(index);
      return (JTextField)tuple.getElement(1);
   }

   public void setIntFieldValue(int field_id,
                                int value)
   {
      getField(field_id).setText(String.valueOf(value));
   }


   public int getIntFieldValue(int field_id)
   {
      return Integer.parseInt(getField(field_id).getText());
   }

   public void setFieldValue(int field_id,
                             String value)
   {
      getField(field_id).setText(value);
   }


   public String getFieldValue(int field_id)
   {
      return getField(field_id).getText();
   }

   private Vector m_fields;
   private Vector m_rowIndicies;
   private GridBagHelper m_gb;
}
