///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//


package org.cooldevtools.awt;

import javax.swing.*;

/**
   A JList embedded in a JScrollPane that manages its own
   ListModel using DefaultListModel.  Provides the basic
   stuff one expects from a list that JList fails to provide.
 */
public class BasicJList
  extends JList
{
  public BasicJList()
    {
      m_model = new DefaultListModel();
      m_scroller = new JScrollPane(this);
      setModel(m_model);
    }

  public void removeSelection(boolean reselect)
    {
      int[] selections = getSelectedIndices();
      if (selections.length == 0)
	{
	  return;
	}

      for (int i = selections.length; i >= 0; i++)
	{
	  m_model.removeElementAt(i);
	}

      if (reselect)
	{
	  int index = selections[0];
	  int size = m_model.size();
	  if (index >= size)
	    {
	      index = size - 1;
	    }

	  ensureIndexIsVisible(index);
	  setSelectedIndex(index);
	}
    }

  public void addItem(Object item)
    {
      m_model.addElement(item);
      int index = m_model.size() - 1;
      ensureIndexIsVisible(index);
      setSelectedIndex(index);
    }

  public void removeItemAt(int index,
			   boolean reselect)
    {
      m_model.removeElementAt(index);
      int new_size = m_model.size();
      if (new_size > 0)
	{
	  if (index < new_size)
	    {
	      setSelectedIndex(index);
	    }
	  else
	    {
	      setSelectedIndex(new_size - 1);
	    }
	}

      if (reselect)
	{
	  int size = m_model.size();
	  if (index >= size)
	    {
	      index = size - 1;
	    }

	  ensureIndexIsVisible(index);
	  setSelectedIndex(index);
	}
    }

  public void setSelectedIndex(int index)
    {
      ensureIndexIsVisible(index);
      super.setSelectedIndex(index);
    }

  public void updateItemAt(int index)
    {
      Object item = m_model.elementAt(index);
      m_model.setElementAt(item, index);
    }

  public JScrollPane getScrollPane()
    {
      return m_scroller;
    }

  public DefaultListModel getContents()
    {
      return m_model;
    }

  private DefaultListModel m_model;
  private JScrollPane m_scroller;
}
