///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.awt;

import java.awt.*;

public class EqualSizeLayout
implements LayoutManager
{
   public EqualSizeLayout()
   {
      this(0, false);
   }

   public EqualSizeLayout(int padding)
   {
      this(padding, false);
   }

   public EqualSizeLayout(int padding,
                          boolean is_vertical)
   {
      m_padding = padding;
      m_isVertical = is_vertical;
   }

   public void addLayoutComponent(String name,
                                  Component comp)
   {
   }

   public void removeLayoutComponent(Component comp)
   {
   }

   private void calcSize(Container target,
                         Dimension  dim,
                         int[] num_gaps,
                         int[] num_visible)
   {
      num_gaps[0] = 0;
      num_visible[0] = 0;
      int num_components = target.getComponentCount();
      for (int i = 0; i < num_components; i++) {
         Component comp = target.getComponent(i);
         if (comp.isVisible()) {
            Dimension comp_dim = comp.getPreferredSize();
            if (comp_dim.width > dim.width) {
               dim.width = comp_dim.width;
            }
            if (comp_dim.height > dim.height) {
               dim.height = comp_dim.height;
            }
            if (comp instanceof Gap) {
               num_gaps[0]++;
            } else {
               num_visible[0]++;
            }
         }
      }
   }

   private void adjustDimension(Dimension dim,
                                int num_elements,
                                Insets insets)
   {
      if (m_isVertical) {
         dim.height = dim.height * num_elements + m_padding * (num_elements - 1);
      } else {
         dim.width = dim.width * num_elements + m_padding * (num_elements - 1);
      }
      dim.height += insets.top + insets.bottom;
      dim.width  += insets.left + insets.right;
   }

   public Dimension preferredLayoutSize(Container target)
   {
      int[] num_gaps = { 0 };
      int[] num_visible = { 0 };
      Dimension dim = new Dimension(0, 0);

      calcSize(target, dim, num_gaps, num_visible);
      adjustDimension(dim, num_gaps[0] + num_visible[0], target.getInsets());
      return dim;
   }

   public Dimension minimumLayoutSize(Container target)
   {
      int[] num_gaps = { 0 };
      int[] num_visible = { 0 };
      Dimension dim = new Dimension(0, 0);

      calcSize(target, dim, num_gaps, num_visible);
      adjustDimension(dim, num_visible[0], target.getInsets());
      return dim;
   }

   public void layoutContainer(Container target)
   {
      int[] num_gaps_array = { 0 };
      int[] num_visible_array = { 0 };
      Dimension dim = new Dimension(0, 0);
      calcSize(target, dim, num_gaps_array, num_visible_array);

      int num_visible = num_visible_array[0];
      int num_gaps    = num_gaps_array[0];
      
      Dimension tsize = target.getSize();
      Insets insets = target.getInsets();
      int x = insets.left;
      int y = insets.top;
      int x_increment = 0;
      int y_increment = 0;
      int x_gap_width = 0;
      int x_gap_extra = 0;
      int y_gap_width = 0;
      int y_gap_extra = 0;

      if (m_isVertical) {
         if (num_gaps > 0) {
            int min_size = dim.height * num_visible + m_padding * (num_visible - 1)
                         + insets.top + insets.bottom;
            int gap_space = tsize.height - min_size;
            if (gap_space < 0) {
               gap_space = 0;
            }
            y_gap_width = gap_space / num_gaps;
            y_gap_extra = gap_space % num_gaps;
         }
         y_increment = dim.height + m_padding;
      } else {
         if (num_gaps > 0) {
            int min_size = dim.width * num_visible + m_padding * (num_visible - 1)
                         + insets.left + insets.right;
            int gap_space = tsize.width - min_size;
            if (gap_space < 0) {
               gap_space = 0;
            }
            x_gap_width = gap_space / num_gaps;
            x_gap_extra = gap_space % num_gaps;
         }
         x_increment = dim.width + m_padding;
      }
      
      int gap_count = 0;
      int num_components = target.getComponentCount();
      for (int i = 0; i < num_components; i++) {
         Component comp = target.getComponent(i);
         if (comp.isVisible()) {
            if (comp instanceof Gap) {
               x += x_gap_width;
               y += y_gap_width;
               if (gap_count == 0) {
                  x += x_gap_extra;
                  y += y_gap_extra;
               }
               gap_count++;
            } else {
               comp.setSize(dim);
               comp.setBounds(x, y, dim.width, dim.height);
               x += x_increment;
               y += y_increment;
            }
         }
      }
   }

   private int m_padding;
   private boolean m_isVertical;
}
