///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.awt;

import java.awt.*;

public class GapFlowLayout
implements LayoutManager
{
   public GapFlowLayout()
   {
      this(0, false);
   }

   public GapFlowLayout(int padding)
   {
      this(padding, false);
   }

   public GapFlowLayout(int padding,
                        boolean is_vertical)
   {
      this(padding, is_vertical, true);
   }

   public GapFlowLayout(int padding,
                        boolean is_vertical,
                        boolean fill)
   {
      m_padding = padding;
      m_isVertical = is_vertical;
      m_fill = fill;
   }

   public void addLayoutComponent(String name,
                                  Component comp)
   {
   }

   public void removeLayoutComponent(Component comp)
   {
   }

   private void calcSize(Container target,
                         Dimension dim,
                         int[] num_gaps_arg,
                         int[] num_visible_arg)
   {
      int num_gaps = 0;
      int num_visible = 0;
      int num_components = target.getComponentCount();
      for (int i = 0; i < num_components; i++) {
         Component comp = target.getComponent(i);
         if (comp.isVisible()) {
            Dimension comp_dim = comp.getPreferredSize();
            if (m_isVertical) {
               dim.height += comp_dim.height;
               if (comp_dim.width > dim.width) {
                  dim.width = comp_dim.width;
               }
            } else {
               dim.width += comp_dim.width;
               if (comp_dim.height > dim.height) {
                  dim.height = comp_dim.height;
               }
            }
            if (comp instanceof Gap) {
               num_gaps++;
            } else {
               num_visible++;
            }
         }
      }

      if (num_visible > 0) {
         if (m_isVertical) {
            dim.height += m_padding * (num_visible - 1);
         } else {
            dim.width  += m_padding * (num_visible - 1);
         }
      }

      Insets insets = target.getInsets();
      dim.width  += insets.left + insets.right;
      dim.height += insets.top + insets.bottom;
      
      if (num_gaps_arg != null) {
         num_gaps_arg[0] = num_gaps;
      }
      if (num_visible_arg != null) {
         num_visible_arg[0] = num_visible;
      }
   }

   public Dimension preferredLayoutSize(Container target)
   {
      Dimension dim = new Dimension();
      calcSize(target, dim, null, null);
      return dim;
   }

   public Dimension minimumLayoutSize(Container target)
   {
      return preferredLayoutSize(target);
   }

   public void layoutContainer(Container target)
   {
      int[] num_gaps_array = { 0 };
      int[] num_visible_array = { 0 };
      Dimension dim = new Dimension(0, 0);
      calcSize(target, dim, num_gaps_array, num_visible_array);

      int num_visible = num_visible_array[0];
      int num_gaps    = num_gaps_array[0];
      
      Dimension tsize = target.getSize();
      Insets insets = target.getInsets();
      int x = insets.left;
      int y = insets.top;
      int x_gap_width = 0;
      int x_gap_extra = 0;
      int y_gap_width = 0;
      int y_gap_extra = 0;
      int fixed_size  = 0;

      if (m_isVertical) {
         if (num_gaps > 0) {
            int gap_space = tsize.height - dim.height;
            if (gap_space < 0) {
               gap_space = 0;
            }
            y_gap_width = gap_space / num_gaps;
            y_gap_extra = gap_space % num_gaps;
         }
         fixed_size = tsize.width - insets.left - insets.right;
      } else {
         if (num_gaps > 0) {
            int gap_space = tsize.width - dim.width;
            if (gap_space < 0) {
               gap_space = 0;
            }
            x_gap_width = gap_space / num_gaps;
            x_gap_extra = gap_space % num_gaps;
         }
         fixed_size = tsize.width - insets.top - insets.bottom;
      }
      
      int gap_count = 0;
      int num_components = target.getComponentCount();
      for (int i = 0; i < num_components; i++) {
         Component comp = target.getComponent(i);
         if (comp.isVisible()) {
            Dimension comp_dim = comp.getPreferredSize();
            if (comp instanceof Gap) {
               x += x_gap_width;
               y += y_gap_width;
               if (gap_count == 0) {
                  x += x_gap_extra;
                  y += y_gap_extra;
               }
               gap_count++;
            } else {
               if (m_isVertical) {
                  if (m_fill) {
                     comp_dim.width = fixed_size;
                  }
                  comp.setSize(comp_dim);
                  comp.setBounds(x, y, comp_dim.width, comp_dim.height);
                  y += m_padding;
               } else {
                  if (m_fill) {
                     comp_dim.height = fixed_size;
                  }
                  comp.setSize(comp_dim);
                  comp.setBounds(x, y, comp_dim.width, comp_dim.height);
                  x += m_padding;
               }
            }
            if (m_isVertical) {
               y += comp_dim.height;
            } else {
               x += comp_dim.width;
            }
         }
      }
   }

   private int m_padding;
   private boolean m_isVertical;
   private boolean m_fill;
}
