///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.awt;

import java.awt.*;

public class GridBagHelper
{
  public GridBagHelper(Container container)
    {
      m_gridbag = new GridBagLayout();
      m_constraints = new GridBagConstraints();
      m_container = container;

      m_container.setLayout(m_gridbag);

      setBounds(0, 0, 1, 1);
      setInsets(4, 4, 4, 4);
    }

  public void addComponent(Component component)
    {
      m_gridbag.setConstraints(component, m_constraints);
      m_container.add(component);
    }

  public Container getContainer()
    {
      return m_container;
    }

  public GridBagConstraints getConstraints()
    {
      return (GridBagConstraints)m_constraints.clone();
    }

  public GridBagLayout getGridBag()
    {
      return m_gridbag;
    }

  public void resetConstraints()
    {
      m_constraints = new GridBagConstraints();
    }

  public void setAnchor(int value)
    {
      m_constraints.anchor = value;
    }

  public void setBounds(int x,
			int y,
			int width,
			int height)
    {
      m_constraints.gridx = x;
      m_constraints.gridy = y;
      m_constraints.gridwidth = width;
      m_constraints.gridheight = height;
    }

  public void setConstraints(GridBagConstraints constraints)
    {
      m_constraints = (GridBagConstraints)constraints.clone();
    }

  public void setCoord(int x,
		       int y)
    {
      m_constraints.gridx = x;
      m_constraints.gridy = y;
    }

  public void setFill(int value)
    {
      m_constraints.fill = value;
    }

  public void setInsets(int top,
			int left,
			int bottom,
			int right)
    {
      m_constraints.insets = new Insets(top, left, bottom, right);
    }

  public void setPadding(int h,
			 int v)
    {
      m_constraints.ipadx = h;
      m_constraints.ipady = v;
    }

  public void setSize(int width,
		      int height)
    {
      m_constraints.gridwidth  = width;
      m_constraints.gridheight = height;
    }

  public void setWeights(double h,
			 double v)
    {
      m_constraints.weightx = h;
      m_constraints.weighty = v;
    }

  private GridBagLayout m_gridbag;
  private GridBagConstraints m_constraints;
  private Container m_container;
};
