///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.awt;

import java.awt.*;
import java.io.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.border.*;
import org.cooldevtools.utility.LinkedList;

public class HelpWindow
extends JFrame
implements HyperlinkListener
{
   public HelpWindow(String title)
   {
      super(title);
      initDisplay();
   }

   private class ContentsHandler implements ActionListener {
      public void actionPerformed(ActionEvent ev)
      {
         try {
            m_browser.setPage(m_contentsURL);
            m_history.clear();
         } catch (IOException ex) {
         }
      }
   }

   private class IndexHandler implements ActionListener {
      public void actionPerformed(ActionEvent ev)
      {
         try {
            m_browser.setPage(m_indexURL);
            m_history.clear();
         } catch (IOException ex) {
         }
      }
   }

   private class BackHandler implements ActionListener {
      public void actionPerformed(ActionEvent ev)
      {
         try {
            if (m_history.isEmpty()) {
               m_browser.setPage(m_contentsURL);
            } else {
               m_browser.setPage((String)m_history.pop());
            }
         } catch (IOException ex) {
         }
      }
   }

   private class WindowHandler extends WindowAdapter {
      public void windowClosing(WindowEvent ev)
      {
         ev.getWindow().setVisible(false);
      }
   }

   public void hyperlinkUpdate(HyperlinkEvent ev)
   {
      try {
         m_history.push(m_browser.getPage().toString());
         m_browser.setPage(ev.getURL());
      } catch (IOException ex) {
         try {
            m_browser.setPage((String)m_history.pop());
         } catch (IOException ignored) {
            ex.printStackTrace();
         }
      }
   }

   private void initDisplay()
   {
      m_browser = new JEditorPane();
      m_browser.addHyperlinkListener(this);
      m_browser.setEditable(false);
      m_history = new LinkedList();

      m_backButton = new JButton("Back");
      m_indexButton  = new JButton("Index");
      m_contentsButton  = new JButton("Contents");

      JPanel p = new JPanel();
      p.setBorder(new EmptyBorder(5, 5, 5, 5));
      p.setLayout(new GridLayout(1, 4));
      p.add(m_contentsButton);
      p.add(m_indexButton);
      p.add(new JLabel(""));
      p.add(m_backButton);

      getContentPane().setLayout(new BorderLayout());
      getContentPane().add(p, "North");
      getContentPane().add(new JScrollPane(m_browser), "Center");
      pack();

      addWindowListener(new WindowHandler());
      m_contentsButton.addActionListener(new ContentsHandler());
      m_indexButton.addActionListener(new IndexHandler());
      m_backButton.addActionListener(new BackHandler());
   }

   public Dimension getPreferredSize()
   {
      return new Dimension(480, 320);
   }

   public void setContents(String url)
   {
      m_contentsURL = url;
   }

   public void setIndex(String url)
   {
      m_indexURL = url;
      m_indexButton.setEnabled(url != null);
   }

   public void showURL(String url)
   throws IOException
   {
      if (m_contentsURL == null) {
         m_contentsURL = url;
      }

      m_browser.setPage(url);
      setVisible(true);
      toFront();
   }

   public void showTop()
   throws IOException
   {
      m_browser.setPage(m_contentsURL);
      setVisible(true);
      toFront();
   }

   public static void main(String[] args)
   throws IOException
   {
      if (args.length != 1) {
         System.err.println("usage: HelpWindow filename");
         return;
      }
      HelpWindow win = new HelpWindow("HTML Window");
      win.setLocation(10, 30);
      win.showURL("file:" + (new File(args[0])).getAbsolutePath());
   }

   private String m_contentsURL;
   private String m_indexURL;
   private JButton m_backButton;
   private JButton m_contentsButton;
   private JButton m_indexButton;
   private JEditorPane m_browser;
   private LinkedList m_history;
}
