///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//


package org.cooldevtools.coffee_server.config;

import BccUnix;
import java.io.*;
import java.awt.*;
import java.util.*;
import org.cooldevtools.hdl.*;
import org.cooldevtools.utility.*;
import org.cooldevtools.utility.logging.*;
import org.cooldevtools.coffee_server.framework.CoffeeServiceException;
import org.cooldevtools.awt.*;

public class CoffeeServerConfigInfo
{
   public CoffeeServerConfigInfo()
   {
      m_numThreads = 0;
      m_filename   = null;
      m_libraries  = new SuperVector();
      m_services   = new ServiceConfigInfo[0];
      m_logLevel   = DEFAULT_LOG_LEVEL;
      m_isGuiBased = false;
   }

   public void setGuiBased(boolean value)
   {
      m_isGuiBased = value;
   }

   public boolean isGuiBased()
   {
      return m_isGuiBased;
   }

   public Frame getParentFrame()
   {
      return m_ParentFrame;
   }

   public void setParentFrame(Frame value)
   {
      m_ParentFrame = value;
   }

   public boolean getLogToFile()
   {
      return m_logToFile;
   }

   public String getLogParam()
   {
      return m_logParam;
   }

   public String getLogLevel()
   {
      return LOG_LEVEL_TOKENS[m_logLevel].strValue();
   }

   public void setLogLevel(String level)
   throws CoffeeServiceException
   {
      for (int i = 0; i < LOG_LEVEL_TOKENS.length; i++) {
         if (LOG_LEVEL_TOKENS[i].strValue().equals(level)) {
            m_logLevel = i;
            Logger.s().setMinPriority(LOG_PRIORITIES[m_logLevel]);
            return;
         }
      }
      throw new CoffeeServiceException("invalid log level: " + level);
   }

   public boolean isValidLogLevel(String log_level)
   {
      if (log_level == null) {
         return false;
      }

      for (int i = 0; i < LOG_LEVEL_TOKENS.length; i++) {
         if (LOG_LEVEL_TOKENS[i].strValue().equals(log_level)) {
            return true;
         }
      }

      return false;
   }

   public void setLogging(boolean log_to_file,
                          String log_param)
   throws CoffeeServiceException
   {
      LoggerInterface old_logger = Logger.s();
      try {
         if (log_to_file) {
            if (log_param.equals("-")) {
               Logger.restoreDefaultLogger();
            } else {
               Logger.initSingleton(new FileLogger(log_param, 4));
            }
         } else {
            Logger.initSingleton(new SyslogLogger(SyslogFacility.LOG_NEWS, log_param));
         }
         m_logToFile = log_to_file;
         m_logParam  = log_param;
         old_logger.close();
      } catch (LoggingException ex) {
         Logger.initSingleton(old_logger);
         throw new CoffeeServiceException(ex);
      }
      Logger.s().setMinPriority(LOG_PRIORITIES[m_logLevel]);
   }

   public String getFilename()
   {
      return m_filename;
   }

   public int getNumThreads()
   {
      return m_numThreads;
   }

   public void setNumThreads(int value)
   {
      m_numThreads = value;
   }

   public ServiceConfigInfo[] getServices()
   {
      ServiceConfigInfo[] copy = new ServiceConfigInfo[m_services.length];
      System.arraycopy(m_services, 0, copy, 0, copy.length);
      return copy;
   }

   public void setServices(ServiceConfigInfo[] value)
   {
      m_services = new ServiceConfigInfo[value.length];
      System.arraycopy(value, 0, m_services, 0, m_services.length);
   }

   public void setServices(Vector value)
   {
      m_services = new ServiceConfigInfo[value.size()];
      value.copyInto(m_services);
   }

   public void loadLibraries()
   throws ConfigurationException
   {
      String[] libs = m_libraries.getStrings();
      for (int i = 0; i < libs.length; i++) {
         try {
            File f = new File(libs[i]);
            if (f.exists()) {
               System.load(libs[i]);
            } else {
               System.loadLibrary(libs[i]);
            }
         } catch (Throwable ex) {
            Logger.s().logSilently(MessagePriority.LOG_NOTICE,
                                   "load library {0}: {1}",
                                   new Object[] { libs[i], ex.toString() });
         }
      }
   }

   public void changeIdentity()
   throws ConfigurationException
   {
      if (m_groupName == null) {
         if (Assert.DEBUG) Assert.pre(m_userName == null);
         return;
      }

      if (!BccUnix.loadLibrary()) {
         Logger.s().logSilently(MessagePriority.LOG_NOTICE, "failed to load Unix library bccj");
         return;
      }

      try {
         BccUnix.setGroupID(m_groupName);
         Logger.s().logSilently(MessagePriority.LOG_NOTICE, "group id set to {0}",
                                new Object[] { m_groupName });
      } catch (Throwable ex) {
         Logger.s().logSilently(MessagePriority.LOG_NOTICE,
                                "set group failed: {0}: {1}",
                                new Object[] { m_groupName, ex.toString() });
      }

      try {
         BccUnix.setUserID(m_userName);
         Logger.s().logSilently(MessagePriority.LOG_NOTICE,
                                "user id set to {0}",
                                new Object[] { m_groupName });
      } catch (Throwable ex) {
         Logger.s().logSilently(MessagePriority.LOG_NOTICE,
                                "set user failed: {0}: {1}",
                                new Object[] { m_userName, ex.getMessage() });
      }
   }

   public void setUserName(String name)
   {
      if (name == null || name.length() == 0) {
         m_userName = null;
      } else {
         m_userName = name;
      }
   }

   public String getUserName()
   {
      return m_userName;
   }

   public void setGroupName(String name)
   {
      if (name == null || name.length() == 0) {
         m_groupName = null;
      } else {
         m_groupName = name;
      }
   }

   public String getGroupName()
   {
      return m_groupName;
   }

   public String getServerName()
   {
      return m_serverName;
   }

   public static CoffeeServerConfigInfo readFromHDL(Statement root)
   throws IOException, BadClassNameException, HDLException, CoffeeServiceException
   {
      SyntaxChecker syntax = new SyntaxChecker(HDL_SYNTAX);
      syntax.checkSyntax(root);

      CoffeeServerConfigInfo info = new CoffeeServerConfigInfo();

      int num_children = root.numChildren();
      SuperVector services = new SuperVector(num_children);

      for (int i = 0; i < num_children; i++) {
         Statement child = root.child(i);
         Token name = child.name();
         if (name.equals(TOK_THREADS)) {
            info.m_numThreads = child.argument(0).intValue();
         } else if (name.equals(TOK_SERVER_NAME)) {
            info.m_serverName = child.argument(0).strValue();
         } else if (name.equals(TOK_LOG)) {
            info.setLogging(child.argument(0).equals(TOK_FILE), child.argument(1).strValue());
         } else if (name.equals(TOK_LOG_LEVEL)) {
            info.setLogLevel(child.argument(0).strValue());
         } else if (name.equals(TOK_LOAD_LIBRARY)) {
            info.m_libraries.addElement(child.argument(0).strValue());
         } else if (name.equals(TOK_UNIX_ID)) {
            info.m_userName = child.argument(0).strValue();
            info.m_groupName = child.argument(1).strValue();
         } else {
            services.addElement(ServiceConfigInfo.readFromHDL(info, child));
         }
      }
      info.m_services = ServiceConfigInfo.toArray(services);;

      return info;
   }

   public static CoffeeServerConfigInfo readFromFile(String filename)
   throws IOException, BadClassNameException, HDLException, LoggingException,
          CoffeeServiceException
   {
      CoffeeServerConfigInfo config = readFromHDL(new Parser().parse(filename));
      config.m_filename = filename;
      return config;
   }

   public void writeToFile(String filename)
   throws IOException
   {
      Statement root = writeToHDL();
      PrintWriter out = new PrintWriter(new FileWriter(filename));
      try {
         root.write(out, "", "   ");
      } finally {
         out.close();
      }
   }

   public void writeToFile()
   throws IOException
   {
      writeToFile(m_filename);
   }

   public static CoffeeServerConfigInfo readFromStream(InputStream ins)
   throws IOException, BadClassNameException, HDLException, LoggingException,
          CoffeeServiceException
   {
      BufferedReader in = new BufferedReader(new InputStreamReader(ins));
      try {
         Parser p = new Parser();
         Statement stmt = p.parse(in);
         return readFromHDL(stmt);
      } finally {
         in.close();
      }
   }

   public Statement writeToHDL()
   throws IOException
   {
      Statement root = new Statement(TOK_SERVER, true);

      Statement stmt = new Statement(TOK_LOG, false);
      stmt.addArgument(m_logToFile ? TOK_FILE : TOK_SYSLOG);
      stmt.addArgument(Token.strToken(m_logParam));
      root.addChild(stmt);

      stmt = new Statement(TOK_LOG_LEVEL, false);
      stmt.addArgument(LOG_LEVEL_TOKENS[m_logLevel]);
      root.addChild(stmt);

      stmt = new Statement(TOK_THREADS, false);
      stmt.addArgument(Token.intToken(m_numThreads, "", 0));
      root.addChild(stmt);

      if (m_serverName != null) {
         stmt = new Statement(TOK_SERVER_NAME, false);
         stmt.addArgument(Token.strToken(m_serverName, "", 0));
         root.addChild(stmt);
      }

      String[] libs = m_libraries.getStrings();
      for (int i = 0; i < libs.length; i++) {
         stmt = new Statement(TOK_LOAD_LIBRARY, false);
         stmt.addArgument(Token.strToken(libs[i], "", 0));
         root.addChild(stmt);
      }

      if (m_groupName != null) {
         stmt = new Statement(TOK_UNIX_ID, false);
         stmt.addArgument(Token.strToken(m_userName, "", 0));
         stmt.addArgument(Token.strToken(m_groupName, "", 0));
         root.addChild(stmt);
      }

      for (int i = 0; i < m_services.length; i++) {
         root.addChild(m_services[i].writeToHDL());
      }

      return root;
   }

   private int m_numThreads;
   private String m_userName;
   private String m_groupName;
   private String m_serverName;
   private SuperVector m_libraries;
   private ServiceConfigInfo[] m_services;
   private String m_filename;
   private boolean m_logToFile;
   private int m_logLevel;
   private String  m_logParam;
   private boolean m_isGuiBased;
   private Frame m_ParentFrame;

   public static final String HDL_SYNTAX =
      "begin language server;\n" +
      "  begin statement server_name;\n" +
      "    args 1 1 string;\n" +
      "  end;\n" +
      "  begin statement num_threads;\n" +
      "    args 1 1 int;\n" +
      "  end;\n" +
      "  begin statement log;\n" +
      "    args 2 2 table string;\n" +
      "    table id file id syslog;\n" +
      "  end;\n" +
      "  begin statement log_level;\n" +
      "    args 1 1 table;\n" +
      "    table id verbose id normal id quiet;\n" +
      "  end;\n" +
      "  begin statement load_library;\n" +
      "    args 1 1 string;\n" +
      "  end;\n" +
      "  begin statement unix_id;\n" +
      "    args 2 2 string string;\n" +
      "  end;\n" +
      "  begin block service;\n" +
      "    args 1 1 string;\n" +
      "    begin statement class;\n" +
      "      args 1 1 string;\n" +
      "    end;\n" +
      "    begin statement address;\n" +
      "      args 1 1 string;\n" +
      "    end;\n" +
      "    begin statement port;\n" +
      "      args 1 1 int;\n" +
      "    end;\n" +
      "    begin statement alternate_port;\n" +
      "      args 1 1 int;\n" +
      "    end;\n" +
      "    begin statement socket_timeout;\n" +
      "      args 1 1 int;\n" +
      "    end;\n" +
      "    begin statement thread_model;\n" +
      "      args 1 1 int;\n" +
      "    end;\n" +
      "    begin statement property;\n" +
      "      args 2 2 anyid table;\n" +
      "      table string int;\n" +
      "    end;\n" +
      "    unchecked configuration;\n" +
      "  end;\n" +
      "end;\n";

   private static Token TOK_SERVER       = Token.idToken("server");
   private static Token TOK_SERVER_NAME  = Token.idToken("server_name");
   private static Token TOK_THREADS      = Token.idToken("num_threads");
   private static Token TOK_LOG          = Token.idToken("log");
   private static Token TOK_FILE         = Token.idToken("file");
   private static Token TOK_SYSLOG       = Token.idToken("syslog");
   private static Token TOK_LOAD_LIBRARY = Token.idToken("load_library");
   private static Token TOK_UNIX_ID      = Token.idToken("unix_id");
   private static Token TOK_LOG_LEVEL    = Token.idToken("log_level");
   private static Token TOK_LOG_VERBOSE  = Token.idToken("verbose");
   private static Token TOK_LOG_NORMAL   = Token.idToken("normal");
   private static Token TOK_LOG_QUIET    = Token.idToken("quiet");
   private static Token[] LOG_LEVEL_TOKENS = {
      TOK_LOG_VERBOSE,
      TOK_LOG_NORMAL,
      TOK_LOG_QUIET
   };
   private static final MessagePriority[] LOG_PRIORITIES = {
      MessagePriority.LOG_DEBUG,
      MessagePriority.LOG_NOTICE,
      MessagePriority.LOG_WARNING
   };
   private static final int DEFAULT_LOG_LEVEL = 0;
}
