///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//


package org.cooldevtools.coffee_server.config;

import java.awt.*;
import java.util.*;
import javax.swing.*;
import org.cooldevtools.awt.*;

public class ConfigFrame
  extends JFrame
{
  public ConfigFrame(String label)
    {
      super(label);
      initDisplay();
    }

  public void showMainPage()
    {
      m_cards.show(m_pages, TABS_PAGE);
    }

  public void showBeanPage(Component bean)
    {
      m_beanPanel.remove(m_beanComponent);
      m_beanComponent = bean;
      m_beanPanel.add(m_beanComponent);
      m_beanPanel.doLayout();

      m_cards.show(m_pages, BEAN_PAGE);
    }

  private void initDisplay()
    {
      m_menuItems = new Hashtable();

      m_cards = new CardLayout();
      m_pages = new JPanel();
      m_pages.setLayout(m_cards);
      getContentPane().add(m_pages);

      m_tabs = new JTabbedPane();
      m_pages.add(TABS_PAGE, m_tabs);

      m_framework = new FrameworkConfigPanel();
      m_services = new ServicesConfigPanel();

      m_tabs.addTab("Framework", m_framework);
      m_tabs.addTab("Services", m_services);

      JMenuBar mb = new JMenuBar();
      setJMenuBar(mb);

      JMenu file_menu = new JMenu("File");
      addMenuItem(file_menu, "File:New", "New");
      file_menu.addSeparator();
      addMenuItem(file_menu, "File:Open", "Open");
      addMenuItem(file_menu, "File:Save", "Save");
      addMenuItem(file_menu, "File:Save As", "Save As...");
      file_menu.addSeparator();
      addMenuItem(file_menu, "File:Revert", "Revert");
      file_menu.addSeparator();
      addMenuItem(file_menu, "File:Quit", "Quit");
      mb.add(file_menu);

      if (false)
	{
	  JMenu edit_menu = new JMenu("Edit");
	  addMenuItem(edit_menu, "Edit:Cut", "Cut");
	  addMenuItem(edit_menu, "Edit:Copy", "Copy");
	  addMenuItem(edit_menu, "Edit:Paste", "Paste");
	  mb.add(edit_menu);
	}

      JMenu help_menu = new JMenu("Help");
      addMenuItem(help_menu, "Help:About", "About CoffeeServer");
      mb.add(help_menu);

      JPanel p = new JPanel();
      m_pages.add(BEAN_PAGE, p);

      p.setLayout(new BorderLayout());

      m_beanPanel = new JPanel();
      p.add(m_beanPanel, "Center");

      m_beanComponent = new JTextArea(10, 40);
      m_beanPanel.add(m_beanComponent);

      JPanel tmp    = new JPanel();
      m_beanOK     = new JButton("OK");
      m_beanCancel = new JButton("Cancel");
      tmp.add(m_beanOK);
      tmp.add(m_beanCancel);

      p.add(tmp, "South");

      pack();
    }

  public JButton getBeanOK()
    {
      return m_beanOK;
    }

  public JButton getBeanCancel()
    {
      return m_beanCancel;
    }

  public ServicesConfigPanel getServicesConfigPanel()
    {
      return m_services;
    }

  public FrameworkConfigPanel getFrameworkConfigPanel()
    {
      return m_framework;
    }

  private void addMenuItem(JMenu menu,
			   String key,
			   String text)
    {
      JMenuItem item = new JMenuItem(text);
      menu.add(item);
      m_menuItems.put(key, item);
    }

  public JMenuItem getMenuItem(String key)
    {
      return (JMenuItem)m_menuItems.get(key);
    }

  public static final void main(String[] args)
    {
      ConfigFrame f = new ConfigFrame("Testing");
      f.show();
    }

  private JPanel m_pages;
  private CardLayout m_cards;

  private JTabbedPane m_tabs;
  private FrameworkConfigPanel m_framework;
  private ServicesConfigPanel m_services;
  private Hashtable m_menuItems;

  private JPanel m_beanPanel;
  private JButton m_beanOK;
  private JButton m_beanCancel;
  private Component m_beanComponent;

  private static final String TABS_PAGE = "Tabs Card";
  private static final String BEAN_PAGE = "Bean Card";
}
