///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//


package org.cooldevtools.coffee_server.config;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import org.cooldevtools.hdl.*;
import org.cooldevtools.awt.*;

public class ConfigFrameDriver
{
  public ConfigFrameDriver(ConfigFrame frame,
			   String filename)
    {
      m_frame = frame;
      m_filename = filename;
      m_services = new ServicesConfigPanelDriver(m_frame.getServicesConfigPanel(),
						 new BeanPageDriver(m_frame));
      addMenuListener("File:New", new NewItemListener());
      addMenuListener("File:Open", new OpenItemListener());
      addMenuListener("File:Save", new SaveItemListener());
      addMenuListener("File:Save As", new SaveAsItemListener());
      addMenuListener("File:Revert", new RevertItemListener());
      addMenuListener("File:Quit", new QuitItemListener());
      addMenuListener("Help:About", new AboutItemListener());

      if (m_filename != null)
	{
	  read();
	}
      else
	{
	  load(new CoffeeServerConfigInfo());
	}
    }

  private void addMenuListener(String key,
			       ActionListener listener)
    {
      JMenuItem item = m_frame.getMenuItem(key);
      item.addActionListener(listener);
    }

  private String askFilename(int mode,
			     String title)
    {
      FileDialog fd = new FileDialog(m_frame, title, mode);
      fd.show();
      return fd.getFile();
    }

  private void read()
    {
      try
	{
	  InputStream in = new FileInputStream(m_filename);
	  try
	    {
	      CoffeeServerConfigInfo info = CoffeeServerConfigInfo.readFromStream(in);
	      load(info);
	    }
	  finally
	    {
	      in.close();
	    }
	}
      catch (Exception ex)
	{
	  ex.printStackTrace();
	}
    }

  private void open()
    {
      String filename = askFilename(FileDialog.LOAD, "Enter File Name");
      if (filename != null)
	{
	  m_filename = filename;
	  read();
	}
    }

  private void save()
    {
      CoffeeServerConfigInfo info = new CoffeeServerConfigInfo();

      try
	{
	  store(info);
	}
      catch (ConfigurationException ex)
	{
	  String[] message = {
	    "Unable to save current configuration:",
	    ex.getMessage()
	  };
	  JOptionPane.showMessageDialog(null,
					message,
					"Invalid Configuration",
					JOptionPane.ERROR_MESSAGE);
	  return;
	}

      try
	{
	  PrintWriter out = new PrintWriter(new FileWriter(m_filename));
	  try
	    {
	      info.writeToHDL().write(out, "", "    ");
	    }
	  finally
	    {
	      out.close();
	    }
	}
      catch (Exception ex)
	{
	  ex.printStackTrace();
	}
    }

  private void saveAs()
    {
      String filename = askFilename(FileDialog.SAVE, "Enter File Name");
      if (filename != null)
	{
	  m_filename = filename;
	  save();
	}
    }

  public class NewItemListener
    implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
      {
	load(new CoffeeServerConfigInfo());
      }
  }

  public class OpenItemListener
    implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
      {
	open();
      }
  }

  public class SaveItemListener
    implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
      {
	if (m_filename == null)
	  {
	    saveAs();
	  }
	else
	  {
	    save();
	  }
      }
  }

  public class SaveAsItemListener
    implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
      {
	saveAs();
      }
  }

  public class RevertItemListener
    implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
      {
	load(m_info);
      }
  }

  public class QuitItemListener
    implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
      {
	System.exit(0);
      }
  }

  public class AboutItemListener
    implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
      {
	String[] lines = {
	  "CoffeeServer v 0.5",
	  "(C) 1998 Burton Computer Corporation",
	  "http://www.burton-computer.com",
	  "   ",
	  "CoffeeServer comes with ABSOLUTELY NO WARRANTY.",
	  "This is free software, and you are welcome to redistribute it",
	  "under the terms of the Mozilla Public License Version 1.0.",
	  "You may obtain a copy of the License at",
	  "http://www.mozilla.org/MPL/"
	};
	JOptionPane.showMessageDialog(null,
				      lines,
				      "About CoffeeServer",
				      JOptionPane.INFORMATION_MESSAGE);
      }
  }

  public void load(CoffeeServerConfigInfo server_info)
    {
      m_info = server_info;
      FrameworkConfigPanel framework = m_frame.getFrameworkConfigPanel();
      framework.setNumThreads(m_info.getNumThreads());
      m_services.load(m_info);
    }

  public void store(CoffeeServerConfigInfo server_info)
    throws ConfigurationException
    {
      FrameworkConfigPanel framework = m_frame.getFrameworkConfigPanel();
      server_info.setNumThreads(framework.getNumThreads());
      m_services.store(server_info);
    }

  public static final void main(String[] args)
    {
      try
	{
	  ConfigFrame f = new ConfigFrame("Testing");
	  ServiceConfigInfo[] info = new ServiceConfigInfo[1];
	  info[0] = new ServiceConfigInfo();
	  info[0].setName("Hello");
	  info[0].setPort(10);
	  info[0].setClassName("org.cooldevtools.coffee_server.services.test.Hello");
     //BTB     info[0].setConfiguration("Initial message.".getBytes());

	  CoffeeServerConfigInfo config = new CoffeeServerConfigInfo();
	  config.setServices(info);

	  ConfigFrameDriver driver = new ConfigFrameDriver(f, "_filename");
	  driver.load(config);
	  f.show();
	}
      catch (Exception ex)
	{
	  ex.printStackTrace();
	}
    }

  private ConfigFrame m_frame;
  private String m_filename;
  private CoffeeServerConfigInfo m_info;
  private ServicesConfigPanelDriver m_services;
}
