///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//


package org.cooldevtools.coffee_server.config;

import java.awt.*;
import javax.swing.*;
import org.cooldevtools.awt.*;

public class ServicesConfigPanel
  extends JPanel
{
  public ServicesConfigPanel()
    {
      initDisplay();
    }

  private void initDisplay()
    {
      GridBagHelper gb = new GridBagHelper(this);

      gb.setPadding(5, 5);

      gb.setFill(GridBagConstraints.BOTH);
      gb.setWeights(1.0, 1.0);
      gb.setBounds(0, 0, 2, 3);
      m_list = new BasicJList();
      gb.addComponent(m_list.getScrollPane());

      m_fields = new BasicFieldPanel();
      m_name  = m_fields.addField(0, "Name: ", "", 30);
      m_port  = m_fields.addField(1, "Port: ", "", 30);
      m_class = m_fields.addField(2, "Class: ", "", 30);
      gb.setFill(GridBagConstraints.HORIZONTAL);
      gb.setWeights(0.0, 0.0);
      gb.setBounds(0, 3, 1, 1);
      gb.addComponent(m_fields);

      gb.setAnchor(GridBagConstraints.CENTER);
      gb.setBounds(1, 3, 1, 1);
      m_threadButtons = new BasicRadioPanel(3, 1);
      m_threadButtons.addButton("Single Threaded", true);
      m_threadButtons.addButton("Multi Threaded", false);
      m_threadButtons.addButton("Thread Pool", false);
      gb.addComponent(m_threadButtons);

      JPanel buttons = new JPanel();
      m_new = new JButton("New");
      buttons.add(m_new);

      m_remove = new JButton("Remove");
      buttons.add(m_remove);

      m_update = new JButton("Update");
      buttons.add(m_update);

      m_configure = new JButton("Configure");
      buttons.add(m_configure);

      gb.setAnchor(GridBagConstraints.CENTER);
      gb.setBounds(0, 4, 2, 1);
      gb.addComponent(buttons);
    }

  public static final void main(String[] args)
    {
      JFrame f = new JFrame("Testing");
      f.getContentPane().add(new ServicesConfigPanel());
      f.pack();
      f.show();
    }

  public JButton getNewButton()
    {
      return m_new;
    }

  public JButton getUpdateButton()
    {
      return m_update;
    }

  public JButton getConfigureButton()
    {
      return m_configure;
    }

  public JButton getRemoveButton()
    {
      return m_remove;
    }

  public BasicJList getServicesList()
    {
      return m_list;
    }

  public String getName()
    {
      return m_name.getText();
    }

  public void setName(String name)
    {
      m_name.setText(name == null ? "" : name);
    }

  public int getPort()
    {
      try
	{
	  return Integer.parseInt(m_port.getText());
	}
      catch (NumberFormatException ex)
	{
	  return 0;
	}
    }

  public void setPort(int port)
    {
      m_port.setText(String.valueOf(port));
    }

  public String getClassName()
    {
      return m_class.getText();
    }

  public void setClassName(String class_name)
    {
      m_class.setText(class_name == null ? "" : class_name);
    }

  public int getThreadModel()
    {
      return m_threadButtons.getSelectedIndex();
    }

  public void setThreadModel(int model)
    {
      m_threadButtons.setSelectedIndex(model);
    }

  public void enableConfigureButton(boolean yes_no)
    {
      m_configure.setEnabled(yes_no);
    }

  public void enableRemoveButton(boolean yes_no)
    {
      m_remove.setEnabled(yes_no);
    }

  public void enableUpdateButton(boolean yes_no)
    {
      m_update.setEnabled(yes_no);
    }

  public void allowInput(boolean yes_or_no)
    {
      m_fields.setEnabled(yes_or_no);
      m_threadButtons.setEnabled(yes_or_no);
    }

  private BasicJList m_list;
  private JTextField m_name;
  private JTextField m_port;
  private JTextField m_class;
  private JButton m_new;
  private JButton m_update;
  private JButton m_configure;
  private JButton m_remove;
  private BasicFieldPanel m_fields;
  private BasicRadioPanel m_threadButtons;

  private static final int SINGLE_THREADED = 0;
  private static final int MULTI_THREADED  = 1;
  private static final int THREAD_POOLED   = 2;
}
