///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//


package org.cooldevtools.coffee_server.config;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import org.cooldevtools.awt.*;
import org.cooldevtools.coffee_server.framework.*;

public class ServicesConfigPanelDriver
{
  public ServicesConfigPanelDriver(ServicesConfigPanel panel,
				   BeanPageDriver beaner)
    {
      m_panel    = panel;
      m_beaner   = beaner;
      m_list     = m_panel.getServicesList();
      m_tempInfo = new ServiceConfigInfo();
      m_services = new DefaultListModel();
      m_selectedIndex = INITIAL_INDEX;
      m_services = m_list.getContents();

      m_panel.getNewButton().addActionListener(new NewButtonListener());
      m_panel.getUpdateButton().addActionListener(new UpdateButtonListener());
      m_panel.getRemoveButton().addActionListener(new RemoveButtonListener());
      m_panel.getConfigureButton().addActionListener(new ConfigureButtonListener());

      m_list.addListSelectionListener(new SelectionListener());

      enableButtons(false);
    }

  private void enableButtons(boolean value)
    {
      m_panel.enableUpdateButton(value);
      m_panel.enableRemoveButton(value);
      m_panel.enableConfigureButton(value && m_tempInfo.hasBean());
      m_panel.allowInput(value);
    }

  private void selectService(int index)
    {
      if (index == m_selectedIndex)
	{
	  return;
	}

      m_selectedIndex = index;

      ServiceConfigInfo info = currentService();

      if (info == null)
	{
	  m_panel.setName("");
	  m_panel.setPort(0);
	  m_panel.setClassName("");
	  m_panel.setThreadModel(0);
	  enableButtons(false);
	}
      else
	{
	  m_tempInfo.copyFrom(info);
	  m_panel.setName(m_tempInfo.getName());
	  m_panel.setPort(m_tempInfo.getPort());
	  m_panel.setClassName(m_tempInfo.getClassName());
	  m_panel.setThreadModel(m_tempInfo.getThreadModel());
	  enableButtons(true);
	}
    }

  private ServiceConfigInfo currentService()
    {
      if (m_selectedIndex < 0)
	{
	  return null;
	}

      return (ServiceConfigInfo)m_services.elementAt(m_selectedIndex);
    }

  public void load(CoffeeServerConfigInfo server_info)
    {
      m_services.clear();
      ServiceConfigInfo[] services = server_info.getServices();
      for (int i = 0; i < services.length; i++)
	{
	  m_services.addElement(services[i]);
	}
      if (services.length > 0)
	{
	  m_list.setSelectedIndex(0);
	  enableButtons(true);
	}
      else
	{
	  enableButtons(false);
	}
    }

  public void store(CoffeeServerConfigInfo server_info)
    throws ConfigurationException
    {
      ServiceConfigInfo[] services = new ServiceConfigInfo[m_services.size()];
      m_services.copyInto(services);

      for (int i = 0; i < services.length; i++)
	{
	  StringBuffer message = new StringBuffer();
	  message.append("Service '");
	  message.append(services[i].getName());
	  message.append("': ");
	  if (!services[i].isValid(message))
	    {
	      throw new ConfigurationException(message.toString());
	    }
	}

      server_info.setServices(services);
    }

  public class NewButtonListener
    implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
      {
	m_list.addItem(new ServiceConfigInfo("Untitled"));
	enableButtons(true);
      }
  }

  public class RemoveButtonListener
    implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
      {
	if (m_selectedIndex >= 0)
	  {
	    m_list.removeItemAt(m_selectedIndex, true);
	    enableButtons(m_services.size() > 0);
	  }
      }
  }

  public class UpdateButtonListener
    implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
      {
	if (m_selectedIndex < 0)
	  {
	    return;
	  }

	ServiceConfigInfo info = currentService();

	try
	  {
	    m_tempInfo.setName(m_panel.getName());
	    m_tempInfo.setPort(m_panel.getPort());
	    m_tempInfo.setThreadModel(m_panel.getThreadModel());
	    m_tempInfo.setClassName(m_panel.getClassName());

	    StringBuffer message = new StringBuffer();
	    if (m_tempInfo.isValid(message))
	      {
		info.copyFrom(m_tempInfo);
		enableButtons(true);
	      }
	    else
	      {
		String[] lines = {
		  "Cannot update service '" + m_tempInfo.getName() + "': ",
		  message.toString()
		};
		JOptionPane.showMessageDialog(null,
					      lines,
					      "Error",
					      JOptionPane.ERROR_MESSAGE);
		m_tempInfo.copyFrom(info);
	      }
	    m_list.updateItemAt(m_selectedIndex);
	  }
	catch (Exception ex)
	  {
	    String[] message = {
	      "Cannot update service '" + m_tempInfo.getName() + "': ",
	      ex.getMessage()
	    };
	    JOptionPane.showMessageDialog(null,
					  message,
					  "Update Failed",
					  JOptionPane.ERROR_MESSAGE);
	    m_tempInfo.copyFrom(info);
	  }
      }
  }

  public class ConfigureButtonListener
    implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
      {
	m_beaner.execute(m_tempInfo);
      }
  }

  public class SelectionListener
    implements ListSelectionListener
  {
    public void valueChanged(ListSelectionEvent e)
      {
	selectService(m_list.getSelectedIndex());
      }
  }

  public static final void main(String[] args)
    {
      try
	{
	  ConfigFrame f = new ConfigFrame("Testing");
	  ServiceConfigInfo[] info = new ServiceConfigInfo[1];
	  info[0] = new ServiceConfigInfo();
	  info[0].setName("Hello");
	  info[0].setPort(10);
	  info[0].setClassName("org.cooldevtools.coffee_server.services.test.Hello");
//BTB	  info[0].setConfiguration("Initial message.".getBytes());

	  CoffeeServerConfigInfo config = new CoffeeServerConfigInfo();
	  config.setServices(info);

	  BeanPageDriver beaner = new BeanPageDriver(f);
	  ServicesConfigPanelDriver driver = new ServicesConfigPanelDriver(f.getServicesConfigPanel(),
									   beaner);
	  driver.load(config);
	  f.show();
	}
      catch (Exception ex)
	{
	  ex.printStackTrace();
	}
    }

  ServicesConfigPanel m_panel;
  BasicJList m_list;
  int m_selectedIndex;
  ServiceConfigInfo m_tempInfo;
  DefaultListModel m_services;
  BeanPageDriver m_beaner;

  // Chosen to be different than any possible value of JList index
  // to force configuration when selectService() first called.
  public static final int INITIAL_INDEX = -2;
}
