///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//


package org.cooldevtools.coffee_server.framework;

import java.awt.Component;
import org.cooldevtools.hdl.Statement;

/**
   Public interface to identify a bean that is used to configure
   a CoffeeService inside of the CoffeeServer config panel.
 */
public interface CoffeeServiceConfigBean
{
  /**
     Gives the bean an opportunity to validate its contents
     prior to the framework writing them to the config db.
     Bean can put any desired error text into message
     for display to end user when validation fails.
   */
  public boolean isValidConfiguration(StringBuffer message);

  /**
     Return state as configured in the bean.
     State must be stored in a byte array so it can be
     stored by framework.
   */
  public Statement getConfiguration();

  /**
     Initialize bean for the specified configuration.
     If config is null, go to default configuration.  Otherwise
     it is an array previously returned by getConfiguration().
   */
  public void setConfiguration(Statement config);

  /**
     Return the Component (could be bean itself or a helper class).
   */
  public Component getComponent();
}
