///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//


package org.cooldevtools.coffee_server.framework;

import java.io.*;
import java.net.*;

/**
   Subclass to invoke service with in a background thread.
 */
public class MultiThreadServiceAcceptor extends ServiceAcceptor
{
   public MultiThreadServiceAcceptor(CoffeeServer server,
                                     ServerSocket socket,
                                     CoffeeServiceManager manager,
                                     ExceptionHandler handler)
   {
      super(server, socket, manager, handler);
   }

   protected synchronized void callService(ClientRequest request,
                                           CoffeeService service)
   throws IOException, CoffeeServiceException
   {
      Thread t = new BackgroundThread(getExceptionHandler(), request, service);
      t.start();
   }

   private static class BackgroundThread extends Thread {
      BackgroundThread(ExceptionHandler handler,
                       ClientRequest request,
                       CoffeeService service)
      {
         m_request = new PendingRequest(handler, request, service);
      }

      public void run()
      {
         m_request.execute();
      }

      private PendingRequest m_request;
   }
}
