///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//


package org.cooldevtools.coffee_server.framework;

import java.io.*;
import java.net.*;

/**
   Class which accepts requests from client and invokes service
   on newly created service object.
 */
public abstract class ServiceAcceptor
extends Thread
{
   public ServiceAcceptor(CoffeeServer server,
                          ServerSocket socket,
                          CoffeeServiceManager manager,
                          ExceptionHandler handler)
   {
      setDaemon(true);
      m_server  = server;
      m_socket  = socket;
      m_manager = manager;
      m_handler = handler;
      m_stop    = false;
   }

   protected abstract void callService(ClientRequest request,
                                       CoffeeService service)
   throws IOException, CoffeeServiceException;

   protected ExceptionHandler getExceptionHandler()
   {
      return m_handler;
   }

   public synchronized void shutdown()
   throws CoffeeServiceException
   {
      m_stop = true;
   }

   public ServerSocket getServerSocket()
   {
      return m_socket;
   }

   private synchronized boolean shouldStop()
   {
      return m_stop;
   }

   public void run()
   {
      try {
         while (!shouldStop()) {
            Socket s = m_socket.accept();
            if (shouldStop()) {
               break;
            }

	    if (m_manager.getConfiguration() != null) {
	       int timeout = m_manager.getConfiguration().getSocketTimeout();
	       if (timeout > 0) {
		  try {
		     s.setSoTimeout(timeout);
		  } catch (IOException ex) {
		     // we can simply ignore timeout value change errors
		     // since they're harmless
		  }
	       }
	    }

            ClientRequest request = new SocketRequest(m_server, m_manager, s);
            CoffeeService service = m_manager.newServiceInstance();
            try {
               callService(request, service);
            } catch (Exception ex) {
               ex.printStackTrace();
               m_handler.serviceFailure(request, service, ex);
            }
         }
      } catch (Exception ex) {
         m_handler.fatalAcceptorException(this, ex);
      }
      try {
         m_socket.close();
      } catch (IOException ex) {
      }
   }

   private boolean              m_stop;
   private ServerSocket         m_socket;
   private CoffeeServiceManager m_manager;
   private ExceptionHandler     m_handler;
   private CoffeeServer         m_server;
}
