///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//


package org.cooldevtools.coffee_server.framework;

import java.io.*;
import java.net.*;

/**
   Class which accepts requests from client and invokes service
   on newly created service object.
 */
public class SocketRequest
implements ClientRequest
{
   public SocketRequest(CoffeeServer server,
                        CoffeeServiceManager manager,
                        Socket s)
   {
      m_server  = server;
      m_manager = manager;
      m_socket  = s;
   }

   public CoffeeServer getServer()
   {
      return m_server;
   }

   public CoffeeServiceManager getServiceManager()
   {
      return m_manager;
   }

   public InetAddress getInetAddress()
   {
      return m_socket.getInetAddress();
   }

   public InetAddress getLocalAddress()
   {
      return m_socket.getLocalAddress();
   }

   public int getPort()
   {
      return m_socket.getPort();
   }

   public int getLocalPort()
   {
      return m_socket.getLocalPort();
   }

   public void setTcpNoDelay(boolean on)
   throws SocketException
   {
      m_socket.setTcpNoDelay(on);
   }

   public boolean getTcpNoDelay()
   throws SocketException
   {
      return m_socket.getTcpNoDelay();
   }

   public void setSoLinger(boolean on,
                           int val)
   throws SocketException
   {
      m_socket.setSoLinger(on, val);
   }

   public int getSoLinger()
   throws SocketException
   {
      return m_socket.getSoLinger();
   }

   public void setSoTimeout(int timeout)
   throws SocketException
   {
      m_socket.setSoTimeout(timeout);
   }

   public int getSoTimeout()
   throws SocketException
   {
      return m_socket.getSoTimeout();
   }

   public InputStream getInputStream()
   throws IOException
   {
      return m_socket.getInputStream();
   }

   public OutputStream getOutputStream()
   throws IOException
   {
      return m_socket.getOutputStream();
   }

   public void close()
   throws IOException
   {
      m_socket.close();
   }

   private CoffeeServer m_server;
   private CoffeeServiceManager m_manager;
   private Socket m_socket;
}
