///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.coffee_server.services.nntp.protocol;

import java.io.*;
import java.util.*;
import org.cooldevtools.utility.SuperVector;
import org.cooldevtools.hdl.Token;
import org.cooldevtools.coffee_server.config.*;
import org.cooldevtools.coffee_server.framework.*;
import org.cooldevtools.coffee_server.services.nntp.pub.*;

public abstract class AbstractPropertySetter extends AbstractConfigCommand
{
   protected AbstractPropertySetter(int min,
				    int max,
				    String property,
				    String ok_response)
   {
      m_minValue = min;
      m_maxValue = max;
      m_propertyName = property;
      m_okResponse = ok_response;
   }

   protected boolean executeCommandImpl(NntpSession session,
                                        NntpReader in,
                                        NntpWriter out)
   throws IOException, CoffeeServiceException
   {
      try {
         int value = Integer.parseInt(in.getNextCommand());
         if (value < m_minValue || value > m_maxValue) {
            out.writeResponse(BCC_INVALID_COMMAND, "invalid integer value");
            return false;
         }

         session.getFactory().acquireWriteLock();
         try {
            ServiceConfigInfo config = session.getRequest().getServiceManager().getConfiguration();
            config.setProperty(m_propertyName, Token.intToken(value));
         } finally {
            session.getFactory().releaseWriteLock();
         }

         out.writeResponse(BCC_OK, m_okResponse);
         return true;
      } catch (NumberFormatException ex) {
         out.writeResponse(BCC_INVALID_COMMAND, "invalid integer value");
         return false;
      }
   }

   private int m_minValue;
   private int m_maxValue;
   private String m_propertyName;
   private String m_okResponse;
}
