///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//


package org.cooldevtools.coffee_server.services.nntp.protocol;

import java.util.*;
import org.cooldevtools.utility.logging.*;
import org.cooldevtools.coffee_server.framework.*;

/**
   Date parsing class to work around java.text.SimpleDateFormat
   bug in linux JDK 1.1.3 implementation.
 */
public class NntpDate
{
   public NntpDate(String s)
   throws CoffeeServiceException
   {
      m_date = parseDate(s);
   }

   public Date getDate()
   {
      return m_date;
   }

   private static String centuryFor(String s)
   {
      int year = Integer.parseInt(s.substring(0, 2));
      int this_century = Calendar.getInstance().get(Calendar.YEAR) / 100;
      int this_year    = Calendar.getInstance().get(Calendar.YEAR) % 100;
      int new_century = (year <= this_year) ? (this_century) : (this_century - 1);
      return String.valueOf(new_century);
   }

   private static String fixDate(String s,
                                 TimeZone tz)
   {
      String tzID = tz.getID();
      switch (s.length()) {
      case 13:
         return centuryFor(s) + s + " " + tzID;

      case 15:
         return s + " " + tzID;

      case 17:
         return centuryFor(s) + s;

      case 21:
         return s;

      default:
         Logger.s().logSilently(MessagePriority.LOG_WARNING,
                                "invalid date recieved from client ({0})",
				new Object[] { s });
         return null;
      }
   }

   public static Date parseDate(String s)
   throws CoffeeServiceException
   {
      Calendar cal = Calendar.getInstance();

      try {
         s = fixDate(s.trim(), cal.getTimeZone());

         int year   = Integer.parseInt(s.substring(0, 4));
         int month  = Integer.parseInt(s.substring(4, 6));
         int day    = Integer.parseInt(s.substring(6, 8));

         int hour   = Integer.parseInt(s.substring(9, 11));
         int minute = Integer.parseInt(s.substring(11, 13));
         int second = Integer.parseInt(s.substring(13, 15));

         String tzID = s.substring(16);

         cal.set(year, month - 1, day, hour, minute, second);
         cal.setTimeZone(TimeZone.getTimeZone(tzID));
      } catch (Exception ex) {
         throw new CoffeeServiceException("invalid date: " + s);
      }

      return cal.getTime();
   }

   private Date m_date;
}
