///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//


package org.cooldevtools.coffee_server.services.nntp.protocol;

import java.io.*;
import java.util.*;
import org.cooldevtools.utility.*;
import org.cooldevtools.coffee_server.framework.*;
import org.cooldevtools.coffee_server.services.nntp.pub.*;

public class NntpService_ARTICLE extends NntpCommand
{
   public NntpService_ARTICLE(int mode)
   {
      m_mode = mode;
   }

   private void writeArticle(NntpSession session,
                             NntpWriter out,
                             boolean include_header,
                             boolean include_body,
                             Message message)
   throws IOException, CoffeeServiceException
   {
      if (include_header) {
         out.writeEscapedText(message.getHeader());
         if (include_body) {
            out.writeEscapedText("");
         }
      }
      if (include_body) {
         out.writeEscapedText(message.getBody());
      }
      out.writeEndOfText();
   }

   private boolean executeCommandImpl(NntpSession session,
                                      NntpReader in,
                                      NntpWriter out)
   throws IOException, CoffeeServiceException
   {
      String number = in.getRemainderOfLine();

      long index = -1;
      Message message = null;
      if ((number.length() > 0) && (number.charAt(0) == ID_START)) {
         message = session.getMessages().findMessage(number);
      } else {
         NewsGroup group = session.getCurrentGroup();
         if (group == null) {
            out.writeResponse(412, "no news group selected");
            return true;
         }
         if (number.length() == 0) {
            index = session.getCurrentIndex();
            message = group.getMessage(index);
         } else {
            try {
               index = Long.parseLong(number);
               message = group.getMessage(index);
            } catch (NumberFormatException ex) {
            }
         }
      }

      if (message == null) {
         out.writeResponse(423, "no such article found");
         return true;
      }

      if (!session.getUser().canReadMessage(message)) {
         permissionDenied(session, out);
         return true;
      }

      StringBuffer buffer = new StringBuffer();
      if (index >= 0) {
         session.setCurrentIndex(index);
         buffer.append(index);
      } else {
         buffer.append(0);
      }

      buffer.append(" ");
      buffer.append(message.getField(Message.MESSAGE_ID));

      switch (m_mode) {
      case MODE_ARTICLE:
         buffer.append(" article retrieved - head and body follow");
         out.writeResponse(220, buffer.toString());
         writeArticle(session, out, true, true, message);
         break;

      case MODE_HEADER:
         buffer.append(" article retrieved - head follows");
         out.writeResponse(221, buffer.toString());
         writeArticle(session, out, true, false, message);
         break;

      case MODE_BODY:
         buffer.append(" article retrieved - body follows");
         out.writeResponse(222, buffer.toString());
         writeArticle(session, out, false, true, message);
         break;

      case MODE_STAT:
         buffer.append(" article retrieved - request text separately");
         out.writeResponse(223, buffer.toString());
         break;

      default:
         if (Assert.DEBUG) Assert.impossible();
         break;
      }

      return true;
   }

   public boolean executeCommand(NntpSession session,
                                 NntpReader in,
                                 NntpWriter out)
   throws IOException, CoffeeServiceException
   {
      session.getFactory().acquireReadLock();
      try {
         return executeCommandImpl(session, in, out);
      } finally {
         session.getFactory().releaseReadLock();
      }
   }

   private int m_mode;

   public static final int MODE_ARTICLE = 1;
   public static final int MODE_HEADER  = 2;
   public static final int MODE_BODY    = 3;
   public static final int MODE_STAT    = 4;

   private static final char ID_START = '<';
}
