///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//


package org.cooldevtools.coffee_server.services.nntp.protocol;

import java.io.*;
import java.util.*;
import org.cooldevtools.coffee_server.framework.*;
import org.cooldevtools.coffee_server.services.nntp.pub.*;
import org.cooldevtools.utility.logging.*;

public class NntpService_AUTHINFO extends NntpCommand
{
   public NntpService_AUTHINFO()
   {
      super(false);
   }

   public boolean executeCommand(NntpSession session,
                                 NntpReader in,
                                 NntpWriter out)
   throws IOException, CoffeeServiceException
   {
      String arg = in.getNextCommand();
      if (!arg.equals(USER)) {
         out.writeResponse(ServerConnection.CODE_AUTH_SYNTAX, "command syntax error");
         return true;
      }

      String username = in.getRemainderOfLine();
      out.writeResponse(ServerConnection.CODE_AUTH_SEND_PASSWORD, "send password");
      out.flush();

      in.readNewLine();
      arg = in.getNextCommand();
      if (!arg.equals(AUTHINFO)) {
         out.writeResponse(ServerConnection.CODE_AUTH_SYNTAX,
			   "command syntax error: expecting AUTHINFO PASS");
         return true;
      }

      arg = in.getNextCommand();
      if (!arg.equals(PASS)) {
	 out.writeResponse(ServerConnection.CODE_AUTH_SYNTAX, "error: expecting AUTHINFO PASS");
         return true;
      }

      String password = in.getRemainderOfLine();
      try {
         User user = session.getUsers().getUser(username, password);
         session.setUser(user);
	 session.setAuthenticated(true);
         out.writeResponse(ServerConnection.CODE_AUTH_ACCEPTED, "authorization accepted");
         Logger.s().logSilently(MessagePriority.LOG_INFO, "authorization accepted for user {0}",
				new Object[] { user.getSenderAddress() });
      } catch (InvalidUserOrPasswordException ex) {
         out.writeResponse(ServerConnection.CODE_AUTH_REJECTED, "authorization rejected");
         Logger.s().logSilently(MessagePriority.LOG_WARNING, "authorization rejected for user {0}",
				new Object[] { session.getUser().getSenderAddress() });
      }

      return true;
   }

   private static final String AUTHINFO = "authinfo";
   private static final String USER     = "user";
   private static final String PASS     = "pass";
}
