///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//


package org.cooldevtools.coffee_server.services.nntp.protocol;

import java.io.*;
import java.util.*;
import org.cooldevtools.utility.SuperVector;
import org.cooldevtools.utility.logging.*;
import org.cooldevtools.coffee_server.framework.*;
import org.cooldevtools.coffee_server.services.nntp.pub.*;

public class NntpService_BCC_DEFINE_GROUP extends NntpCommand
{
   public NntpService_BCC_DEFINE_GROUP()
   {
   }

   public static boolean canCreateNewsGroup(NntpSession session,
                                            User user,
                                            String group_name,
                                            boolean is_create)
   throws CoffeeServiceException
   {
      if (!user.canCreateNewsGroup(group_name)) {
         Logger.s().logSilently(MessagePriority.LOG_ERROR,
                                "user {0} not authoized to create news groups",
                                new Object[] { user.getUsername() });
         return false;
      }

      return true;
   }

   public boolean executeCommand(NntpSession session,
                                 NntpReader in,
                                 NntpWriter out)
   throws IOException, CoffeeServiceException
   {
      String group_name  = in.getNextCommand();
      String email       = "";
      String description = "";

      while (true) {
         in.readNewLine();
         if (in.getEntireLine().equals(NntpReader.MULTI_LINE_TERM)) {
            break;
         }

         String command = in.getNextCommand();
         if (command.equals(CMD_DESCRIPTION)) {
            description = in.getRemainderOfLine();
         } else if (command.equals(CMD_EMAIL)) {
            email = in.getRemainderOfLine();
         } else {
            in.readMultipleLines();
            out.writeResponse(BCC_INVALID_COMMAND, "invalid command: " + command);
            return true;
         }
      }

      NewsGroup group = session.getNewsGroups().findNewsGroup(group_name);

      User user = session.getUser();
      if (!canCreateNewsGroup(session, user, group_name, (group == null))) {
         out.writeResponse(BCC_PERMISSION_DENIED, "permission denied");
         return true;
      }

      if (group != null) {
         group.changeAttributes(group_name, email, description);
      } else {
         session.getNewsGroups().createNewsGroup(user, group_name, email, description);
      }

      out.writeResponse(BCC_OK, "operation successful");
      return true;
   }

   private static final String CMD_DESCRIPTION = "description";
   private static final String CMD_EMAIL       = "email";
}
