///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//


package org.cooldevtools.coffee_server.services.nntp.protocol;

import java.io.*;
import java.util.*;
import org.cooldevtools.utility.SuperVector;
import org.cooldevtools.coffee_server.framework.*;
import org.cooldevtools.coffee_server.services.nntp.pub.*;

public class NntpService_BCC_DEFINE_USER extends NntpCommand
{
   public NntpService_BCC_DEFINE_USER()
   {
   }

   public boolean executeCommand(NntpSession session,
                                 NntpReader in,
                                 NntpWriter out)
   throws IOException, CoffeeServiceException
   {
      String user_name = in.getRemainderOfLine();

      String password   = "";
      String email      = "";
      SuperVector roles = new SuperVector();

      while (true) {
         in.readNewLine();
         if (in.getEntireLine().equals(NntpReader.MULTI_LINE_TERM)) {
            break;
         }

         String command = in.getNextCommand();
         if (command.equals(CMD_PASSWORD)) {
            password = in.getRemainderOfLine();
         } else if (command.equals(CMD_EMAIL)) {
            email = in.getRemainderOfLine();
         } else if (command.equals(CMD_ROLE)) {
            roles.addElement(in.getRemainderOfLine());
         } else {
            in.readMultipleLines();
            out.writeResponse(BCC_INVALID_COMMAND, "invalid command: " + command);
            return true;
         }
      }

      if (user_name.length() == 0) {
         out.writeResponse(BCC_INVALID_COMMAND, "name required");
         return true;
      }

      if (roles.size() == 0) {
         out.writeResponse(BCC_INVALID_COMMAND, "ROLE required");
         return true;
      }

      if (!session.getUser().canDefineUser(user_name)) {
         out.writeResponse(BCC_PERMISSION_DENIED, "permission denied");
         return true;
      }
      
      String[] role_names = roles.getStrings();
      if (!session.getUser().canUseRoles(role_names)) {
         out.writeResponse(BCC_PERMISSION_DENIED, "permission denied");
         return true;
      }

      session.getUsers().defineUser(user_name,
                                    password,
                                    email,
                                    role_names);

      out.writeResponse(BCC_OK, "operation successful");
      return true;
   }

   private static final String CMD_PASSWORD = "password";
   private static final String CMD_EMAIL    = "email";
   private static final String CMD_ROLE     = "role";
}
