///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.coffee_server.services.nntp.protocol;

import java.io.*;
import java.util.*;
import org.cooldevtools.utility.SuperVector;
import org.cooldevtools.coffee_server.framework.*;
import org.cooldevtools.coffee_server.services.nntp.pub.*;

public class NntpService_BCC_SET_MAX_CONNECT
extends AbstractConfigCommand
{
   protected boolean executeCommandImpl(NntpSession session,
                                        NntpReader in,
                                        NntpWriter out)
   throws IOException, CoffeeServiceException
   {
      try {
         int num_threads = Integer.parseInt(in.getNextArgument());
         if (num_threads < MIN_THREADS || num_threads > MAX_THREADS) {
            out.writeResponse(BCC_INVALID_COMMAND, "invalid integer value");
            return false;
         }

         session.getFactory().acquireWriteLock();
         try {
            session.getRequest().getServer().getConfig().setNumThreads(num_threads);
         } finally {
            session.getFactory().releaseWriteLock();
         }
         out.writeResponse(BCC_OK, "max connections changed");
         return true;
      } catch (NumberFormatException ex) {
         out.writeResponse(BCC_INVALID_COMMAND, "invalid integer value");
         return false;
      }
   }

   private static final int MIN_THREADS = 1;
   private static final int MAX_THREADS = 1000;
}
