///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.coffee_server.services.nntp.protocol;

import java.io.*;
import java.net.*;
import java.util.*;
import org.cooldevtools.utility.SuperVector;
import org.cooldevtools.coffee_server.framework.*;
import org.cooldevtools.coffee_server.config.*;
import org.cooldevtools.coffee_server.services.nntp.pub.*;

public class NntpService_BCC_SET_PORTS
extends AbstractConfigCommand
{
   protected boolean executeCommandImpl(NntpSession session,
                                        NntpReader in,
                                        NntpWriter out)
   throws IOException, CoffeeServiceException
   {
      try {
         String primary_port_str   = in.getNextArgument();
         String alternate_port_str = in.getNextArgument();
	 String address_str        = in.getNextArgument();

         if (alternate_port_str.length() == 0) {
            alternate_port_str = primary_port_str;
         }
         
         int primary_port = Integer.parseInt(primary_port_str);
         int alternate_port = Integer.parseInt(alternate_port_str);

         session.getFactory().acquireWriteLock();
         try {
            ServiceConfigInfo config = session.getRequest().getServiceManager().getConfiguration();
            config.setPort(primary_port);
            config.setAlternatePort(alternate_port);
	    config.setAddress(address_str);
         } finally {
            session.getFactory().releaseWriteLock();
         }

         out.writeResponse(BCC_OK, "ports changed");
         return true;
      } catch (NumberFormatException ex) {
         out.writeResponse(BCC_INVALID_COMMAND, "invalid integer value");
         return false;
      } catch (UnknownHostException ex) {
         out.writeResponse(BCC_INVALID_COMMAND, "invalid hostname/address");
         return false;
      }
   }
}
