///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//


package org.cooldevtools.coffee_server.services.nntp.protocol;

import java.io.*;
import java.util.*;
import org.cooldevtools.coffee_server.framework.*;
import org.cooldevtools.coffee_server.services.nntp.pub.*;

public class NntpService_LAST extends NntpCommand
{
   public NntpService_LAST()
   {
      m_noArticleCode = 422;
      m_noArticleMessage = "no previous article in this group";
   }

   public boolean executeCommand(NntpSession session,
                                 NntpReader in,
                                 NntpWriter out)
   throws IOException, CoffeeServiceException
   {
      NewsGroup group = session.getCurrentGroup();

      if (group == null) {
         out.writeResponse(412, "no news group selected");
         return true;
      }

      long index = findIndex(session, group);
      if (index < 0) {
         out.writeResponse(m_noArticleCode, m_noArticleMessage);
         return true;
      }

      Message message = group.getMessage(index);
      if (message == null) {
         // NOTE: THIS SHOULD NEVER HAPPEN!!
         // For some reason the group is lying about it's message index
         // or we've hit an index at which the message was deleted.
         // This response is not strictly correct, but at least the
         // news client can understand it.
         out.writeResponse(m_noArticleCode, m_noArticleMessage);
         return true;
      }

      if (!session.getUser().canReadMessage(message)) {
	 permissionDenied(session, out);
	 return true;
      }

      // wait to set the index until we know we've found a message with it!
      session.setCurrentIndex(index);

      StringBuffer buffer = new StringBuffer();
      buffer.append(index);
      buffer.append(" ");
      buffer.append(message.getField(Message.MESSAGE_ID));
      buffer.append(" article retrieved - request text separately");
      out.writeResponse(223, buffer.toString());
      return true;
   }

   /**
      NEXT command overrides this to search in other direction.
    */
   protected long findIndex(NntpSession session,
                            NewsGroup group)
   throws CoffeeServiceException
   {
      long index = session.getCurrentIndex();
      for (index--; index >= group.getLowMessageIndex(); index--) {
         if (group.isValidMessageIndex(index)) {
            return index;
         }
      }
      return -1;
   }

   protected int m_noArticleCode;
   protected String m_noArticleMessage;
}
