///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//


package org.cooldevtools.coffee_server.services.nntp.protocol;

import java.io.*;
import java.util.*;
import org.cooldevtools.coffee_server.framework.*;
import org.cooldevtools.coffee_server.services.nntp.pub.*;

public class NntpService_LIST extends NntpCommand
{
   public NntpService_LIST()
   {
   }

   private boolean includeGroup(WildMat wildmat,
                                NewsGroup group,
                                User user)
   throws CoffeeServiceException
   {
      return (((wildmat == null) || wildmat.match(group.getName())) &&
	      (LIST_ALL_GROUPS || user.canReadNewsGroup(group)));
   }

   public boolean active(NntpSession session,
                         NntpReader in,
                         NntpWriter out)
   throws IOException, CoffeeServiceException
   {
      String arg = in.getNextCommand();
      if (arg.length() == 0) {
         return list(session, in, out, null);
      } else {
         return list(session, in, out, new WildMat(arg));
      }
   }

   public boolean list(NntpSession session,
                       NntpReader in,
                       NntpWriter out,
                       WildMat wildmat)
   throws IOException, CoffeeServiceException
   {
      User user = session.getUser();
      NewsGroup[] groups = session.getNewsGroups().getNewsGroups();

      out.writeResponse(215, "list of newsgroups follows");

      for (int i = 0; i < groups.length; i++) {
         NewsGroup g = groups[i];
         if (includeGroup(wildmat, g, user)) {
            StringBuffer buffer = new StringBuffer();
            buffer.append(g.getName());
            buffer.append(" ");
            buffer.append(g.getHighMessageIndex());
            buffer.append(" ");
            buffer.append(g.getLowMessageIndex());
            buffer.append(user.canPostToNewsGroup(g) ? " y" : " n");
            out.writeEscapedText(buffer.toString());
         }
      }

      out.writeEndOfText();

      return true;
   }

   public boolean active_times(NntpSession session,
                               NntpReader in,
                               NntpWriter out)
   throws IOException, CoffeeServiceException
   {
      User user = session.getUser();
      NewsGroup[] groups = session.getNewsGroups().getNewsGroups();

      WildMat wildmat = null;
      String arg = in.getNextCommand();
      if (arg.length() > 0) {
         wildmat = new WildMat(arg);
      }

      out.writeResponse(215, "list of newsgroups follows");

      for (int i = 0; i < groups.length; i++) {
         NewsGroup g = groups[i];
         if (includeGroup(wildmat, g, user)) {
            StringBuffer buffer = new StringBuffer();
            buffer.append(g.getName());
            buffer.append(" ");
            buffer.append(g.getAgeInSeconds());
            buffer.append(" ");
            buffer.append(g.getEmailAddress());
            out.writeEscapedText(buffer.toString());
         }
      }

      out.writeEndOfText();

      return true;
   }

   public boolean newsgroups(NntpSession session,
                             NntpReader in,
                             NntpWriter out)
   throws IOException, CoffeeServiceException
   {
      User user = session.getUser();
      NewsGroup[] groups = session.getNewsGroups().getNewsGroups();

      WildMat wildmat = null;
      String arg = in.getNextCommand();
      if (arg.length() > 0) {
         wildmat = new WildMat(arg);
      }

      out.writeResponse(215, "list of newsgroups follows");

      for (int i = 0; i < groups.length; i++) {
         NewsGroup g = groups[i];
         if (includeGroup(wildmat, g, user)) {
            StringBuffer buffer = new StringBuffer();
            buffer.append(g.getName());
            buffer.append(" ");
            buffer.append(g.getDescription());
            out.writeEscapedText(buffer.toString());
         }
      }

      out.writeEndOfText();

      return true;
   }

   public boolean extensions(NntpSession session,
                             NntpReader in,
                             NntpWriter out)
   throws IOException, CoffeeServiceException
   {
      out.writeResponse(202, "list of extensions follows");
      out.writeEscapedText(" LISTGROUP");
      out.writeEscapedText(" AUTHINFO");
      out.writeEscapedText(" OVER");
      out.writeEscapedText(" PAT");
      out.writeEndOfText();
      return true;
   }

   public boolean distributions(NntpSession session,
                                NntpReader in,
                                NntpWriter out)
   throws IOException, CoffeeServiceException
   {
      out.writeResponse(215, "information follows");
      out.writeEscapedText("local:To this news server only.");
      out.writeEndOfText();
      return true;
   }

   public boolean distrib_pats(NntpSession session,
                               NntpReader in,
                               NntpWriter out)
   throws IOException, CoffeeServiceException
   {
      out.writeResponse(215, "information follows");
      out.writeEscapedText("1:*:local");
      out.writeEndOfText();
      return true;
   }

   public boolean overview_fmt(NntpSession session,
                               NntpReader in,
                               NntpWriter out)
   throws IOException, CoffeeServiceException
   {
      out.writeResponse(215, "information follows");
      String[] fields = Message.OVERVIEW_FIELDS_LIST;
      for (int i = 0; i < fields.length; i++) {
         out.writeEscapedText(fields[i]);
      }
      out.writeEndOfText();
      return true;
   }

   public boolean executeCommand(NntpSession session,
                                 NntpReader in,
                                 NntpWriter out)
   throws IOException, CoffeeServiceException
   {
      String arg = in.getNextCommand();

      if (arg.length() == 0) {
         return list(session, in, out, null);
      }

      if (arg.equals(ACTIVE)) {
         return active(session, in, out);
      }

      if (arg.equals(ACTIVE_TIMES)) {
         return active_times(session, in, out);
      }

      if (arg.equals(DISTRIB_PATS)) {
         return distrib_pats(session, in, out);
      }

      if (arg.equals(DISTRIBUTIONS)) {
         return distributions(session, in, out);
      }

      if (arg.equals(EXTENSIONS)) {
         return extensions(session, in, out);
      }

      if (arg.equals(NEWSGROUPS)) {
         return newsgroups(session, in, out);
      }

      if (arg.equals(OVERVIEW_FMT)) {
         return overview_fmt(session, in, out);
      }

      if (arg.equals(SUBSCRIPTIONS)) {
         return list(session, in, out, null);
      }

      return false;
   }

   private static final String ACTIVE        = "active";
   private static final String ACTIVE_TIMES  = "active.times";
   private static final String DISTRIBUTIONS = "distributions";
   private static final String DISTRIB_PATS  = "distrib.pats";
   private static final String EXTENSIONS    = "extensions";
   private static final String NEWSGROUPS    = "newsgroups";
   private static final String OVERVIEW_FMT  = "overview.fmt";
   private static final String SUBSCRIPTIONS = "subscriptions";

   private static final boolean LIST_ALL_GROUPS = true;
}
