///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//


package org.cooldevtools.coffee_server.services.nntp.protocol;

import java.io.*;
import java.util.*;
import org.cooldevtools.utility.*;
import org.cooldevtools.coffee_server.framework.*;
import org.cooldevtools.coffee_server.services.nntp.pub.*;

public class NntpService_LISTGROUP extends NntpCommand
{
   public NntpService_LISTGROUP()
   {
   }

   public boolean executeCommand(NntpSession session,
                                 NntpReader in,
                                 NntpWriter out)
   throws IOException, CoffeeServiceException
   {
      NewsGroup group = session.getCurrentGroup();

      String group_name = in.getNextCommand();
      if (group_name.length() > 0) {
         group = session.getNewsGroups().findNewsGroup(group_name);
	 if (group == null) {
	    out.writeResponse(411, "No such news group");
	    return true;
	 } else if (!session.getUser().canReadNewsGroup(group)) {
	    permissionDenied(session, out);
	    return true;
	 }
      } else {
         group = session.getCurrentGroup();
         if (group == null) {
            out.writeResponse(412, "no news group selected");
            return true;
	 } else if (!session.getUser().canReadNewsGroup(group)) {
	    permissionDenied(session, out);
	    return true;
	 }
      }

      out.writeResponse(211, "articles follow");

      long[] indexes = new long[MAX_MESSAGES_PER_CALL];
      int num_returned = 1;
      do {
         long start_index = indexes[num_returned-1] + 1;
         num_returned = group.listGroup(start_index, indexes, indexes.length);
         for (int i = 0; i < num_returned; i++) {
            out.writeEscapedText(String.valueOf(indexes[i]));
         }
      } while (num_returned == indexes.length);

      out.writeEndOfText();

      return true;
   }

   private static final int MAX_MESSAGES_PER_CALL = 500;
}
