///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//


package org.cooldevtools.coffee_server.services.nntp.protocol;

import java.io.*;
import java.util.*;
import java.text.*;
import org.cooldevtools.coffee_server.framework.*;
import org.cooldevtools.coffee_server.services.nntp.pub.*;

public class NntpService_NEWNEWS extends NntpCommand {
   public NntpService_NEWNEWS()
   {
   }

   private boolean includeGroup(WildMat wildmat,
                                NewsGroup group,
                                User user)
   throws CoffeeServiceException
   {
      return (wildmat.match(group.getName()) && user.canReadNewsGroup(group));
   }

   public boolean list(NntpSession session,
                       NntpReader in,
                       NntpWriter out,
                       WildMat wildmat,
                       Date when)
   throws IOException, CoffeeServiceException
   {
      User user = session.getUser();
      Message[] messages = session.getMessages().getMessagesNewerThan(when);

      out.writeResponse(230, "list of new messages follows");

      for (int i = 0; i < messages.length; i++) {
         NewsGroup[] groups = messages[i].getNewsGroups();
         for (int k = 0; k < groups.length; k++) {
            if (includeGroup(wildmat, groups[k], user)) {
               out.writeEscapedText(messages[i].messageId());
               break;
            }
         }
         if ((i % 300) == 0) {
            Thread.yield();
         }
      }

      out.writeEndOfText();

      return true;
   }


   public boolean executeCommand(NntpSession session,
                                 NntpReader in,
                                 NntpWriter out)
   throws IOException, CoffeeServiceException
   {
      String pattern = in.getNextCommand();
      WildMat wildmat = new WildMat(pattern);

      String arg = in.getRemainderOfLine();
      if (arg.length() == 0) {
         return false;
      }

      Date when = NntpDate.parseDate(arg);
      return list(session, in, out, wildmat, when);
   }
}
