///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//


package org.cooldevtools.coffee_server.services.nntp.protocol;

import java.io.*;
import java.util.*;
import org.cooldevtools.coffee_server.framework.*;
import org.cooldevtools.coffee_server.services.nntp.pub.*;

public class NntpService_PAT extends NntpCommand
{
   public NntpService_PAT()
   {
   }

   private void matchAndWrite(String s,
                              NntpWriter out,
                              WildMat wildmat)
   throws IOException
   {
      if ((wildmat == null) || wildmat.match(s)) {
         out.writeText(s);
      }
   }

   public boolean executeCommand(NntpSession session,
                                 NntpReader in,
                                 NntpWriter out)
   throws IOException, CoffeeServiceException
   {
      String field_name = in.getNextCommand();
      if (field_name.length() == 0) {
         out.writeResponse(500, "header name required");
         return true;
      }

      String range_str = in.getNextCommand();

      String pattern = in.getRemainderOfLine();
      WildMat wildmat = null;
      if (pattern.length() > 0) {
         wildmat = new WildMat(pattern);
      }

      if (Message.isMessageID(range_str)) {
         Message message = session.getMessages().findMessage(range_str);
         if (message == null) {
            out.writeResponse(430, "no such article");
	    return true;
         }

	 if (!session.getUser().canReadMessage(message)) {
	    permissionDenied(session, out);
	    return true;
	 }

	 out.writeResponse(221, "overview information follows");
	 matchAndWrite(message.getFieldAlways(field_name), out, wildmat);
	 out.writeEndOfText();
      } else {
         NewsGroup group = session.getCurrentGroup();
         if (group == null) {
            out.writeResponse(412, "no news group selected");
            return true;
         }

	 if (!session.getUser().canReadNewsGroup(group)) {
	    permissionDenied(session, out);
	    return true;
	 }

         try {
            out.writeResponse(221, "overview information follows");

            ArticleRange range = new ArticleRange(session, range_str);
            long first = range.first();
            long last = range.last();

            for (long index = first; index <= last; index++) {
               Message message = group.getMessage(index);
               if (message != null) {
                  matchAndWrite(message.getFieldAlways(field_name), out, wildmat);
               }
            }
            out.writeEndOfText();
         } catch (NumberFormatException ex) {
            out.writeResponse(501, "invalid range specification");
         }
      }

      return true;
   }
}
