///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//


package org.cooldevtools.coffee_server.services.nntp.protocol;

import java.io.*;
import java.util.*;
import org.cooldevtools.coffee_server.framework.*;
import org.cooldevtools.coffee_server.services.nntp.pub.*;

public class NntpService_XOVER extends NntpCommand
{
   public NntpService_XOVER()
   {
   }

   private long maxToReturn(long first,
                            long last)
   {
      long remaining = last - first + 1;
      if (remaining > MAX_MESSAGES_PER_CALL) {
         return MAX_MESSAGES_PER_CALL;
      } else {
         return remaining;
      }
   }

   private void generateOverview(NewsGroup group,
                                 long first,
                                 long last,
                                 NntpWriter out)
   throws IOException, CoffeeServiceException
   {
      int total_returned = 0;

      try {
	 long[] indexes = new long[MAX_MESSAGES_PER_CALL];
	 indexes[0] = first - 1;
	 int num_returned = 1;
	 do {
	    long start_index = indexes[num_returned-1] + 1;
	    long max_to_return = maxToReturn(start_index, last);
	    num_returned = group.listGroup(start_index, indexes, max_to_return);
	    for (int i = 0; i < num_returned; i++) {
	       long index = indexes[i];
	       if (index > last) {
		  return;
	       }
	       String overview = group.getMessageOverview(index);
	       if (overview != null) {
		  if (total_returned == 0) {
		     out.writeResponse(224, "overview information follows");
		  }
		  out.writeEscapedText(String.valueOf(index) + "\t" + overview);
		  total_returned++;
	       }
	    }
	 } while (num_returned == indexes.length);
      } finally {
	 if (total_returned > 0) {
	    out.writeEndOfText();
	 } else {
	    out.writeResponse(420, "no articles in range");
	 }
      }
   }

   public boolean executeCommand(NntpSession session,
                                 NntpReader in,
                                 NntpWriter out)
   throws IOException, CoffeeServiceException
   {
      NewsGroup group = session.getCurrentGroup();

      if (group == null) {
         out.writeResponse(412, "no news group selected");
         return true;
      }

      if (!session.getUser().canReadNewsGroup(group)) {
	 permissionDenied(session, out);
	 return true;
      }

      String range_str = in.getNextCommand();

      try {
         ArticleRange range = new ArticleRange(session, range_str);
         long first = range.first();
         long last = range.last();

         generateOverview(group, first, last, out);
      } catch (NumberFormatException ex) {
         out.writeResponse(501, "invalid range specification");
      }

      return true;
   }

   private static final int MAX_MESSAGES_PER_CALL = 500;
}
