///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//


package org.cooldevtools.coffee_server.services.nntp.protocol;

import java.io.*;
import java.util.Vector;

public class WildMat
{
  public WildMat(String expression)
    {
      compile(expression);
    }

  public synchronized void compile(String expression)
    {
      m_expression = expression;
      setChars(expression);

      Vector wildmats = new Vector();

      int char_index = 0;
      while (char_index < m_chars.length)
	{
	  char_index = compile(char_index, wildmats, false);
	}

      m_wildmats = new WildMatMatch[ wildmats.size() ];
      wildmats.copyInto(m_wildmats);
      m_chars = null;
    }

  private int compile(int index,
		      Vector wildmats,
		      boolean wildcard)
    {
      char ch = m_chars[index];
      switch (ch)
	{
	case '*':
	  return compile(index + 1, wildmats, true);

	case '?':
	  wildmats.addElement(WildMatMatch.createMatch(wildcard));
	  return index + 1;

	case '\\':
	  wildmats.addElement(WildMatMatch.createMatch(m_chars[index + 1], wildcard));
	  return index + 2;

	case '[':
	  return compileChoice(index + 1, wildmats, wildcard);

	default:
	  wildmats.addElement(WildMatMatch.createMatch(ch, wildcard));
	  return index + 1;
	}
    }

  private int compileChoice(int index,
			    Vector wildmats,
			    boolean wildcard)
    {
      boolean invert = false;
      Vector choices = new Vector();

      if (m_chars[index] == '^')
	{
	  invert = true;
	  index++;
	}

      if (m_chars[index] == ']')
	{
	  choices.addElement(WildMatMatch.createMatch(']', false));
	  index++;
	}

      if (m_chars[index] == '-')
	{
	  choices.addElement(WildMatMatch.createMatch('-', false));
	  index++;
	}

      while (m_chars[index] != ']')
	{
	  if ((m_chars[index + 1] == '-') && (m_chars[index + 2] != ']'))
	    {
	      choices.addElement(WildMatMatch.createMatch(m_chars[index],
							  m_chars[index + 2],
							  false));
	      index += 3;
	    }
	  else
	    {
	      choices.addElement(WildMatMatch.createMatch(m_chars[index], false));
	      index++;
	    }
	}

      wildmats.addElement(WildMatMatch.createMatch(invert, choices, wildcard));
      return index + 1;
    }

  private void setChars(String s)
    {
      m_chars = new char[s.length() + 1];
      s.getChars(0, s.length(), m_chars, 0);
      m_chars[m_chars.length - 1] = WildMatMatch.EOS;
    }

  private boolean match_normal(int match_index,
			       int char_index)
    {
      if (DEBUG)
	{
	  System.out.println("normal trying match on '" + m_chars[char_index] + "'");
	}
      return m_wildmats[match_index].match(m_chars[char_index]);
    }

  private boolean match_wildcard(int match_index,
				 int char_index)
    {
      WildMatMatch wildcard = m_wildmats[match_index];

      while (char_index < m_chars.length)
	{
	  if (DEBUG)
	    {
	      System.out.println("wildcard trying match on '" + m_chars[char_index] + "'");
	    }
	  if (wildcard.match(m_chars[char_index]))
	    {
	      if (DEBUG)
		{
		  System.out.println("match found trying next wildmat...");
		}
	      if (match(match_index + 1, char_index + 1))
		{
		  if (DEBUG)
		    {
		      status("wild card successful", match_index, char_index);
		    }
		  return true;
		}
	      if (DEBUG)
		{
		  System.out.println("wildcard failed!");
		}
	    }

	  char_index++;
	}

      return false;
    }

  private void status(String message,
		      int match_index,
		      int char_index)
    {
      System.out.println(message);

      System.out.print("match_index : ");
      System.out.print(match_index);
      System.out.print(" of ");
      System.out.println(m_wildmats.length);

      System.out.print("char_index : ");
      System.out.print(char_index);
      System.out.print(" of ");
      System.out.println(m_chars.length);
    }

  private boolean match(int match_index,
			int char_index)
    {
      while (true)
	{
	  if ((match_index == m_wildmats.length) || (char_index == m_chars.length))
	    {
	      if (DEBUG)
		{
		  status("at end", match_index, char_index);
		}
	      return (match_index >= m_wildmats.length) && (char_index >= m_chars.length);
	    }

	  if (m_wildmats[match_index].isWildcard())
	    {
	      return match_wildcard(match_index, char_index);
	    }

	  if (!match_normal(match_index, char_index))
	    {
	      return false;
	    }

	  match_index++;
	  char_index++;
	}
    }

  public synchronized boolean match(String s)
    {
      try
	{
	  if (DEBUG)
	    {
	      System.out.println("NEW SEARCH: '" + s + "'");
	    }
	  setChars(s);
	  return match(0, 0);
	}
      finally
	{
	  m_chars = null;
	}
    }

  public static void main(String[] args)
    {
      if (args.length != 1)
	{
	  System.err.println("usage: <class> expression");
	  System.exit(1);
	}

      try
	{
	  WildMat wildmat = new WildMat(args[0]);

	  BufferedReader in = new BufferedReader(new InputStreamReader(System.in));

	  for (String s = in.readLine(); s != null; s = in.readLine())
	    {
	      if (wildmat.match(s))
		{
		  System.out.println(s);
		}
	    }
	}
      catch (Exception ex)
	{
	  ex.printStackTrace();
	}
    }

   public String getExpression()
   {
      return m_expression;
   }

  private String m_expression;
  private WildMatMatch[] m_wildmats;
  private char[] m_chars;

  private static final boolean DEBUG = false;
}
