///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//


package org.cooldevtools.coffee_server.services.nntp.protocol;

import java.util.Vector;

public abstract class WildMatMatch
{
  public static WildMatMatch createMatch(char ch,
					 boolean wildcard)
    {
      return new MatchChar(ch, wildcard);
    }

  public static WildMatMatch createMatch(char first,
					 char last,
					 boolean wildcard)
    {
      return new MatchRange(first, last, wildcard);
    }

  public static WildMatMatch createMatch(boolean wildcard)
    {
      return new MatchAny(wildcard);
    }

  public static WildMatMatch createMatch(boolean invert,
					 Vector matches,
					 boolean wildcard)
    {
      return new MatchMulti(invert, matches, wildcard);
    }

  protected WildMatMatch(boolean wildcard)
    {
      m_isWildcard = wildcard;
    }

  public abstract boolean match(char ch);

  public boolean isWildcard()
    {
      return m_isWildcard;
    }

  private static class MatchRange
    extends WildMatMatch
  {
    MatchRange(char first,
	       char last,
	       boolean wildcard)
      {
	super(wildcard);
	m_first = first;
	m_last = last;
      }

    public boolean match(char ch)
      {
	return (ch >= m_first) && (ch <= m_last);
      }

    private char m_first;
    private char m_last;
  }

  private static class MatchChar
    extends WildMatMatch
  {
    MatchChar(char ch,
	      boolean wildcard)
      {
	super(wildcard);
	m_char = ch;
      }

    public boolean match(char ch)
      {
	return m_char == ch;
      }

    private char m_char;
  }

  private static class MatchAny
    extends WildMatMatch
  {
    MatchAny(boolean wildcard)
      {
	super(wildcard);
      }

    public boolean match(char ch)
      {
	return ch != WildMatMatch.EOS;
      }

    private char m_char;
  }

  private static class MatchMulti
    extends WildMatMatch
  {
    MatchMulti(boolean invert,
	       Vector choices,
	       boolean wildcard)
      {
	super(wildcard);
	m_choices = new WildMatMatch[ choices.size() ];
	choices.copyInto(m_choices);
	m_positive = !invert;
      }

    public boolean match(char ch)
      {
	for (int i = 0; i < m_choices.length; i++)
	  {
	    if (m_choices[i].match(ch))
	      {
		return m_positive;
	      }
	  }

	return !m_positive;
      }

    private boolean m_positive;
    private WildMatMatch[] m_choices;
  }

  private boolean m_isWildcard;

  public static final char EOS = '|';
}
