///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.coffee_server.services.nntp.pub;

import java.text.*;
import java.util.*;
import org.cooldevtools.coffee_server.framework.CoffeeServiceException;

public abstract class DateUtils
{
   public static String now()
   {
      return dateToString(new Date());
   }

   public static synchronized String dateToString(Date dt)
   {
      return s_format.format(dt);
   }

   public static synchronized String dateToNntpString(Date dt)
   {
      return s_nntpFormat.format(dt);
   }

   public static Date stringToDate(String s)
   throws CoffeeServiceException
   {
      Calendar cal = Calendar.getInstance();

      try {
         int year   = Integer.parseInt(s.substring(0, 4));
         int month  = Integer.parseInt(s.substring(4, 6));
         int day    = Integer.parseInt(s.substring(6, 8));

         int hour   = Integer.parseInt(s.substring(8, 10));
         int minute = Integer.parseInt(s.substring(10, 12));
         int second = Integer.parseInt(s.substring(12, 14));

         cal.set(year, month - 1, day, hour, minute, second);
         return cal.getTime();
      } catch (Exception ex) {
         throw new CoffeeServiceException("invalid date: " + s + ": " + ex.getMessage());
      }
   }

   public static Date nDaysAgo(int days_ago)
   {
      Calendar now = Calendar.getInstance();
      now.add(Calendar.DATE, -days_ago);
      return now.getTime();
   }

   public static String nDaysAgoString(int days_ago)
   {
      return dateToString(nDaysAgo(days_ago));
   }

   public static final int DATE_LEN                  = 14;
   public static final SimpleDateFormat s_format     = new SimpleDateFormat("yyyyMMddkkmmss");
   public static final SimpleDateFormat s_nntpFormat = new SimpleDateFormat("EEE, dd MMM yyyy kk:mm:ss zzz");
   public static final String ZERO_DATE_STRING       = "00000000000000";
   public static final String INFINITE_DATE_STRING   = "99999999999999";
}
