///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//


package org.cooldevtools.coffee_server.services.nntp.pub;

import java.net.*;
import java.util.*;
import java.io.*;
import org.cooldevtools.utility.*;
import org.cooldevtools.coffee_server.framework.*;

public abstract class Message
implements Serializable
{
   public Message(long number)
   {
      m_number    = number;
      m_header    = null;
      m_body      = null;
      m_fields    = new Hashtable();
      m_date      = new Date();
   }

   public Date getDate()
   {
      return m_date;
   }

   public void setHeader(Vector lines)
   {
      m_header = new String[lines.size()];
      lines.copyInto(m_header);
      parseHeader();
   }

   private SuperVector charsToVector(char[] s)
   {
      SuperVector vec = new SuperVector();

      int next = 0;
      for (int i = 0; i < s.length; i++) {
         if (s[i] == '\n') {
            vec.addElement(new String(s, next, i - next));
            next = i + 1;
         }
      }

      if (next < s.length) {
         vec.addElement(new String(s, next, s.length - next));
      }

      return vec;
   }

   private Vector stringToVector(String text)
   {
      return charsToVector(text.toCharArray());
   }

   public void setHeader(String text)
   {
      setHeader(stringToVector(text));
   }

   public void setBody(Vector lines)
   {
      m_body = new String[lines.size()];
      lines.copyInto(m_body);
   }

   public void setBody(String text)
   {
      setBody(stringToVector(text));
   }

   public boolean isNewerThan(Date when)
   {
      return (when == null) || m_date.after(when);
   }

   /**
      Field name must be in lower case or you won't find it!
    */
   public String getField(String fieldName)
   {
      return (String)m_fields.get(fieldName);
   }

   /**
      Field name must be in lower case or you won't find it!
    */
   public String getFieldAlways(String fieldName)
   {
      String value = (String)m_fields.get(fieldName);
      return (value == null) ? "" : value;
   }

   public int getByteCount()
   {
      int count = 2;

      for (int i = 0; i < m_header.length; i++) {
         count += m_header[i].length() + 2;
      }

      for (int i = 0; i < m_body.length; i++) {
         count += m_body[i].length() + 2;
      }

      return count;
   }

   public int getLineCount()
   {
      return m_body.length;
   }

   public String[] getHeader()
   throws CoffeeServiceException
   {
      return m_header;
   }

   public long getNumber()
   {
      return m_number;
   }

   private String arrayToString(String[] array)
   {
      StringBuffer buffer = new StringBuffer();
      for (int i = 0; i < array.length; i++) {
         buffer.append(array[i]);
         buffer.append("\n");
      }
      return buffer.toString();
   }

   public String getHeaderAsString()
   throws CoffeeServiceException
   {
      return arrayToString(getHeader());
   }

   public String[] getBody()
   throws CoffeeServiceException
   {
      return m_body;
   }

   public String getBodyAsString()
   throws CoffeeServiceException
   {
      return arrayToString(getBody());
   }

   protected String parseField(StringBuffer value,
                               String line)
   {
      int index = line.indexOf(':');
      if (index <= 0) {
         return null;
      }

      String name = line.substring(0, index);
      index++;
      while ((index < line.length()) && (line.charAt(index) == ' ')) {
         index++;
      }

      value.append(line.substring(index));
      return name;
   }

   protected void addContinuation(StringBuffer value,
                                  String line)
   {
      for (int index = 0; index < line.length(); index++) {
         char ch = line.charAt(index);
         if ((ch != ' ') && (ch != '\t')) {
            value.append(line.substring(index - 1));
            return;
         }
      }
   }

   protected void addField(String name,
                           StringBuffer value)
   {
      String field_name = name.toLowerCase();
      m_fields.put(field_name, value.toString().trim());
   }

   protected void setAuthor()
   {
      String name = getField(FROM);
      if (name != null) {
         m_fields.put(AUTHOR, name);
         return;
      }

      name = getField(SENDER);
      if (name != null) {
         m_fields.put(AUTHOR, name);
         return;
      }

      m_fields.put(AUTHOR, UNKNOWN);
   }

   private static synchronized String localAddress()
   {
      if (s_localAddress == null) {
         try {
            InetAddress address = InetAddress.getLocalHost();
            s_localAddress = address.getHostName();
            if (s_localAddress == null) {
               s_localAddress = address.toString();
            }
         } catch (UnknownHostException ex) {
            s_localAddress = "host.address.unknown";
         }
      }
      return s_localAddress;
   }

   protected void setMessageID()
   {
      String id = getField(MESSAGE_ID);
      if (id == null) {
         m_fields.put(MESSAGE_ID, createMessageID());
      }
   }

  private static int nextRandomNumber()
  {
    synchronized (s_random) {
      return s_random.nextInt();
    }
  }

  public static String createMessageID()
  {
    StringBuffer buffer = new StringBuffer();
    buffer.append("<");
    buffer.append(Long.toHexString(System.currentTimeMillis()));
    buffer.append(".");
    buffer.append(Integer.toHexString(nextRandomNumber()));
    buffer.append(".BC!JNews@");
    buffer.append(localAddress());
    buffer.append(">");
    return buffer.toString();
  }

   public void setMessageID(String new_id)
   {
      m_fields.put(MESSAGE_ID, new_id);
   }

   protected void parseHeader()
   {
      String current_field = null;
      StringBuffer current_value = null;

      m_fields.clear();

      for (int i = 0; i < m_header.length; i++) {
         String line = m_header[i];
         char first_char = line.charAt(0);
         if ((first_char == ' ') || (first_char == '\t')) {
            addContinuation(current_value, line);
         } else {
            if (current_field != null) {
               addField(current_field, current_value);
            }
            current_value = new StringBuffer();
            current_field = parseField(current_value, line);
         }
      }
      if (current_field != null) {
         addField(current_field, current_value);
      }

      setAuthor();
      setMessageID();
   }

   public static boolean isMessageID(String str)
   {
      if (str == null) {
         return false;
      }

      if (str.charAt(0) != '<') {
         return false;
      }

      return str.indexOf('>') == (str.length() - 1);
   }

   public final String messageId()
   {
      return getField(MESSAGE_ID);
   }

   public String messageOverview()
   {
      StringBuffer buffer = new StringBuffer();
      for (int i = 0; i < OVERVIEW_FIELD_KEYS.length; i++) {
         buffer.append(getFieldAlways(OVERVIEW_FIELD_KEYS[i]));
         buffer.append('\t');
      }
      buffer.append(getByteCount());
      buffer.append('\t');
      buffer.append(getLineCount());
      return buffer.toString();
   }

   private static void writeArray(DataOutputStream s,
                                  String[] array)
   throws IOException
   {
      s.writeInt(array.length);
      for (int i = 0; i < array.length; i++) {
         s.writeUTF(array[i]);
      }
   }

   private static Vector readArray(DataInputStream s)
   throws IOException
   {
      int length = s.readInt();
      Vector array = new Vector();
      for (int i = 0; i < length; i++) {
         array.addElement(s.readUTF());
      }
      return array;
   }

   public abstract NewsGroup[] getNewsGroups()
   throws CoffeeServiceException;

   private String[]  m_header;
   private String[]  m_body;
   private long      m_number;
   private Hashtable m_fields;
   private Date      m_date;

   public static final String AUTHOR     = "author";
   public static final String CONTROL    = "control";
   public static final String DATE       = "date";
   public static final String FROM       = "from";
   public static final String MESSAGE_ID = "message-id";
   public static final String NEWSGROUPS = "newsgroups";
   public static final String REFERENCES = "references";
   public static final String SENDER     = "sender";
   public static final String SUBJECT    = "subject";
   public static final String BYTES      = "bytes";
   public static final String LINES      = "lines";

   public static final String UNKNOWN       = "author_unknown";

   private static String s_localAddress;
   private static Random s_random = new Random(System.currentTimeMillis());

   private static final String[] OVERVIEW_FIELD_KEYS = {
      SUBJECT,
      AUTHOR,
      DATE,
      MESSAGE_ID,
      REFERENCES,
   };

   public static final String[] OVERVIEW_FIELDS_LIST = {
      "Subject:",
      "Author:",
      "Date:",
      "Message-ID:",
      "References:",
      "Bytes:",
      "Lines:"
   };
}
