///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//


package org.cooldevtools.coffee_server.services.nntp.pub;

import java.io.*;
import java.net.*;
import java.util.*;
import org.cooldevtools.utility.Assert;
import org.cooldevtools.coffee_server.framework.*;

public class NntpSession
{
   public NntpSession(ClientRequest request,
                      NewsRepositoryFactory factory)
   throws CoffeeServiceException
   {
      m_request = request;
      m_factory = factory;
      setUser(factory.getUserRepository().getDefaultUser());
   }

   public NewsRepositoryFactory getFactory()
   {
      return m_factory;
   }

   public User getUser()
   {
      if (m_authenticated) {
	 return m_user;
      }
      if (m_none != null) {
	 return m_none;
      }
      m_none = new NoneUser();
      m_none.setSenderAddress(getClientHost());
      return m_none;
   }

   public void setUser(User value)
   {
      m_user = value;
      m_user.setSenderAddress(getClientHost());
      try {
	 String password = value.getPassword();
	 m_authenticated = password.length() == 0;
      } catch (CoffeeServiceException ex) {
	 m_authenticated = false;
      }
   }

   public void setAuthenticated(boolean value)
   {
      m_authenticated = value;
      if (value) {
	 m_none = null;
      }
   }

   public boolean isAuthenticated()
   {
      return m_authenticated;
   }

   public NewsGroupRepository getNewsGroups()
   {
      return m_factory.getNewsGroupRepository();
   }

   public MessageRepository getMessages()
   {
      return m_factory.getMessageRepository();
   }

   public UserRepository getUsers()
   {
      return m_factory.getUserRepository();
   }

   public RoleRepository getRoles()
   {
      return m_factory.getRoleRepository();
   }

   public NewsGroup getCurrentGroup()
   {
      return m_CurrentGroup;
   }

   public void setCurrentGroup(NewsGroup value)
   throws CoffeeServiceException
   {
      m_CurrentGroup = value;
      if (value != null) {
         m_currentIndex = value.getLowMessageIndex();
      }
   }

   public long getCurrentIndex()
   {
      return m_currentIndex;
   }

   public void setCurrentIndex(long value)
   {
      m_currentIndex = value;
   }

   public ClientRequest getRequest()
   {
      return m_request;
   }

   public String getClientHost()
   {
      InetAddress address = m_request.getInetAddress();
      if (Assert.DEBUG) Assert.pre(address != null);
      if (Assert.DEBUG) Assert.pre(address.getHostName() != null);
      if (Assert.DEBUG) Assert.pre(address.getHostAddress() != null);
      String host_name = address.getHostName();
      if (host_name == null || host_name.length() == 0) {
	 host_name = address.getHostAddress();
      }
      return host_name;
   }

   private ClientRequest m_request;
   private NewsRepositoryFactory m_factory;
   private User m_user;
   private User m_none;
   private NewsGroup m_CurrentGroup;
   private long m_currentIndex;
   private boolean m_authenticated;
}
