///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//


package org.cooldevtools.coffee_server.services.nntp.pub;

import java.io.*;
import org.cooldevtools.utility.Assert;

public class NntpWriter
{
   public NntpWriter(OutputStream s)
   {
      if (DEBUG) {
         m_output = new DebugPrintWriter(s, System.out);
      } else {
         m_output = new PrintWriter(s);
      }
   }

   public void close()
   throws IOException
   {
      m_output.close();
   }

   public void flush()
   throws IOException
   {
      m_output.flush();
   }

   public void writeResponse(int code,
                             String message)
   throws IOException
   {
      m_output.print(code);
      m_output.print(' ');
      m_output.print(message);
      m_output.print(CRLF);
   }

   public void writeText(String text)
   throws IOException
   {
      if (text.length() == 0) {
         m_output.print(CRLF);
      } else {
         if (text.charAt(0) == DOT) {
            m_output.print(DOT);
         }
         m_output.print(text);
         m_output.print(CRLF);
      }
   }

   public void writeText(String[] text)
   throws IOException
   {
      for (int i = 0; i < text.length; i++) {
         writeText(text[i]);
      }
   }

   public void writeEscapedText(String text)
   throws IOException
   {
      m_output.print(text);
      m_output.print(CRLF);
   }

   public void writeEscapedText(String[] text)
   throws IOException
   {
      for (int i = 0; i < text.length; i++) {
         m_output.print(text[i]);
         m_output.print(CRLF);
      }
   }

   public void writeEndOfText()
   throws IOException
   {
      m_output.print(MULTI_LINE_TERM);
   }

   private PrintWriter m_output;

   private static char   DOT = '.';
   private static char[] CRLF = { (char)13, (char)10};
   private static char[] MULTI_LINE_TERM = {
      '.',
      (char)13, (char)10
   };

   private static final boolean DEBUG = false;
}
