///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//


package org.cooldevtools.coffee_server.services.nntp.pub;

import org.cooldevtools.utility.Assert;
import org.cooldevtools.coffee_server.framework.*;

public class NoneUser
implements User
{
   public NoneUser()
   {
   }

   public boolean canShutdown()
   throws CoffeeServiceException
   {
      Role[] roles = getRoles();
      for (int i = 0; i < roles.length; i++) {
         if (roles[i].canShutdown()) {
            return true;
         }
      }
      return false;
   }

   public boolean canRead()
   throws CoffeeServiceException
   {
      Role[] roles = getRoles();
      for (int i = 0; i < roles.length; i++) {
         if (roles[i].canRead()) {
            return true;
         }
      }
      return false;
   }

   public boolean canPost()
   throws CoffeeServiceException
   {
      Role[] roles = getRoles();
      for (int i = 0; i < roles.length; i++) {
         if (roles[i].canPost()) {
            return true;
         }
      }
      return false;
   }

   public boolean canCreateNewsGroup(String group_name)
   throws CoffeeServiceException
   {
      Role[] roles = getRoles();
      for (int i = 0; i < roles.length; i++) {
         if (roles[i].canCreateNewsGroup(group_name)) {
            return true;
         }
      }
      return false;
   }

   public boolean canRemoveNewsGroup(NewsGroup group)
   throws CoffeeServiceException
   {
      Role[] roles = getRoles();
      for (int i = 0; i < roles.length; i++) {
         if (roles[i].canRemoveNewsGroup(group)) {
            return true;
         }
      }
      return false;
   }

   public boolean canReadNewsGroup(NewsGroup group)
   throws CoffeeServiceException
   {
      Role[] roles = getRoles();
      for (int i = 0; i < roles.length; i++) {
         if (roles[i].canReadNewsGroup(group)) {
            return true;
         }
      }
      return false;
   }

   public boolean canReadMessage(Message m)
   throws CoffeeServiceException
   {
      Role[] roles = getRoles();
      for (int i = 0; i < roles.length; i++) {
         if (roles[i].canReadMessage(m)) {
            return true;
         }
      }
      return false;
   }

   public boolean canPostToNewsGroup(NewsGroup group)
   throws CoffeeServiceException
   {
      Role[] roles = getRoles();
      for (int i = 0; i < roles.length; i++) {
         if (roles[i].canPostToNewsGroup(group)) {
            return true;
         }
      }
      return false;
   }

   public boolean canCancelMessage(Message m)
   throws CoffeeServiceException
   {
      Role[] roles = getRoles();
      for (int i = 0; i < roles.length; i++) {
         if (roles[i].canCancelMessage(m)) {
            return true;
         }
      }
      return false;
   }

   public String getEmailAddress()
   throws CoffeeServiceException
   {
      return "unknownuser";
   }

   public String getUsername()
   throws CoffeeServiceException
   {
      return "unknownuser";
   }

   public String getPassword()
   throws CoffeeServiceException
   {
      return "";
   }

   public Role[] getRoles()
   throws CoffeeServiceException
   {
      return NONE_ROLE;
   }

   public boolean canDefineRole(String rolename)
   throws CoffeeServiceException
   {
      Role[] roles = getRoles();
      for (int i = 0; i < roles.length; i++) {
         if (roles[i].canDefineRole(rolename)) {
            return true;
         }
      }
      return false;
   }

   public boolean canListRole(String rolename)
   throws CoffeeServiceException
   {
      Role[] roles = getRoles();
      for (int i = 0; i < roles.length; i++) {
         if (roles[i].canListRole(rolename)) {
            return true;
         }
      }
      return false;
   }

   public boolean canDeleteRole(String rolename)
   throws CoffeeServiceException
   {
      Role[] roles = getRoles();
      for (int i = 0; i < roles.length; i++) {
         if (roles[i].canDeleteRole(rolename)) {
            return true;
         }
      }
      return false;
   }

   public boolean canListRoles()
   throws CoffeeServiceException
   {
      Role[] roles = getRoles();
      for (int i = 0; i < roles.length; i++) {
         if (roles[i].canListRoles()) {
            return true;
         }
      }
      return false;
   }

   public boolean canDefineUser(String username)
   throws CoffeeServiceException
   {
      if (getUsername().equals(username)) {
         return true;
      }

      Role[] roles = getRoles();
      for (int i = 0; i < roles.length; i++) {
         if (roles[i].canDefineUser(username)) {
            return true;
         }
      }
      return false;
   }

   public boolean canListUser(String username)
   throws CoffeeServiceException
   {
      if (getUsername().equals(username)) {
         return true;
      }

      Role[] roles = getRoles();
      for (int i = 0; i < roles.length; i++) {
         if (roles[i].canListUser(username)) {
            return true;
         }
      }
      return false;
   }

   public boolean canDeleteUser(String username)
   throws CoffeeServiceException
   {
      Role[] roles = getRoles();
      for (int i = 0; i < roles.length; i++) {
         if (roles[i].canDeleteUser(username)) {
            return true;
         }
      }
      return false;
   }

   public boolean canListUsers()
   throws CoffeeServiceException
   {
      Role[] roles = getRoles();
      for (int i = 0; i < roles.length; i++) {
         if (roles[i].canListUsers()) {
            return true;
         }
      }
      return false;
   }

   public boolean canChangeUserPassword(String user_to_change)
   throws CoffeeServiceException
   {
      if (getUsername().equals(user_to_change)) {
         return true;
      }

      Role[] roles = getRoles();
      for (int i = 0; i < roles.length; i++) {
         if (roles[i].canChangeUserPassword(user_to_change)) {
            return true;
         }
      }
      return false;
   }

   public boolean canUseRoles(String[] role_names)
   throws CoffeeServiceException
   {
      Role[] roles = getRoles();
      for (int i = 0; i < roles.length; i++) {
         if (roles[i].canUseRoles(role_names)) {
            return true;
         }
      }
      return false;
   }

   public void setSenderAddress(String hostname)
   {
      try {
         if (Assert.DEBUG) Assert.pre(hostname != null);
         m_senderAddress = getUsername() + "@" + hostname;
      } catch (CoffeeServiceException ex) {
      }
   }

   public String getSenderAddress()
   {
      if (Assert.DEBUG) Assert.pre(m_senderAddress != null);
      return m_senderAddress;
   }

   private String m_senderAddress;
   private static final Role[] NONE_ROLE = {
      new NoneRole()
   };
}
