///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//


package org.cooldevtools.coffee_server.services.nntp.test;

import java.io.*;
import java.net.*;
import java.util.*;
import org.cooldevtools.utility.*;
import org.cooldevtools.coffee_server.services.nntp.protocol.ServerConnection;
import org.cooldevtools.coffee_server.services.nntp.pub.DateUtils;

public class Client
{
   public Client(String host,
                 int port)
   throws UnknownHostException,
   IOException
   {
      m_server = new ServerConnection();
      m_server.setHost(host, port);
      m_debug = System.out;
      m_rand = new Random();
   }

   public void testArticle(long article_number)
   throws IOException
   {
      String[] message = m_server.article(article_number);
      if (message == null) {
         return;
      }

      String[] head = m_server.head(article_number);
      if (head == null) {
         throw new IOException("message exists but body not found: " + article_number);
      }

      String[] body = m_server.body(article_number);
      if (body == null) {
         throw new IOException("message exists but body not found: " + article_number);
      }

      if (message.length != (head.length + body.length + 1)) {
         throw new IOException("message/body/header length mismatch: " + article_number);
      }

      String[] message2 = new String[ message.length ];
      System.arraycopy(head, 0, message2, 0, head.length);
      message2[head.length] = "";
      System.arraycopy(body, 0, message2, head.length + 1, body.length);

      for (int i = 0; i < message.length; i++) {
         if (!message[i].equals(message2[i])) {
            throw new IOException("message/body/header mismatch: " + article_number);
         }
      }
   }

   public void dumpArticle(long article_number)
   throws IOException
   {
      String[] message = m_server.article(article_number);
      if (message == null) {
         return;
      }

      m_debug.println("Message #" + article_number + " retrieved:");
      for (int i = 0; i < message.length; i++) {
         m_debug.print("\t");
         m_debug.println(message[i]);
      }
   }

   private void dumpGroup()
   throws IOException
   {
      long[] counters = setGroup();
      long first = counters[1];
      long last  = counters[2];

      for (long i = first; i <= last; i++) {
         testArticle(i);
      }
   }

   public void dumpGroupsTest(int num_loops)
   throws IOException
   {
      m_server.modeReader();
      m_server.listExtensions();
      for (int i = 0; i < num_loops; i++) {
         String[] groups = m_server.listActive();
         for (int k = 0; k < groups.length; k++) {
            m_group_name = groups[k];
            dumpGroup();
         }
      }
   }

   public void cancel(String message_id)
   throws IOException
   {
      m_server.cancel(message_id, m_group_name, "tester@nowhere.com");
   }

   public void cancel(long article_number)
   throws IOException
   {
      System.out.println("cancel " + article_number);
      String message_id = m_server.stat(article_number);
      cancel(message_id);
   }

   public void cancel_one()
   throws IOException
   {
      long[] numbers = m_server.listGroup();
      if (numbers.length == 0) {
         return;
      }

      cancel(numbers[random(numbers.length)]);
   }

   public void cancel_all()
   throws IOException
   {
      long[] numbers = m_server.listGroup();
      if (numbers.length == 0) {
         return;
      }

      for (int i = 0; i < numbers.length; i++) {
         cancel(numbers[i]);
      }
   }

   public int calcChecksum(String[] message)
   {
      int check_sum = 0;
      int last = message.length - 1;
      int i = 0;
      while (i < last && message[i].length() > 0) {
        // skipping header lines...
        ++i;
      }
      for (++i; i < last; i++) {
	 if (!message[i].startsWith("Sender: ")) {
	    check_sum = CrcCalc.crc32(message[i].getBytes(), check_sum);
	 }
      }
      return check_sum;
   }

   public void post_one()
   throws IOException
   {
      String message_id = "<"
                        + String.valueOf(System.currentTimeMillis())
                        + "."
                        + String.valueOf(random(1000000))
                        + "@nowhere.com>";

      boolean skip_message_id = (random(4) == 1);

      int wrapper_lines = 7;
      int num_lines = 10 + random(800);
      if (skip_message_id) {
        --wrapper_lines;
        message_id = "NONE - test OE bug";
      }

      char ch = (char)(65 + random(26));
      String[] message = new String[wrapper_lines + num_lines];

      System.out.println("post " + message_id);
      
      int index = 0;
      message[index++] = "From: tester@nowhere.com";
      message[index++] = "Subject: testing " + message_id;
      message[index++] = "Newsgroups: " + m_group_name;
      if (!skip_message_id) {
        message[index++] = "Message-ID: " + message_id;
      }
      message[index++] = "Date: " + DateUtils.dateToNntpString(new Date());
      message[index++] = "";
      for (int i = 0; i < num_lines; i++) {
         char[] line = new char[random(80)];
         for (int k = 0; k < line.length; k++) {
            line[k] = ch;
         }
         message[index++] = new String(line);
      }

      int check_sum = calcChecksum(message);
      message[index++] = String.valueOf(check_sum);

      m_server.post(message);
   }

   public void validateChecksums()
   throws IOException
   {
      for (String[] message = m_server.article(); message != null; message = m_server.next()) {
         for (int i = 0; i < message.length; i++) {
            if (message[i].toLowerCase().startsWith("message-id:")) {
               System.out.println("read " + message[i]);
               break;
            }
         }
         int check_sum_msg  = Integer.parseInt(message[message.length - 1]);
         int check_sum_calc = calcChecksum(message);
         if (check_sum_msg != check_sum_calc) {
            throw new IOException("checksum error");
         }
      }
   }

   private long[] setGroup()
   throws IOException
   {
      long[] counters = new long[3];
      if (!m_server.setGroup(m_group_name, counters)) {
         throw new IOException("GROUP " + m_group_name + " does not exist!");
      }
      return counters;
   }

   public void readerTest(String group_name,
                          int num_loops)
   throws IOException
   {
      m_group_name = group_name;
      m_server.modeReader();
      for (int i = 0; i < num_loops; i++) {
         setGroup();
         validateChecksums();
      }
   }

   public void cancelTest(String group_name,
                          int num_loops)
   throws IOException
   {
      m_group_name = group_name;
      m_server.modeReader();
      for (int i = 0; i < num_loops; i++) {
         setGroup();
         cancel_all();
      }
   }

   public void writerTest(String group_name,
                          int num_loops)
   throws IOException
   {
      m_group_name = group_name;
      m_server.modeReader();
      setGroup();
      for (int i = 0; i < num_loops; i++) {
         if (random(3) == 0) {
            cancel_one();
         } else {
            post_one();
         }
      }
   }

   private int random(int limit)
   {
      return Math.abs(m_rand.nextInt() >>> 3) % limit;
   }

   public static void main(String[] args)
   {
      try {
         String host = args[0];
         int port = Integer.parseInt(args[1]);
         int loops = Integer.parseInt(args[2]);
         String test = args[3];
         Client client = new Client(host, port);

	 final String GROUP_NAME = "public.discuss";
         if (test.equals("scan")) {
            client.dumpGroupsTest(loops);
         } else if (test.equals("writer")) {
            client.writerTest(GROUP_NAME, loops);
         } else if (test.equals("reader")) {
            client.readerTest(GROUP_NAME, loops);
         } else if (test.equals("cancel")) {
            client.cancelTest(GROUP_NAME, loops);
         } else {
            System.out.println("error: unknown test: " + test);
            return;
         }
      } catch (Exception ex) {
         ex.printStackTrace();
      }
   }

   private PrintStream m_debug;
   private Random m_rand;
   private String m_group_name;
   private ServerConnection m_server;
}
