///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//


package org.cooldevtools.coffee_server.services.test;

import java.io.*;
import java.net.*;
import java.awt.*;
import java.util.*;
import javax.swing.*;
import org.cooldevtools.hdl.*;
import org.cooldevtools.coffee_server.config.*;
import org.cooldevtools.coffee_server.framework.*;

/**
   Hello world service.
 */
public class Hello
extends BasicCoffeeServiceManager
implements CoffeeService
{
   public Hello()
   {
      this("Hello, world.");
   }

   public Hello(String message)
   {
      m_message = message;
      m_bean = new Bean();
   }

   public void configure(ServiceConfigInfo config)
   throws CoffeeServiceException
   {
      super.configure(config);
      Statement configuration = config.getConfiguration();
      System.out.println("configure(" + configuration.name().strValue() + ")");
      m_message = configuration.argument(0).strValue();
   }

   private class Bean
   implements CoffeeServiceConfigBean {
      Bean()
      {
         m_component = new JTextField(m_message, 40);
      }

      public boolean isValidConfiguration(StringBuffer buffer)
      {
         if (m_component.getText().length() > 0) {
            return true;
         }

         buffer.append("message must not be blank");
         return false;
      }

      public Statement getConfiguration()
      {
         Statement config = new Statement(Token.idToken("configuration", "", 0), false);
         config.addArgument(Token.strToken(m_component.getText(), "", 0));
         return config;
      }

      public void setConfiguration(Statement configuration)
      {
         if (configuration == null) {
            m_component.setText("");
         } else {
            m_component.setText(configuration.argument(0).strValue());
         }
      }

      public Component getComponent()
      {
         return m_component;
      }

      private JTextField m_component;
   }

   public void serviceRequest(ClientRequest req)
   throws IOException, CoffeeServiceException
   {
      PrintWriter out = new PrintWriter(req.getOutputStream());
      out.println(m_message);
      out.flush();
      out.close();
   }

   public CoffeeService newServiceInstance()
   {
      return this;
   }

   public CoffeeServiceConfigBean newConfigBeanInstance()
   {
      return m_bean;
   }

   public boolean hasConfigBean()
   {
      return true;
   }

   public static void main(String[] args)
   {
      try {
         int port = 5250;
         if (args.length >= 1) {
            port = Integer.parseInt(args[0]);
         }

         int num_threads = 5;
         if (args.length >= 2) {
            num_threads = Integer.parseInt(args[1]);
         }

         CoffeeServer server = new CoffeeServer();
         server.setNumThreads(num_threads);
         server.addPooledService(null, port, port, new Hello("Hello World"));
         server.addPooledService(null, port + 1, port + 1, new Hello("Goodbye!"));
         server.startup();
      } catch (Exception ex) {
         ex.printStackTrace();
      }
   }

   private String m_message;
   private Bean m_bean;
}
