///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//


package org.cooldevtools.coffee_server.test;

import java.util.*;
import org.cooldevtools.utility.RWLock;

public class TestLocks
  implements Runnable
{
  public TestLocks()
    {
      m_readers = 0;
      m_writers = 0;
      m_lock = new RWLock();
      m_random = new Random();
    }

  public void readerTest(int depth)
    throws InterruptedException
    {
      addReader(depth);
      if (depth < MAX_RECURSION)
	{
	  if ((nextInt() % 2) == 0)
	    {
	      readerTest(depth + 1);
	    }
	}

      Thread.sleep(nextInt() % 50);
      removeReader(depth);
    }

  public void writerTest(int depth)
    throws InterruptedException
    {
      addWriter(depth);
      if (depth < MAX_RECURSION)
	{
	  switch (nextInt() % 4)
	    {
	    case 0:
	      System.out.println("writer acquiring read lock");
	      readerTest(depth + 1);
	      break;

	    case 1:
	      System.out.println("writer acquiring write lock");
	      writerTest(depth + 1);
	      break;

	    default:
	      break;
	    }
	}

      Thread.sleep(nextInt() % 50);
      removeWriter(depth);
    }

  public void run()
    {
      try
	{
	  while (true)
	    {
	      if ((nextInt() % 4) == 0)
		{
		  writerTest(0);
		}
	      else
		{
		  readerTest(0);
		}
	    }
	}
      catch (Exception ex)
	{
	  ex.printStackTrace();
	  System.exit(1);
	}
    }

  public synchronized int nextInt()
    {
      return Math.abs(m_random.nextInt());
    }

  public void addReader(int depth)
    throws InterruptedException
    {
      m_lock.acquireReadLock();
      synchronized (this)
	{
	  if (depth == 0)
	    {
	      m_readers++;
	    }
	  checkCounters(depth);
	}
    }

  public void addWriter(int depth)
    throws InterruptedException
    {
      m_lock.acquireWriteLock();
      synchronized (this)
	{
	  if (depth == 0)
	    {
	      m_writers++;
	    }
	  checkCounters(depth);
	}
    }

  public synchronized void removeReader(int depth)
    {
      if (depth == 0)
	{
	  m_readers--;
	}

      checkCounters(depth);
      m_lock.releaseReadLock();
    }

  public synchronized void removeWriter(int depth)
    {
      if (depth == 0)
	{
	  m_writers--;
	}

      checkCounters(depth);
      m_lock.releaseWriteLock();
    }

  private void checkCounters(int depth)
    {
      if ((m_readers != 0) || (m_writers != 0))
	{
	  System.out.print("readers = ");
	  System.out.print(m_readers);
	  System.out.print(" writers = ");
	  System.out.print(m_writers);
	  System.out.print(" depth = ");
	  System.out.print(depth);
	  System.out.println();
	}

      if (m_writers > 1)
	{
	  throw new RuntimeException("too many writers: " + m_writers);
	}

      if ((m_readers > 0) && (m_writers > 0))
	{
	  throw new RuntimeException("readers and writers: " + m_readers + "/" + m_writers);
	}
    }

  public static void main(String[] args)
    {
      try
	{
	  int num_threads = Integer.parseInt(args[0]);
	  TestLocks locks = new TestLocks();

	  for (int i = 0; i < num_threads; i++)
	    {
	      Thread t = new Thread(locks);
	      t.start();
	    }
	}
      catch (Exception ex)
	{
	  ex.printStackTrace();
	}
    }

  int m_readers;
  int m_writers;
  Random m_random;
  RWLock m_lock;

  private static final int MAX_RECURSION = 10;
}
