///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//


package org.cooldevtools.coffee_server.util;

/**
   Trivial little FIFO queue class since JDK doesn't come with one.
 */
public class FifoQueue
{
   public FifoQueue()
   {
      m_front = null;
      m_back = null;
      m_size = 0;
   }

   private static class Node {
      public Node   next;
      public Object value;
   }

   /**
      Returns true iff queue is empty.
    */
   public boolean isEmpty()
   {
      return m_size == 0;
   }

   /**
      Put item at back of queue.
    */
   public void put(Object obj)
   {
      Node node  = new Node();
      node.next  = m_front;
      node.value = obj;

      if (m_front == null) {
         m_front = node;
         m_back = node;
      } else {
         m_front = node;
      }

      m_size++;
   }

   /**
      Returns oldest item in queue.
      Queue must not be empty when you call this!
    */
   public Object get()
   {
      Node node = m_front;
      m_front = node.next;
      if (m_front == null) {
         m_back = null;
      }
      m_size--;
      return node.value;
   }

   public int size()
   {
      return m_size;
   }

   private Node m_front;
   private Node m_back;
   private int  m_size;
}
