///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.hdl;

import java.util.Enumeration;
import java.util.Hashtable;

class CheckArgument
{
    CheckArgument()
    {
        m_intOK = false;
        m_stringOK = false;
        m_idOK = false;
        m_ids = new Hashtable();
    }

    public void checkArgument(Token arg)
        throws HDLException
    {
        switch (arg.tokenType()) {
        case Token.INTEGER:
            if (!m_intOK)
                throwInvalidArgument(arg);
            break;

        case Token.STRING:
            if (!m_stringOK)
                throwInvalidArgument(arg);
            break;

        case Token.IDENTIFIER:
	    if (m_idOK)
		break;
	    Integer value = (Integer)m_ids.get(arg.strValue());
	    if (value == null)
                throwInvalidArgument(arg);
	    arg.setIntValue(value.intValue());
            break;
        }
    }

    public final void acceptId()
    {
        m_idOK = true;
    }

    public final void acceptId(Token id,
			       int value)
    {
        m_ids.put(id.strValue(), new Integer(value));
    }

    public final void acceptInteger()
    {
        m_intOK = true;
    }

    public final void acceptString()
    {
        m_stringOK = true;
    }

    private final void throwInvalidArgument(Token arg)
        throws HDLException
    {
        StringBuffer msg = new StringBuffer();
        msg.append("invalid argument, expected");
        if (m_intOK)
            msg.append(" integer");
        if (m_stringOK)
            msg.append(" string");
        if (m_idOK) {
            if (m_intOK || m_stringOK)
                msg.append(" or");
            msg.append(" identifier");
        } else if (m_ids.size() > 0) {
            if (m_intOK || m_stringOK)
                msg.append(" or");
            if (m_ids.size() > 1)
                msg.append(" one of");
            Enumeration keys = m_ids.keys();
            while (keys.hasMoreElements()) {
                String key = (String)keys.nextElement();
                Token value = (Token)m_ids.get(key);
                msg.append(" '");
                msg.append(value.strValue());
                msg.append("'");
            }
        }
        throw new HDLException(arg, msg.toString());
    }

    private boolean m_intOK;
    private boolean m_stringOK;
    private boolean m_idOK;
    private Hashtable m_ids;
}
