///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.hdl;

import java.util.*;

class CheckStatement
{
    CheckStatement(String name,
                   boolean is_block)
    {
        m_name = name;
        m_isBlock = is_block;
        m_minArgCount = 0;
        m_maxArgCount = 0;
        m_arguments = new Vector();
        if (is_block)
            m_children = new Hashtable();
    }

    public void addArgument(CheckArgument arg)
    {
        m_arguments.addElement(arg);
    }

    public void addChild(CheckStatement child)
    {
        m_children.put(child.m_name, child);
    }

    public void checkStatement(Statement stmt)
        throws HDLException
    {
        if (stmt.isBlock() != m_isBlock) {
            if (m_isBlock) {
                throw new HDLException(stmt.name(), "block expected");
            } else {
                throw new HDLException(stmt.name(), "statement expected");
            }
        }

        if (stmt.numArguments() < m_minArgCount) {
            throw new HDLException(stmt.name(),
                "expected at least " + m_minArgCount +
                " arguments but found " + stmt.numArguments());
        }

        if (stmt.numArguments() > m_maxArgCount) {
            throw new HDLException(stmt.name(),
                "expected at most " + m_maxArgCount +
                " arguments but found " + stmt.numArguments());
        }

        int i = 0;
        for (; i < m_minArgCount; i++) {
            CheckArgument checker = (CheckArgument)m_arguments.elementAt(i);
            checker.checkArgument(stmt.argument(i));
        }

        for (; i < stmt.numArguments(); i++) {
            CheckArgument checker = (CheckArgument)m_arguments.elementAt(m_minArgCount);
            checker.checkArgument(stmt.argument(i));
        }

        if (m_isBlock) {
            for (i = 0; i < stmt.numChildren(); i++) {
                Statement child = stmt.child(i);
                CheckStatement checker = (CheckStatement)m_children.get(child.name().strValue());
                if (checker == null) {
                    throwInvalidStatement(child.name());
                } else {
                    checker.checkStatement(stmt.child(i));
                }
            }
        }
    }

    private void throwInvalidStatement(Token token)
        throws HDLException
    {
        StringBuffer msg = new StringBuffer();
        msg.append("invalid statement, expected");
        if (m_children.size() > 1)
            msg.append(" one of");

        Enumeration keys = m_children.keys();
        while (keys.hasMoreElements()) {
            String key = (String)keys.nextElement();
            CheckStatement value = (CheckStatement)m_children.get(key);
            msg.append(" '");
            msg.append(value.m_name);
            msg.append("'");
        }
        throw new HDLException(token, msg.toString());
    }

    public void setArgCounts(int min,
                             int max)
    {
        m_minArgCount = min;
        m_maxArgCount = max;
    }

    private String m_name;
    private boolean m_isBlock;
    private int m_minArgCount;
    private int m_maxArgCount;
    private Vector m_arguments;
    private Hashtable m_children;
}
