///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.hdl;

import org.cooldevtools.utility.Assert;
import java.io.*;

public class Printer {
    public Printer(PrintWriter pw,
		   String indent,
		   String increment) {
	m_writer = pw;
	m_at_new_line = true;
	m_depth = 0;
	m_indent = indent;
	m_increment = increment;
    }

    public Printer(PrintWriter pw) {
	this(pw, "", "    ");
    }

    public void beginBlock() {
	if (Assert.DEBUG) Assert.pre(m_at_new_line);
	printIndent();
	m_writer.print("begin ");
	m_depth++;
	m_at_new_line = false;
    }

    public void endBlock() {
	if (Assert.DEBUG) Assert.pre(m_at_new_line);
	if (Assert.DEBUG) Assert.pre(m_depth > 0);
	m_depth--;
	printIndent();
	m_writer.println("end;");
	if (m_depth == 0)
	    m_writer.flush();
    }

    public void addArgument(String str) {
	if (Assert.DEBUG) Assert.pre(!m_at_new_line);
	m_writer.print(" ");
	m_writer.print(str);
    }

    public void addId(String str) {
	if (Assert.DEBUG) Assert.pre(!m_at_new_line);
	m_writer.print(" ");
	m_writer.print(str);
    }

    public void addString(String str) {
	if (Assert.DEBUG) Assert.pre(!m_at_new_line);
	m_writer.print(" \"");
	m_writer.print(str);
	m_writer.print("\"");
    }

    public void addInteger(int i) {
	if (Assert.DEBUG) Assert.pre(!m_at_new_line);
	m_writer.print(" ");
	m_writer.print(i);
    }

    public void beginStatement(String name) {
	if (m_at_new_line)
	    printIndent();
	m_writer.print(name);
	m_at_new_line = false;
    }

    public void endStatement() {
	if (Assert.DEBUG) Assert.pre(!m_at_new_line);
	m_writer.println(";");
	m_at_new_line = true;
    }

    private void printIndent() {
	m_writer.print(m_indent);
	for (int i = 0; i < m_depth; i++)
	    m_writer.print(m_increment);
    }

    private boolean m_at_new_line;
    private String m_indent;
    private String m_increment;
    private int m_depth;
    private PrintWriter m_writer;
}
