///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.hdl;

import java.util.Enumeration;
import java.util.Vector;
import java.io.PrintWriter;
import org.cooldevtools.utility.SuperVector;

public class Statement {
   public Statement(Token name,
                   boolean is_block)
   {
      m_name = name;
      m_isBlock = is_block;
      m_arguments = new SuperVector();
      if (is_block)
         m_children = new SuperVector();
   }

   public final boolean isBlock()
   {
      return m_isBlock;
   }

   public final Token name()
   {
      return m_name;
   }

   public final int numArguments()
   {
      return m_arguments.size();
   }

   public final Token argument(int index)
   {
      return (Token)m_arguments.elementAt(index);
   }

   public final void addArgument(Token arg)
   {
      m_arguments.addElement(arg);
   }

   public final int numChildren()
   {
      return m_children.size();
   }

   public final Statement child(int index)
   {
      return (Statement)m_children.elementAt(index);
   }

   public final Statement child(Token tok)
   {
      Statement[] children = toArray(m_children);
      for (int i = 0; i < children.length; i++) {
         if (children[i].name().equals(tok)) {
            return children[i];
         }
      }
      return null;
   }

   public final Statement child(String name)
   {
      Statement[] children = toArray(m_children);
      for (int i = 0; i < children.length; i++) {
         if (children[i].name().strValue().equals(name)) {
            return children[i];
         }
      }
      return null;
   }

   public final void addChild(Statement stmt)
   {
      m_children.addElement(stmt);
   }

   public final void write(Printer pr) {
      if (m_isBlock)
         pr.beginBlock();

      pr.beginStatement(m_name.asString());

      Enumeration e = m_arguments.elements();
      while (e.hasMoreElements()) {
         Token arg = (Token)e.nextElement();
         pr.addArgument(arg.asString());
      }

      pr.endStatement();

      if (m_isBlock) {
         e = m_children.elements();
         while (e.hasMoreElements()) {
            Statement child = (Statement)e.nextElement();
            child.write(pr);
         }

         pr.endBlock();
      }
   }

   public final void write(PrintWriter writer,
                           String indent,
                           String increment)
   {
      Printer pr = new Printer(writer, indent, increment);
      write(pr);
   }

   public static Statement[] toArray(Vector v)
   {
      synchronized(v) {
         Statement[] answer = new Statement[v.size()];
         v.copyInto(answer);
         return answer;
      }
   }

   private Token m_name;
   private boolean m_isBlock;
   private SuperVector m_arguments;
   private SuperVector m_children;
}
