///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.hdl;

import org.cooldevtools.utility.*;
import java.io.PrintWriter;

public class Token
{
   private Token(int type,
                 String str_value,
                 int int_value,
                 String filename,
                 int line)
   {
      m_type = type;
      m_string = str_value;
      m_integer = int_value;
      m_filename = filename;
      m_line = line;
   }

   public static Token intToken(int value)
   {
      return intToken(value, "", 0);
   }

   public static Token idToken(String value)
   {
      return idToken(value, "", 0);
   }

   public static Token strToken(String value)
   {
      return strToken(value, "", 0);
   }

   public static Token intToken(int value,
                                String filename,
                                int line)
   {
      return new Token(INTEGER, null, value, filename, line);
   }

   public static Token strToken(String value,
                                String filename,
                                int line)
   {
      return new Token(STRING, value, 0, filename, line);
   }

   public static Token idToken(String value,
                               String filename,
                               int line)
   {
      return new Token(IDENTIFIER, value.intern(), 0, filename, line);
   }

   public final int tokenType()
   {
      return m_type;
   }

   public final int intValue()
   {
      return m_integer;
   }

   public final void setIntValue(int value) {
      m_integer = value;
   }

   public final String strValue()
   {
      return m_string;
   }

   public final String filename()
   {
      return m_filename;
   }

   public final int line()
   {
      return m_line;
   }

   public boolean equals(Token other)
   {
      if (m_type != other.m_type)
         return false;

      switch (m_type) {
      case STRING:
         return m_string.equals(other.m_string);

      case IDENTIFIER:
         return m_string == other.m_string;

      case INTEGER:
         return m_integer == other.m_integer;
      }

      // not reached
      if (Assert.DEBUG) Assert.impossible();
      return false;
   }

   public String asString()
   {
      switch (m_type) {
      case IDENTIFIER:
         return m_string;

      case STRING:
         return Misc.quoteString(m_string, Constants.QUOTE_CHAR);

      case INTEGER:
         return Integer.toString(m_integer);
      }

      // not reached
      if (Assert.DEBUG) Assert.impossible();
      return null;
   }

   public final void write(PrintWriter writer)
   {
      writer.print(asString());
   }

   public static String typeName(int type)
   {
      return TYPE_NAMES[type];
   }

   public String toString()
   {
      StringBuffer s = new StringBuffer();
      s.append(getClass());
      s.append("(");
      s.append(m_type);
      s.append(",");
      s.append(m_string);
      s.append(",");
      s.append(m_integer);
      s.append(",");
      s.append(m_filename);
      s.append(",");
      s.append(m_line);
      s.append(")");
      return s.toString();
   }

   public static final int STRING = 1;
   public static final int IDENTIFIER = 2;
   public static final int INTEGER = 3;
   private static String[] TYPE_NAMES = { "string", "identifier", "integer"};

   private int m_type;
   private String m_string;
   private int m_integer;
   private String m_filename;
   private int m_line;

   static final Token TOK_BEGIN = idToken("begin", null, 0);
   static final Token TOK_END   = idToken("end", null, 0);
}
