///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.utility;

import java.util.Vector;

public class Combining
{
  public static Vector indexCombinations(int length)
    {
      Integer[] indicies = new Integer[length];
      for (int i = 0; i < length; i++)
	{
	  indicies[i] = new Integer(i);
	}

      Vector tmp = new Vector();
      combinations(indicies, tmp);

      Vector result = new Vector();
      for (int i = 0; i < tmp.size(); i++)
	{
	  Object[] objects = (Object[])tmp.elementAt(i);
	  int[] ints = new int[objects.length];
	  for (int k = 0; k < objects.length; k++)
	    {
	      ints[k] = ((Integer)objects[k]).intValue();
	    }
	  result.addElement(ints);
	}

      return result;
    }

  public static void combinations(Object[] items,
				  Vector result)
    {
      for (int i = 1; i <= items.length; i++)
	{
	  combinations(items, 0, i, result);
	}
    }

  public static void combinations(Object[] items,
				  int n,
				  Vector result)
    {
      combinations(items, 0, n, result);
    }

  public static void combinations(Object[] items,
				  int first,
				  int n,
				  Vector result)
    {
      if (n == 1)
	{
	  for (int i = first; i < items.length; i++)
	    {
	      Object[] tmp = new Object[1];
	      tmp[0] = items[i];
	      result.addElement(tmp);
	    }
	}
      else if (n == 2)
	{
	  for (int i = first; i < items.length - 1; i++)
	    {
	      for (int k = i + 1; k < items.length; k++)
		{
		  Object[] tmp = new Object[2];
		  tmp[0] = items[i];
		  tmp[1] = items[k];
		  result.addElement(tmp);
		}
	    }
	}
      else
	{
	  for (int i = first; i < items.length - n + 1; i++)
	    {
	      Vector tmp = new Vector();
	      combinations(items, i + 1, n - 1, tmp);
	      for (int k = 0; k < tmp.size(); k++)
		{
		  Object[] r = new Object[n];
		  r[0] = items[i];
		  Object[] c = (Object[])tmp.elementAt(k);
		  System.arraycopy(c, 0, r, 1, c.length);
		  result.addElement(r);
		}
	    }
	}
    }

  public static void print(Object[] c)
    {
      for (int k = 0; k < c.length; k++)
	{
	  System.out.print(" " + c[k].toString());
	}
    }

  public static void print(int n,
			   int len,
			   Vector result)
    {
      System.out.println("Combinations of " + len + " items " + n + " at a time.");
      for (int i = 0; i < result.size(); i++)
	{
	  System.out.print("Combination: " + (i + 1) + ": ");
	  Object[] c = (Object[])result.elementAt(i);
	  print(c);
	  System.out.println();
	}
    }

  public static void print(int len,
			   Vector result)
    {
      System.out.println("All combinations of " + len + " items.");
      for (int i = 0; i < result.size(); i++)
	{
	  System.out.print("Combination: " + (i + 1) + ": ");
	  Object[] c = (Object[])result.elementAt(i);
	  print(c);
	  System.out.println();
	}
    }

  public static void main(String[] args)
    {
      for (int i = 1; i <= args.length; i++)
	{
	  Vector result = new Vector();
	  combinations(args, i, result);
	  print(i, args.length, result);
	}

      Vector result = new Vector();
      combinations(args, result);
      print(args.length, result);
    }
}
