///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.utility;

import java.util.Hashtable;
import java.util.Enumeration;

public class CommandLine
{
   public CommandLine(boolean unix_style)
   {
      m_unixStyle = unix_style;
      m_options = new SuperHashtable();
   }

   private class Option {
      Option(String _text, int _type)
      {
         text = _text;
         type = _type;
         value = null;
      }

      String text;
      int    type;
      Object value;
   }

   public void addIntOption(String text)
   {
      m_options.put(text, new Option(text, INT_OPT));
   }

   public void addStringOption(String text)
   {
      m_options.put(text, new Option(text, STRING_OPT));
   }

   public void addFlagOption(String text)
   {
      m_options.put(text, new Option(text, FLAG_OPT));
   }

   private void clearOptions()
   {
      Enumeration en = m_options.elements();
      while (en.hasMoreElements()) {
         Option option = (Option)en.nextElement();
         option.value = null;
      }
   }

   private int parseUnix(String[] args,
                         int index,
                         char[] ch)
   throws Failed
   {
      for (int i = 1; i < ch.length; i++) {
         String text = new String(ch, i, ch.length - i);
         Option opt = (Option)m_options.get(text);
         if (opt == null) {
            throw new Failed(index, text);
         }
         if (opt.type == FLAG_OPT) {
            opt.value = new Object();
         } else {
            String arg = new String(ch, i + 1, ch.length - (i + 1));
            i = ch.length;  // ensure we exit for loop now
            if (arg.length() == 0) {
               arg = args[++index];
            }
            if (opt.type == INT_OPT) {
               opt.value = new Integer(Integer.parseInt(arg));
            } else {
               opt.value = arg;
            }
         }
      }
      return index;
   }

   private int parseNormal(String[] args,
                           int index,
                           char[] ch)
   throws Failed
   {
      String text = new String(ch, 1, ch.length - 1);
      Option opt = (Option)m_options.get(text);
      if (opt == null) {
         throw new Failed(index, text);
      }
      if (opt.type == FLAG_OPT) {
         opt.value = new Object();
      } else {
         String arg = args[++index];
         if (opt.type == INT_OPT) {
            opt.value = new Integer(Integer.parseInt(arg));
         } else {
            opt.value = arg;
         }
      }
      return index;
   }

   public int parse(String[] args)
   throws Failed
   {
      clearOptions();

      for (int index = 0; index < args.length; index++) {
         char[] ch = args[index].toCharArray();
         if (ch[0] != '-') {
            return index;
         }
         if (ch.length == 2 && ch[1] == '-') {
            return index + 1;
         }
         if (m_unixStyle) {
            index = parseUnix(args, index, ch);
         } else {
            index = parseNormal(args, index, ch);
         }
      }

      return args.length;
   }

   public boolean getFlag(String text,
                          boolean default_value)
   {
      Option opt = (Option)m_options.get(text);
      return (opt == null) ? default_value : (opt.value != null);
   }

   public String getString(String text,
                           String default_value)
   {
      Option opt = (Option)m_options.get(text);
      return (opt == null) ? default_value : (String)opt.value;
   }

   public int getInt(String text,
                     int default_value)
   {
      Option opt = (Option)m_options.get(text);
      return (opt == null) ? default_value : ((Integer)opt.value).intValue();
   }

   private boolean   m_unixStyle;
   private Hashtable m_options;

   private static final int INT_OPT    = 0;
   private static final int STRING_OPT = 1;
   private static final int FLAG_OPT   = 2;
}

