///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.utility;

import java.io.File;
import java.text.MessageFormat;

public class FileSequence
{
   public FileSequence(String first_pattern,
                       String other_pattern,
                       int last_num)
   {
      init(first_pattern, other_pattern, 0, last_num);
   }

   public FileSequence(String first_pattern,
                       String other_pattern,
                       int first_num,
                       int last_num)
   {
      init(first_pattern, other_pattern, first_num, last_num);
   }

   public FileSequence(String filename_pattern,
                       int last_num)
   {
      init(filename_pattern, filename_pattern, 1, last_num);
   }

   public FileSequence(String filename_pattern,
                       int first_num,
                       int last_num)
   {
      init(filename_pattern, filename_pattern, first_num, last_num);
   }
   
   private void init(String first_pattern,
                     String other_pattern,
                     int first_num,
                     int last_num)
   {
      m_firstNumber = first_num;
      m_current = 0;

      final int num_names = last_num - first_num + 1;
      m_files = new File[num_names];
      for (int i = first_num; i <= last_num; i++) {
         Object[] formatArgs = { new Integer(i) };
         if (i == first_num) {
            m_files[i - first_num] = new File(MessageFormat.format(first_pattern, formatArgs));
         } else {
            m_files[i - first_num] = new File(MessageFormat.format(other_pattern, formatArgs));
         }
      }
   }
   
   public int getNumFiles()
   {
      return m_files.length;
   }

   public File getFile(int number)
   {
      return m_files[number - m_firstNumber];
   }

   public void selectFile(int number)
   {
      m_current = number - m_firstNumber;
   }

   public File getFile()
   {
      return m_files[m_current];
   }

   public void shift()
   {
      final int last = m_files.length - 1;
      if (last > 0) {
         m_files[last].delete();
         for (int i = last - 1; i >= 0; i--) {
            m_files[i].renameTo(m_files[i+1]);
         }
      }
   }

   public void moveForward()
   {
      m_current++;
      if (m_current == m_files.length) {
         m_current = 0;
      }
   }

   public void moveBack()
   {
      if (m_current == 0) {
         m_current = m_files.length - 1;
      } else {
         m_current--;
      }
   }

   public void selectOldestFile()
   {
      selectFile(findOldestFile());
   }

   public int findOldestFile()
   {
      int oldest_index = 0;
      long oldest_date = Long.MAX_VALUE;
      for (int i = 0; i < m_files.length; i++) {
         File f = m_files[i];
         if (!f.exists()) {
            return i + m_firstNumber;
         }
         long mod_date = f.lastModified();
         if (mod_date < oldest_date) {
            oldest_index = i;
            oldest_date = mod_date;
         }
      }
      return oldest_index + m_firstNumber;
   }

   private int m_firstNumber;
   private File[] m_files;
   private int m_current;
}
