///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.utility;

import java.io.*;
import java.text.*;

public class LockFile
{
   public LockFile(String filename)
   {
      this(new File(filename));
   }

   public LockFile(String directory,
		   String filename)
   {
      this(new File(directory, filename));
   }

   private LockFile(File f)
   {
      m_file = f;
      m_created = false;
   }

   public synchronized void delete()
   {
      if (m_created) {
	 m_file.delete();
	 m_created = false;
      }
   }

   public synchronized boolean create(String lock_file_text)
      throws IOException
   {
      if (m_created) {
	 return true;
      }

      if (m_file.exists()) {
	 return false;
      }

      try {
	 FileWriter fw  = new FileWriter(m_file);
	 PrintWriter pw = new PrintWriter(fw);
	 try {
	    String[] wrapped_text = Misc.wrapString(lock_file_text, 80, 0, 0);
	    for (int i = 0; i < wrapped_text.length; i++) {
	       pw.println(wrapped_text[i]);
	    }
	    if (pw.checkError()) {
	       throw new IOException("error writing to lock file");
	    }
	 } finally {
	    pw.close();
	 }
      } catch (IOException ex) {
	 m_file.delete();
	 throw ex;
      }

      m_created = true;
      return true;
   }

   public String getPath()
   {
      return m_file.getPath();
   }

   public boolean create(String fmt,
			 Object[] args)
      throws IOException
   {
      return create(MessageFormat.format(fmt, args));
   }

   public void finalize() throws Throwable
   {
      super.finalize();
      delete();
   }

   private File m_file;
   private boolean m_created;
}
