///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.utility;

import java.io.File;

public abstract class Misc
{
   public static boolean stringInArray(String s,
                                       String[] a)
   {
      for (int i = 0; i < a.length; i++) {
         if (a[i].equals(s)) {
            return true;
         }
      }
      return false;
   }

   public static int findStringInArray(String s,
                                       String[] a,
				       int default_value)
   {
      for (int i = 0; i < a.length; i++) {
         if (a[i].equals(s)) {
            return i;
         }
      }
      return default_value;
   }

   public static String[] splitString(String text,
                                      char sep)
   {
      SuperVector vec = new SuperVector(text.length());
      char[] s = text.toCharArray();

      int start = 0;
      for (int i = 0; i < s.length; i++) {
         if (s[i] == sep) {
            vec.addElement(new String(s, start, i - start));
            start = i + 1;
         }
      }

      if (start < s.length) {
         vec.addElement(new String(s, start, s.length - start));
      }

      return vec.getStrings();
   }

   public static String joinStrings(String[] strings,
                                    char sep)
   {
      if (strings.length == 0) {
         return "";
      }

      int length = strings.length - 1;
      for (int i = 0; i < strings.length; i++) {
         length += strings[i].length();
      }
      
      int offset = 0;
      
      char[] buffer = new char[length];
      for (int i = 0; i < strings.length; i++) {
         if (i > 0) {
            buffer[offset++] = sep;
         }
         char[] tmp = strings[i].toCharArray();
         System.arraycopy(tmp, 0, buffer, offset, tmp.length);
         offset += tmp.length;
      }

      return new String(buffer);
   }

   public static String joinStrings(String s1,
                                    String s2,
                                    char sep)
   {
      String[] tmp = { s1, s2 };
      return joinStrings(tmp, sep);
   }

   public static synchronized String getInstallRoot()
   {
     if (s_install_root == null) {
       for (int i = 0; i < INSTALL_ROOT_PROPERTIES.length; ++i) {
         String propname = INSTALL_ROOT_PROPERTIES[i];
         s_install_root = System.getProperty(propname, "");
         if (s_install_root.length() > 0) {
           if (!s_install_root.endsWith(File.separator)) {
             s_install_root += File.separator;
           }
           break;
         }
       }
     }
     return s_install_root;
   }

   public static String escapeCharInString(String in,
                                           char special_char,
                                           String special_str)
   {
      StringBuffer out = new StringBuffer(2 * in.length());
      char[] ch = in.toCharArray();
      for (int i = 0; i < ch.length; i++) {
         if (ch[i] == special_char) {
            out.append(special_str);
         } else {
            out.append(ch[i]);
         }
      }
      return out.toString();
   }

   public static String quoteString(String in,
                                    char quote_char)
   {
      StringBuffer out = new StringBuffer(2 * in.length() + 2);
      out.append(quote_char);
      char[] ch = in.toCharArray();
      for (int i = 0; i < ch.length; i++) {
         char c = ch[i];
         if (c == '\\') {
            out.append("\\\\");
         } else if (c == quote_char) {
            out.append('\\');
            out.append(quote_char);
         } else {
            out.append(c);
         }
      }
      out.append(quote_char);
      return out.toString();
   }

   private static char[] mergeStringsIntoChars(String[] strings)
   {
      int length = 0;
      for (int i = 0; i < strings.length; i++) {
         length += strings[i].length();
      }

      char[] merged = new char[length];
      int offset = 0;
      for (int i = 0; i < strings.length; i++) {
         strings[i].getChars(0, strings[i].length(), merged, offset);
         offset += strings[i].length();
      }
      return merged;
   }

   private static String makeString(char[] chars,
                                    int offset,
                                    int width,
                                    int indent)
   {
      StringBuffer buffer = new StringBuffer();
      for (int i = 0; i < indent; i++) {
         buffer.append(' ');
      }
      buffer.append(chars, offset, width);
      String answer = buffer.toString();
      return answer;
   }

   private static int findWrapPoint(char[] chars,
                                    int first,
                                    int last)
   {
      int i = last;
      while (i > first && chars[i] != ' ') {
         // looking for first space
         --i;
      }
      return (i == first) ? last : i;
   }

   public static String[] wrapChars(char[] chars,
                                    int max_width,
                                    int first_indent,
                                    int other_indent)
   {
      SuperVector vec = new SuperVector(chars.length / max_width + 1);
      int offset = 0;
      int indent = first_indent;
      int next_wrap = max_width - indent;
      for (int i = 0; i < chars.length; i++) {
         char c = chars[i];
         if (c == '\n') {
            // break the line at the newline character
            vec.addElement(makeString(chars, offset, i - offset, indent));

            // adjust for rest of array
            offset = i + 1;
            indent = other_indent;
            next_wrap = offset + max_width - indent;
         } else if (i == next_wrap) {
            // break the line at the current index of first preceeding space
            i = findWrapPoint(chars, offset, i);
            vec.addElement(makeString(chars, offset, i - offset + 1, indent));
            
            // skip spaces immediately following the wrap point
            int k = i + 1;
            while (k < chars.length && chars[k] == ' ') {
               k++;
            }
            i = k - 1;

            // adjust for rest of array
            offset = i + 1;
            indent = other_indent;
            next_wrap = offset + max_width - indent;
         }
      }
      if (offset < chars.length) {
         vec.addElement(makeString(chars, offset, chars.length - offset, indent));
      }
      return vec.getStrings();
   }

   public static String[] wrapStrings(String[] strings,
                                      int max_width,
                                      int first_indent,
                                      int other_indent)
   {
      return wrapChars(mergeStringsIntoChars(strings), max_width, first_indent, other_indent);
   }

   public static String[] wrapString(String string,
                                     int max_width,
                                     int first_indent,
                                     int other_indent)
   {
      return wrapChars(string.toCharArray(), max_width, first_indent, other_indent);
   }

   public static String prettyBytes(byte[] data,
				    int data_length,
				    int max_length)
   {
      int length = Math.min(max_length - 2, data_length);
      char[] chars = new char[length + 2];

      if (data_length > length) {
	 length--;
      }

      final char QUOTE_CHAR       = '"';
      final char UNPRINTABLE_CHAR = '.';
      final char MORE_AVAIL_CHAR  = '>';

      int dst = 0;
      chars[dst++] = QUOTE_CHAR;
      for (int src = 0; src < length; src++) {
	 byte b = data[src];
	 if (b < 32 || b == (byte)'"') {
	    chars[dst++] = UNPRINTABLE_CHAR;
	 } else {
	    chars[dst++] = (char)b;
	 }
      }
      if (data_length > length) {
	 chars[dst++] = MORE_AVAIL_CHAR;
      }
      chars[dst++] = QUOTE_CHAR;

      if (Assert.DEBUG) Assert.inv(dst == chars.length);
      return new String(chars);
   }

   private static String s_install_root = null;

   public static final String[] INSTALL_ROOT_PROPERTIES = { "install.root", "lax.root.install.dir" };
}
