///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.utility;
import java.util.Vector;
import java.util.Enumeration;
import java.net.Socket;
import java.io.DataOutputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;

/**
  Trivial class for sending email via SMTP.
  Provides only simplistic error handling, but should get the
  job done for simple tasks such as web based "contact email" forms.
 */
public class SimpleMailer
{
    public SimpleMailer(String fromHost,
			String fromUser,
			String smtpHost)
    {
	m_fromHost = fromHost;
	m_fromUser = fromUser;
	m_smtpHost = smtpHost;
	m_targets  = new Vector();
    }

    public void addTarget(String target)
    {
	m_targets.addElement(target);
    }

    private void sendCommand(String command,
			     String response)
	throws SmtpError, IOException
    {
	if (DEBUG) {
	    System.out.println("sending command: " + command);
	}
	char[] data = new char[command.length() + 2];
	m_output.writeBytes(command);
	m_output.write(CRLF, 0, CRLF.length);
	m_output.flush();
	if (DEBUG) {
	    System.out.println("expecting reply: " + response);
	}
	String reply = m_input.readLine();
	if (DEBUG) {
	    System.out.println("received reply: " + reply);
	}
	if (!reply.regionMatches(0, response, 0, response.length()))
	    throw new SmtpError("Expected '" + response + "' received '" + reply + "'");
    }

    private void sendCommand(String command,
			     String response1,
			     String response2)
	throws SmtpError, IOException
    {
	if (DEBUG) {
	    System.out.println("sending command: " + command);
	}
	char[] data = new char[command.length() + 2];
	m_output.writeBytes(command);
	m_output.write(CRLF, 0, CRLF.length);
	m_output.flush();
	if (DEBUG) {
	    System.out.println("expecting reply: " + response1);
	    System.out.println("             or: " + response2);
	}
	String reply = m_input.readLine();
	if (DEBUG) {
	    System.out.println("received reply: " + reply);
	}
	if (!reply.regionMatches(0, response1, 0, response1.length()) &&
	    !reply.regionMatches(0, response2, 0, response2.length()))
	    throw new SmtpError("Expected '" + response1 + "' or '" + response2 + "' received '" + reply + "'");
    }

    private void getStart()
	throws IOException
    {
	if (DEBUG) {
	    System.out.println("waiting for hello from server");
	}
	String reply = m_input.readLine();
	if (DEBUG) {
	    System.out.println("received reply: " + reply);
	}
    }

    private void sendMessage(String[] data,
			     String response)
	throws SmtpError, IOException
    {
	for (int i = 0; i < data.length; i++) {
	    if (data[i].length() > 0 && data[i].charAt(0) == '.')
		data[i] = "." + data[i];
	    if (DEBUG) {
		System.out.println("sending text: " + data[i]);
	    }
	    m_output.writeBytes(data[i]);
	    m_output.write(CRLF, 0, CRLF.length);
	}
	m_output.write(EOD, 0, EOD.length);
	m_output.flush();
	if (DEBUG) {
	    System.out.println("expecting reply: " + response);
	}
	String reply = m_input.readLine();
	if (DEBUG) {
	    System.out.println("received reply: " + reply);
	}
	if (!reply.regionMatches(0, response, 0, response.length()))
	    throw new SmtpError("Expected '" + response + "' received '" + reply + "'");
    }

    public void send(String message)
	throws SmtpError, IOException
    {
	String[] lines = new String[1];
	lines[0] = message;
	send(lines);
    }

    public synchronized void send(String[] lines)
	throws SmtpError, IOException
    {
	Socket server = null;
	try {
	    server   = new Socket(m_smtpHost, SMTP_PORT);
	    m_input  = new BufferedReader(new InputStreamReader((server.getInputStream())));
	    m_output = new DataOutputStream(server.getOutputStream());
	    getStart();
	    sendCommand("HELO " + m_fromHost, "250");
	    sendCommand("MAIL FROM: " + m_fromUser, "250");
	    Enumeration e = m_targets.elements();
	    while (e.hasMoreElements()) {
		String target = (String)e.nextElement();
		sendCommand("RCPT TO: " + target, "250", "251");
	    }
	    sendCommand("DATA", "354");
	    sendMessage(lines, "250");
	    sendCommand("QUIT", "221");
	} finally {
	    if (server != null)
		server.close();
	    m_input  = null;
	    m_output = null;
	}
    }

    public static void main(String[] args)
    {
	SimpleMailer m = new SimpleMailer(args[0], args[1], args[2]);
	try {
	    for (int i = 3; i < args.length; i++)
		m.addTarget(args[i]);
	    String[] messages = new String[7];
	    messages[0] = "Subject: test message";
	    messages[1] = "";
	    messages[2] = ".line 1: this is a test message from Java(tm)";
	    messages[3] = "..line 2: this is a test message from Java(tm)";
	    messages[4] = "...line 3: this is a test message from Java(tm)";
	    messages[5] = "....line 4: this is a test message from Java(tm)";
	    messages[6] = "line 4: this is a test message from Java(tm)";
	    m.send(messages);
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    private String m_fromHost;
    private String m_fromUser;
    private String m_smtpHost;
    private Vector m_targets;
    private BufferedReader  m_input;
    private DataOutputStream m_output;

    private static final short SMTP_PORT = 25;
    private static final byte[] CRLF = { 13, 10 };
    private static final byte[] EOD = { 13, 10, 46, 13, 10 };
    private static final boolean DEBUG = false;
}
