///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.utility;

public class Tuple
{
   public Tuple(int size)
   {
      m_elements = new Object[size];
   }

   public Tuple(Object a,
                Object b)
   {
      this(2);
      m_elements[0] = a;
      m_elements[1] = b;
   }

   public Tuple(Object a,
                Object b,
                Object c)
   {
      this(3);
      m_elements[0] = a;
      m_elements[1] = b;
      m_elements[2] = c;
   }

   public Tuple(Object a,
                Object b,
                Object c,
		Object d)
   {
      this(4);
      m_elements[0] = a;
      m_elements[1] = b;
      m_elements[2] = c;
      m_elements[3] = d;
   }

   public void setElement(int index,
                          Object o)
   {
      m_elements[index] = o;
   }

   public Object getElement(int index)
   {
      return m_elements[index];
   }

   public String toString()
   {
      StringBuffer buffer = new StringBuffer();
      buffer.append(getClass().getName());
      buffer.append("{");
      for (int i = 0; i < m_elements.length; i++) {
         if (i > 0) {
            buffer.append(",");
         }
         buffer.append(m_elements[i].toString());
      }
      buffer.append("}");
      return buffer.toString();
   }

   public int hashCode()
   {
      int hash = 0;

      for (int i = 0; i < m_elements.length; i++) {
         int tmp = (hash << 8) | (hash >>> 24);
         hash = tmp ^ m_elements[i].hashCode();
      }

      return hash;
   }

   public boolean equals(Object o)
   {
      try {
         Tuple other = (Tuple)o;

         if (m_elements.length != other.m_elements.length) {
            return false;
         }

         for (int i = 0; i < m_elements.length; i++) {
            if (!m_elements[i].equals(other.m_elements[i])) {
               return false;
            }
         }

         return true;
      } catch (ClassCastException ex) {
         return false;
      }
   }

   private Object[] m_elements;
}
