///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.utility;

import java.io.*;
import java.util.zip.*;

public abstract class Zipper
{
   public static void zipFile(String rootdir,
                              String filename,
                              ZipOutputStream out)
   throws IOException
   {
      if (Assert.DEBUG) Assert.pre(rootdir.length() > 0);
      if (Assert.DEBUG) Assert.pre(filename.length() > 0);
      out.putNextEntry(new ZipEntry(filename));
      File f = new File(rootdir, filename);
      FileInputStream in_f = new FileInputStream(f);
      BufferedInputStream in = new BufferedInputStream(in_f);
      try {
         byte[] buffer = new byte[4096];
         for (int nb = in.read(buffer); nb > 0; nb = in.read(buffer)) {
            out.write(buffer, 0, nb);
         }
      } finally {
         in.close();
      }
      out.closeEntry();
   }

   public static void zipDirectory(String rootdir,
                                   String dirname,
                                   boolean recursive,
                                   ZipOutputStream out)
   throws IOException
   {
      if (Assert.DEBUG) Assert.pre(rootdir.length() > 0);
      if (Assert.DEBUG) Assert.pre(dirname.length() > 0);
            
      String new_root = dirname + File.separator;
      File d = new File(rootdir, dirname);
      if (!d.exists()) {
         throw new FileNotFoundException(rootdir + dirname);
      }

      String[] filenames = d.list();
      for (int i = 0; i < filenames.length; i++) {
         File f = new File(d, filenames[i]);
         if (f.isFile()) {
            zipFile(rootdir, new_root + filenames[i], out);
         } else if (recursive && f.isDirectory()) {
            zipDirectory(rootdir, new_root + filenames[i], true, out);
         }
      }
   }          

   public static void zip(String zipfilename,
                          String root_dirname,
                          String dirname)
   throws IOException
   {
      if (Assert.DEBUG) Assert.pre(root_dirname.length() > 0);
      if (Assert.DEBUG) Assert.pre(dirname.length() > 0);
      FileOutputStream out_f = new FileOutputStream(zipfilename);
      BufferedOutputStream out_b = new BufferedOutputStream(out_f);
      ZipOutputStream out = new ZipOutputStream(out_b);
      try {
         zipDirectory(root_dirname, dirname, true, out);
      } finally {
         out.close();
      }
   }

   public static void main(String[] args)
   throws IOException
   {
      if (args.length != 3) {
         System.err.println("usage: Zipper zipfile rootdir zipdir");
         System.exit(1);
      }
      zip(args[0], args[1], args[2]);
   }
}

