///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.utility.logging;

import java.text.MessageFormat;
import org.cooldevtools.utility.Assert;

public abstract class AbstractLogger
implements LoggerInterface
{
   AbstractLogger()
   {
      m_minPriority = MessagePriority.LOG_DEBUG;
   }

   public synchronized void setMinPriority(MessagePriority min_priority)
   {
      m_minPriority = min_priority;
   }

   public boolean isPriorityBelowMinimum(MessagePriority priority)
   {
      return priority.lessThan(m_minPriority);
   }

   protected synchronized MessagePriority getMinPriority()
   {
      return m_minPriority;
   }

   public void logSilently(MessagePriority priority,
                           String message)
   {
      try {
         log(priority, message);
      } catch (LoggingException ex) {
      }
   }

   public void log(MessagePriority priority,
                   String message,
		   Object[] args)
   throws LoggingException
   {
      log(priority, MessageFormat.format(message, args));
   }

   public void logSilently(MessagePriority priority,
                           String message,
			   Object[] args)
   {
      try {
	 if (Assert.DEBUG) {
	    Assert.pre(message != null);
	    //System.out.println("message: " + message);
	    for (int i = 0; i < args.length; i++) {
	       Assert.pre(args[i] != null);
	       //System.out.println("args[" + i + "]: " + args[i]);
	    }
	 }
         log(priority, message, args);
      } catch (LoggingException ex) {
      } catch (Exception ex2) {
	 try {
	    log(MessagePriority.LOG_ERROR, "caught exception while logging: " + ex2.toString());
	    log(priority, message);
	 } catch (Exception ex3) {
	 }
      }
   }

   public void reconnectSilently()
   {
      try {
         reconnect();
      } catch (LoggingException ex) {
         
      }
   }

   private MessagePriority m_minPriority;
}
