///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.utility.logging;

import java.awt.*;
import java.awt.event.*;
import java.util.Date;
import javax.swing.*;
import javax.swing.text.*;
import org.cooldevtools.utility.*;

public class DocumentLogger
extends AbstractLogger
implements Runnable
{
   public DocumentLogger(Document target)
   throws LoggingException
   {
      m_document = target;
      m_queue = new LinkedList();
      m_maxLength = 60 * 1024;
   }

   public synchronized void run()
   {
      try {
         while (!m_queue.isEmpty()) {
            String text = (String)m_queue.pop();
            int length = m_document.getLength();
            if (length >= m_maxLength) {
               m_document.remove(0, length);
               length = m_document.getLength();
            }
            if (text == CLEAR_MESSAGE) {
               m_document.remove(0, length);
            } else {
               m_document.insertString(length, text, null);
            }
         }
      } catch (BadLocationException ex) {
      }
   }

   public synchronized void clear()
   {
      m_queue.append(CLEAR_MESSAGE);
      SwingUtilities.invokeLater(this);
   }

   public synchronized void log(MessagePriority priority,
                                String message)
   throws LoggingException
   {
      if (isPriorityBelowMinimum(priority)) {
         return;
      }

      StringBuffer buffer = new StringBuffer();
      buffer.append(new Date().toString());
      buffer.append(":");
      buffer.append(priority.getName());
      buffer.append(":");
      buffer.append(message);
      buffer.append("\n");

      m_queue.append(buffer.toString());
      SwingUtilities.invokeLater(this);
   }

   public synchronized void close()
   {
      m_queue.clear();
   }

   public synchronized void reconnect()
   throws LoggingException
   {
   }

   private LinkedList m_queue;
   private Document   m_document;
   private int        m_maxLength;

   private static final String CLEAR_MESSAGE = "\0DocuMentLogger::CleAr\0";
}
