///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.utility.logging;

import java.io.*;
import org.cooldevtools.utility.FileSequence;

public class FileLogger
extends StreamLogger
{
   public FileLogger(String filename)
      throws LoggingException
   {
      this(filename, 1);
   }

   public FileLogger(String filename_pattern,
                     int num_files)
      throws LoggingException
   {
      super((OutputStream)null);
      m_isToConsole = filename_pattern.equals("-");
      String pattern = adjustPattern(filename_pattern, num_files);
      m_fileSequence = new FileSequence(filename_pattern, pattern, num_files);
      m_numLines = 0;
      reconnect();
   }

   public boolean isToConsole()
   {
      return m_isToConsole;
   }

   private String adjustPattern(String filename_pattern,
                                int num_files)
   {
      if (num_files == 1) {
         return filename_pattern;
      }

      if (filename_pattern.indexOf(NUMBER_PATTERN) >= 0) {
         return filename_pattern;
      }

      int dot_index = filename_pattern.lastIndexOf(".");
      int slash_index = filename_pattern.lastIndexOf(File.pathSeparator);
      if (dot_index < 0 || dot_index < slash_index ) {   
         return filename_pattern + "." + NUMBER_PATTERN;
      }

      StringBuffer b = new StringBuffer();
      b.append(filename_pattern.substring(0, dot_index));
      b.append("_");
      b.append(NUMBER_PATTERN);
      b.append(filename_pattern.substring(dot_index));
      return b.toString();
   }

   public synchronized void log(MessagePriority priority,
                                String message)
   throws LoggingException
   {
      super.log(priority, message);
      if (m_numLines++ >= MAX_LINES_PER_FILE) {
	 reconnect();
      }
   }

   public synchronized void close()
   {
      if (!m_isToConsole) {
	 super.close();
      }
   }

   public synchronized void reconnect()
   throws LoggingException
   {
      if (m_isToConsole && getStream() != null) {
	 m_numLines = 0;
	 return;
      }
      try {
         close();
         m_fileSequence.shift();
         String filename = m_fileSequence.getFile().getPath();
         if (filename.equals("-")) {
            setStream(new PrintWriter(new OutputStreamWriter(System.out)));
         } else {
            setStream(new PrintWriter(new BufferedWriter(new FileWriter(filename))));
         }
         m_numLines = 0;
      } catch (IOException ex) {
         throw new LoggingException("file open failed: " + ex.getMessage());
      }
   }

   private int m_numLines;
   private FileSequence m_fileSequence;
   private boolean m_isToConsole;
   private static final String NUMBER_PATTERN = "{0,number,integer}";
   private static final int MAX_LINES_PER_FILE = 2500;
}
