///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.utility.logging;

public abstract class Logger
{
   public static void initSingleton(LoggerInterface instance)
   {
      s_instance = instance;
   }

   public static LoggerInterface s()
   {
      return s_instance;
   }

   public static boolean isToConsole()
   {
      try {
	 FileLogger logger = (FileLogger)s_instance;
	 return logger.isToConsole();
      } catch (ClassCastException ex) {
	 return false;
      }
   }

   public static boolean isNull()
   {
      LoggerInterface logger = s_instance;
      return (logger == null) || (logger instanceof NullLogger);
   }

   public static void restoreDefaultLogger()
      throws LoggingException
   {
      s_defaultLogger.reconnect();
      initSingleton(s_defaultLogger);
   }

   public static void setDefaultLogger(LoggerInterface logger)
   {
      if (logger == null) {
	 s_defaultLogger = new NullLogger();
      } else {
	 s_defaultLogger = logger;
      }
   }

   private static LoggerInterface s_instance = new NullLogger();
   private static LoggerInterface s_defaultLogger = new NullLogger();
}
