///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.utility.logging;

public interface LoggerInterface
{
   /**
      Tells the logger to ignore messages of lower priority than min_priority.
    */
   public void setMinPriority(MessagePriority min_priority);

   /**
      Log a message with the specified priority.
    */
   public void log(MessagePriority priority,
                   String message)
   throws LoggingException;

   /**
      Tells the logger to "close" its underlying target.
      This might mean closing a file, closing connection to
      a syslogd, etc.
    */
   public void close();

   /**
      Tells the logger to "reconnect" to its underlying log.
      This might mean closing and reopening a file, reconnecting to
      a syslogd, etc.
    */
   public void reconnect()
   throws LoggingException;

   /**
      Log a message with the specified priority.
      Never throws an exception, quietly ignores logging errors.
    */
   public void logSilently(MessagePriority priority,
                           String message);

   /**
      Tells the logger to "reconnect" to its underlying log.
      This might mean closing and reopening a file, reconnecting to
      a syslogd, etc.
      Never throws an exception, quietly ignores logging errors.
    */
   public void reconnectSilently();

   public void log(MessagePriority priority,
                   String message,
		   Object[] args)
      throws LoggingException;

   public void logSilently(MessagePriority priority,
                           String message,
			   Object[] args);

   public boolean isPriorityBelowMinimum(MessagePriority priority);
}
