///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.utility.logging;

import java.io.*;
import java.util.Date;

public class StreamLogger
extends AbstractLogger
{
   public StreamLogger(OutputStream stream)
   {
      setStream(stream);
   }

   public StreamLogger(PrintWriter stream)
   {
      setStream(stream);
   }

   protected synchronized void setStream(PrintWriter stream)
   {
      close();
      m_stream = stream;
   }

   protected synchronized void setStream(OutputStream stream)
   {
      close();
      if (stream != null) {
         m_stream = new PrintWriter(stream);
      }
   }

   protected synchronized PrintWriter getStream()
   {
      return m_stream;
   }

   public void log(MessagePriority priority,
                   String message)
   throws LoggingException
   {
      if (isPriorityBelowMinimum(priority)) {
         return;
      }

      StringBuffer buffer = new StringBuffer();
      buffer.append(new Date().toString());
      buffer.append(":");
      buffer.append(priority.getName());
      buffer.append(":");
      buffer.append(message);

      synchronized (this) {
         m_stream.println(buffer.toString());
         if (m_stream.checkError()) {
            throw new LoggingException("write failed");
         }
      }
   }

   public synchronized void close()
   {
      if (m_stream != null) {
         m_stream.close();
         m_stream = null;
      }
   }

   public synchronized void reconnect()
   throws LoggingException
   {
   }

   private PrintWriter m_stream;
}
