///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.utility.logging;

import java.io.*;
import java.net.*;

public class SyslogLogger
extends AbstractLogger
{
   public SyslogLogger(SyslogFacility facility,
                       String hostname)
   throws LoggingException
   {
      this("", facility, hostname, SYSLOGD_PORT);
   }

   public SyslogLogger(String ident,
                       SyslogFacility facility,
                       String hostname)
   throws LoggingException
   {
      this(ident, facility, hostname, SYSLOGD_PORT);
   }

   public SyslogLogger(String ident,
                       SyslogFacility facility,
                       String hostname,
                       int port)
   throws LoggingException
   {
      m_ident = ident;
      m_facility = facility;
      m_hostname = hostname;
      m_port = port;
      reconnect();
   }

   public synchronized void log(MessagePriority priority,
                                String message)
   throws LoggingException
   {
      if (isPriorityBelowMinimum(priority)) {
         return;
      }

      StringBuffer buffer = new StringBuffer();
      buffer.append("<");
      buffer.append(String.valueOf(m_facility.getCode() | priority.getIndex()));
      buffer.append(">");
      buffer.append(m_ident);
      buffer.append(message);

      byte[] msg = buffer.toString().getBytes();
      DatagramPacket packet = new DatagramPacket(msg,
                                                 msg.length,
                                                 m_address,
                                                 m_port);

      try {
         m_socket.send(packet);
      } catch (IOException ex) {
         throw new LoggingException("send failed: " + ex.getMessage());
      }
   }

   public synchronized void close()
   {
      if (m_socket != null) {
         m_socket.close();
         m_socket = null;
      }
   }

   public synchronized void reconnect()
   throws LoggingException
   {
      try {
         m_address = InetAddress.getByName(m_hostname);
      } catch (UnknownHostException ex) {
         throw new LoggingException("unknown host: " + m_hostname + ": " + ex.getMessage());
      }
      try {
         m_socket = new DatagramSocket();
      } catch (SocketException ex) {
         throw new LoggingException("socket exception: " + ex.getMessage());
      }
   }

   public static void main(String[] args)
   throws LoggingException
   {
      if (args.length < 2) {
         System.err.println("usage: SyslogLogger hostname message...");
         System.exit(1);
      }

      LoggerInterface logger = new SyslogLogger("SyslogLogger: ", SyslogFacility.LOG_LOCAL0, args[0]);

      for (int i = 1; i < args.length; i++) {
         logger.log(MessagePriority.LOG_ERROR, args[i]);
      }

      logger.close();
   }

   private SyslogFacility   m_facility;
   private String           m_ident;
   private String           m_hostname;
   private int              m_port;
   private InetAddress      m_address;
   private DatagramSocket   m_socket;

   private static final int SYSLOGD_PORT = 514;
}
