///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.utility.storage;

import java.util.Vector;

public class FieldMeta {
   public FieldMeta(String name,
                    int type)
   {
      this(name, -1, type);
      if ((type == TYPE_STRING) || (type == TYPE_USTRING)) {
         throw new RuntimeException("string fields require length");
      }
   }

   public FieldMeta(String name,
                    int length,
                    int type)
   {
      m_name   = name;
      m_type   = type;

      switch (type) {
      case TYPE_STRING:
         m_length = length + 4;
         m_maxStringLength = length;
         break;

      case TYPE_USTRING:
         m_length = 2 * length + 4;
         m_maxStringLength = length;
         break;

      case TYPE_SHORT:
         m_length = 2;
         m_maxStringLength = 1;
         break;

      case TYPE_INT:
         m_length = 4;
         m_maxStringLength = 2;
         break;

      case TYPE_LONG:
         m_length = 8;
         m_maxStringLength = 4;
         break;

      case TYPE_BLOB:
         m_length = 4;
         m_maxStringLength = -1;
         break;

      default:
         throw new RuntimeException("invalid field type");
      }
   }

   public String getName()
   {
      return m_name;
   }

   public int getLength()
   {
      return m_length;
   }

   public int getMaxStringLength()
   {
      return m_maxStringLength;
   }

   public int getType()
   {
      return m_type;
   }

   public static String shortToString(short num)
   {
      char[] tmp = new char[1];
      tmp[0] = (char)num;
      return new String(tmp);
   }

   public static String intToString(int num)
   {
      char[] tmp = new char[2];
      tmp[0] = (char)(num >>> 16);
      tmp[1] = (char)(num & 0x0000ffff);
      return new String(tmp);
   }

   public static String longToString(long num)
   {
      char[] tmp = new char[4];
      tmp[0] = (char) (num >>> 48);
      tmp[1] = (char)((num & 0x0000ffff00000000L) >>> 32);
      tmp[2] = (char)((num & 0x00000000ffff0000L) >>> 16);
      tmp[1] = (char) (num & 0x000000000000ffffL);
      return new String(tmp);
   }

   public static FieldMeta[] toArray(Vector v)
   {
      synchronized (v) {
         FieldMeta[] answer = new FieldMeta[v.size()];
         v.copyInto(answer);
         return answer;
      }
   }

   private String m_name;
   private int    m_length;
   private int    m_type;
   private int    m_maxStringLength;

   public static final int TYPE_STRING  = 0;
   public static final int TYPE_USTRING = 1;
   public static final int TYPE_INT     = 2;
   public static final int TYPE_SHORT   = 3;
   public static final int TYPE_LONG    = 4;
   public static final int TYPE_BLOB    = 5;
}
