///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.utility.storage;

import java.util.Vector;

public class IndexMeta
{
   public IndexMeta(String name,
                    RecordMeta record,
                    String[] fields)
   {
      init(name, record, fields);
   }

   public void init(String name,
                    RecordMeta record,
                    String[] fields)
   {
      m_name = name;
      m_meta = record;
      m_length = 0;
      m_fields = new int[fields.length];
      for (int i = 0; i < fields.length; i++) {
         record.setIsIndexed(fields[i], true);
         m_fields[i] = record.getFieldIndex(fields[i]);
         m_length += 1 + record.getField(m_fields[i]).getMaxStringLength();
      }
   }

   public String getName()
   {
      return m_name;
   }

   public int getKeyLength()
   {
      return m_length;
   }

   public String computeKey(Record rec)
   {
      StringBuffer buffer = new StringBuffer();
      for (int i = 0; i < m_fields.length; i++) {
         buffer.append(rec.getString(m_fields[i]));
         buffer.append(KEY_SEP_CHAR);
      }
      return buffer.toString();
   }

   public static IndexMeta[] toArray(Vector v)
   {
      synchronized (v) {
         IndexMeta[] answer = new IndexMeta[v.size()];
         v.copyInto(answer);
         return answer;
      }
   }

   private String m_name;
   private RecordMeta m_meta;
   private int[] m_fields;
   private int m_length;

   public static final char KEY_SEP_CHAR = (char)0;
}
