///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.utility.storage;

import java.io.IOException;
import java.util.Hashtable;

public class RecordMeta
{
  public RecordMeta(FieldMeta[] fields)
    {
      init(fields);
    }

  public void init(FieldMeta[] fields)
    {
      m_fields = fields;
      m_length = 0;

      m_fieldsByName = new Hashtable();
      m_offsets = new int[fields.length];
      m_indexedField = new boolean[fields.length];

      for (int i = 0; i < fields.length; i++)
	{
	  m_fieldsByName.put(fields[i].getName(), new Integer(i));
	  m_offsets[i] = m_length;
	  m_length += fields[i].getLength();
	}
    }

  public boolean isIndexed(int index)
    {
      return m_indexedField[index];
    }

  public void setIsIndexed(String name,
			   boolean value)
    {
      m_indexedField[((Integer)m_fieldsByName.get(name)).intValue()] = value;
    }

  public int getFieldIndex(String name)
    {
      return ((Integer)m_fieldsByName.get(name)).intValue();
    }

  public FieldMeta getField(String name)
    {
      return m_fields[((Integer)m_fieldsByName.get(name)).intValue()];
    }

  public FieldMeta getField(int index)
    {
      return m_fields[index];
    }

  public FieldMeta[] getFields()
    {
      return m_fields;
    }

  public int getNumFields()
    {
      return m_fields.length;
    }

  public int getLength()
    {
      return m_length;
    }

  public short getShortFromBuffer(int field_index,
		                  byte[] buffer)
    {
      return (short)getLongFromBuffer(field_index, buffer);
    }

  public int getIntFromBuffer(int field_index,
			      byte[] buffer)
    {
      return (int)getLongFromBuffer(field_index, buffer);
    }

  public long getLongFromBuffer(int field_index,
	                        byte[] buffer)
    {
      FieldMeta field = m_fields[field_index];
      switch (field.getType())
	{
	case FieldMeta.TYPE_STRING:
        case FieldMeta.TYPE_USTRING:
	  return Long.parseLong(Packer.getString(buffer, m_offsets[field_index]));

	case FieldMeta.TYPE_SHORT:
	  return Packer.getShort(buffer, m_offsets[field_index]);

	case FieldMeta.TYPE_INT:
	  return Packer.getInt(buffer, m_offsets[field_index]);

        case FieldMeta.TYPE_LONG:
          return Packer.getLong(buffer, m_offsets[field_index]);

	default:
	  throw new RuntimeException();
	}
    }

  public String getStringFromBuffer(int field_index,
				    byte[] buffer)
    {
      FieldMeta field = m_fields[field_index];
      switch (field.getType())
	{
	case FieldMeta.TYPE_STRING:
	  return Packer.getString(buffer, m_offsets[field_index]);

        case FieldMeta.TYPE_USTRING:
          return Packer.getUString(buffer, m_offsets[field_index]);

	case FieldMeta.TYPE_SHORT:
	  return FieldMeta.shortToString(Packer.getShort(buffer, m_offsets[field_index]));

	case FieldMeta.TYPE_INT:
	  return FieldMeta.intToString(Packer.getInt(buffer, m_offsets[field_index]));

        case FieldMeta.TYPE_LONG:
          return FieldMeta.longToString(Packer.getLong(buffer, m_offsets[field_index]));

	default:
	  throw new RuntimeException();
	}
    }

  public void setStringInBuffer(String value,
				int field_index,
				byte[] buffer)
    {
      FieldMeta field = m_fields[field_index];
      switch (field.getType())
	{
	case FieldMeta.TYPE_STRING:
	  Packer.putString(value, field.getLength(), buffer, m_offsets[field_index]);
	  break;

        case FieldMeta.TYPE_USTRING:
          Packer.putUString(value, field.getLength(), buffer, m_offsets[field_index]);
          break;

	case FieldMeta.TYPE_SHORT:
	  Packer.putShort((short)Integer.parseInt(value), buffer, m_offsets[field_index]);
	  break;

	case FieldMeta.TYPE_INT:
	  Packer.putInt(Integer.parseInt(value), buffer, m_offsets[field_index]);
	  break;

        case FieldMeta.TYPE_LONG:
          Packer.putLong(Long.parseLong(value), buffer, m_offsets[field_index]);
          break;

	default:
	  throw new RuntimeException();
	}
    }

  public void setShortInBuffer(short value,
			       int field_index,
			       byte[] buffer)
    {
      setLongInBuffer(value, field_index, buffer);
    }

  public void setIntInBuffer(int value,
			     int field_index,
			     byte[] buffer)
    {
      setLongInBuffer(value, field_index, buffer);
    }

  public void setLongInBuffer(long value,
                              int field_index,
                              byte[] buffer)
    {
      FieldMeta field = m_fields[field_index];
      switch (field.getType())
	{
	case FieldMeta.TYPE_STRING:
        case FieldMeta.TYPE_USTRING:
	  Packer.putString(String.valueOf(value), field.getLength(), buffer, m_offsets[field_index]);
	  break;

	case FieldMeta.TYPE_SHORT:
	  Packer.putShort((short)value, buffer, m_offsets[field_index]);
	  break;

	case FieldMeta.TYPE_INT:
	  Packer.putInt((int)value, buffer, m_offsets[field_index]);
	  break;

        case FieldMeta.TYPE_LONG:
          Packer.putLong(value, buffer, m_offsets[field_index]);
          break;

	default:
	  throw new RuntimeException();
	}
    }

  public byte[] getBlobFromBuffer(int field_index,
				  byte[] buffer,
				  Storage blobs)
    throws IOException, StorageException
    {
      FieldMeta field = m_fields[field_index];
      if (field.getType() != FieldMeta.TYPE_BLOB)
	{
	  throw new RuntimeException();
	}

      int record_id = Packer.getInt(buffer, m_offsets[field_index]) - 1;
      if (record_id == Storage.NULL)
	{
	  return new byte[0];
	}
      else
	{
	  return blobs.get(record_id);
	}
    }

   public void setBlobInBuffer(byte[] value,
			       int field_index,
			       byte[] buffer,
			       Storage blobs)
      throws IOException, StorageException
   {
      FieldMeta field = m_fields[field_index];
      if (field.getType() != FieldMeta.TYPE_BLOB) {
	 throw new RuntimeException();
      }

      int record_id = Packer.getInt(buffer, m_offsets[field_index]) - 1;
      if (record_id != Storage.NULL) {
	 blobs.release(record_id);
      }

      record_id = blobs.put(value) + 1;
      Packer.putInt(record_id, buffer, m_offsets[field_index]);
   }

  public void deleteBlobsForBuffer(byte[] buffer,
				   Storage blobs)
    throws IOException, StorageException
    {
      for (int i = 0; i < m_fields.length; i++)
	{
	  FieldMeta field = m_fields[i];
	  if (field.getType() == FieldMeta.TYPE_BLOB)
	    {
	      int record_id = Packer.getInt(buffer, m_offsets[i]) - 1;
	      if (record_id != Storage.NULL)
		{
		  blobs.release(record_id);
		}
	    }
	}
    }

  private boolean[] m_indexedField;
  private Hashtable m_fieldsByName;
  private FieldMeta[] m_fields;
  private int[] m_offsets;
  private int m_length;
}
