///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.utility.storage;

import java.io.*;

public class Sequence
implements Commitable
{
   public Sequence(String path,
                   String dbname)
   throws IOException, StorageException
   {
      FieldMeta[] fms = {
         new FieldMeta(COL_Name,   MAX_NAME_LENGTH, FieldMeta.TYPE_STRING),
         new FieldMeta(COL_Number, FieldMeta.TYPE_LONG)
      };
      RecordMeta rm = new RecordMeta(fms);
      String[] i1_fields = { COL_Name};
      IndexMeta[] ims = {
         new IndexMeta(NDX_ByName, rm, i1_fields)
      };
      m_db = new SimpleDatabase(rm, ims, path, dbname);
   }

   public void commitTo(CommitManager cmgr)
   throws IOException
   {
      m_db.commitTo(cmgr);
   }

   public void commitWrite()
   throws IOException
   {
      m_db.commitWrite();
   }

   public void commitFailed()
   {
      m_db.commitFailed();
   }

   public void commitSuccessful()
   {
      m_db.commitSuccessful();
   }

   public void commitRecovered()
   {
      m_db.commitRecovered();
   }

   public synchronized void sync()
   throws IOException, StorageException
   {
      m_db.sync();
   }

   public synchronized void abort()
   {
      m_db.abort();
   }

   public synchronized void close()
   {
      m_db.close();
   }

   public synchronized long next(String name)
   throws IOException, StorageException
   {
      Record key = m_db.getBlank();
      key.setString(COL_Name, name);
      try {
         Record r = m_db.find(NDX_ByName, key);
         long value = r.getLong(COL_Number);
         r.setLong(COL_Number, value + 1);
         m_db.update(r);
         return value;
      } catch (UnknownKeyException ex) {
         key.setLong(COL_Number, 1);
         m_db.insert(key);
         return 0;
      }
   }

   public synchronized static Sequence initSingleton(String dirname,
                                                     String dbname)
   throws IOException, StorageException
   {
      if (s_instance != null) {
         throw new RuntimeException("singleton already initialized");
      }
      s_instance = new Sequence(dirname, dbname);
      return s_instance;
   }

   public synchronized static void destroySingleton()
   {
      if (s_instance == null) {
         throw new RuntimeException("no singleton to destroy");
      }
      s_instance.close();
      s_instance = null;
   }

   public synchronized static Sequence s()
   {
      return s_instance;
   }

   private Database m_db;

   private static Sequence s_instance;

   private static final int MAX_NAME_LENGTH   = 24;

   private static final String COL_Name       = "Name";
   private static final String COL_Number     = "Number";

   private static final String NDX_ByName     = "ByName";
}
