///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.utility.storage;

import java.io.*;
import java.util.*;
import org.cooldevtools.utility.Assert;

public class test {
   public static void verify(int[] values,
                             IndexNavigator nav,
                             int num_values)
   throws IOException
   {
      try {
         int i = 0;
         while (nav.isValid()) {
            if (nav.getRecordId() != values[i]) {
               throw new RuntimeException("mismatch at index " + i + ": expect " + values[i]
                                          + " found " + nav.getRecordId());
            }
            nav.next();
            i++;
         }
         if (i != num_values) {
            throw new RuntimeException("expected " + num_values + " values but found " + i);
         }
      } finally {
         nav.close();
      }
   }

   public static void main(String[] args)
   throws Exception
   {
      if (Assert.DEBUG) Assert.pre(FieldMeta.longToString(100).compareTo(FieldMeta.longToString(99)) > 0);
      if (Assert.DEBUG) Assert.pre(FieldMeta.longToString(99).compareTo(FieldMeta.longToString(100)) < 0);
      if (Assert.DEBUG) Assert.pre(FieldMeta.longToString(99).compareTo(FieldMeta.longToString(99)) == 0);
      if (Assert.DEBUG) Assert.pre(FieldMeta.longToString(Long.MAX_VALUE).compareTo(FieldMeta.longToString(Long.MAX_VALUE - 1)) > 0);
      if (Assert.DEBUG) Assert.pre(FieldMeta.longToString(Long.MAX_VALUE - 1).compareTo(FieldMeta.longToString(Long.MAX_VALUE)) < 0);
      if (Assert.DEBUG) Assert.pre(FieldMeta.longToString(Long.MAX_VALUE).compareTo(FieldMeta.longToString(0)) > 0);
      if (Assert.DEBUG) Assert.pre(FieldMeta.longToString(0).compareTo(FieldMeta.longToString(Long.MAX_VALUE)) < 0);

      if (Assert.DEBUG) Assert.pre(FieldMeta.intToString(100).compareTo(FieldMeta.intToString(99)) > 0);
      if (Assert.DEBUG) Assert.pre(FieldMeta.intToString(99).compareTo(FieldMeta.intToString(100)) < 0);
      if (Assert.DEBUG) Assert.pre(FieldMeta.intToString(99).compareTo(FieldMeta.intToString(99)) == 0);

      if (Assert.DEBUG) Assert.pre(FieldMeta.shortToString((short)100).compareTo(FieldMeta.shortToString((short)99)) > 0);
      if (Assert.DEBUG) Assert.pre(FieldMeta.shortToString((short)99).compareTo(FieldMeta.shortToString((short)100)) < 0);
      if (Assert.DEBUG) Assert.pre(FieldMeta.shortToString((short)99).compareTo(FieldMeta.shortToString((short)99)) == 0);

      File f = new File("_index.dat");
      f.delete();

      final int ORDER = 10;
      BTreeIndex index = new BTreeIndex(new FileStorage(f.getPath(),
                                                        BTreeIndex.calcRecordLength(4, ORDER)),
                                        4,
                                        ORDER);
      try {
         final int LOOPS = 1000;

         int values[] = new int[LOOPS];
         for (int i = 0; i < LOOPS; i++) {
            values[i] = 0x1000 + i;
         }

         System.out.println("inserting " + LOOPS + " values.");
         long start = System.currentTimeMillis();
         for (int i = 0; i < values.length; i++) {
            String key = Integer.toHexString(values[i]);
            index.put(key, values[i]);
         }
         System.out.println("time=" + (System.currentTimeMillis() - start));

         System.out.println("seeking " + LOOPS + " values.");
         start = System.currentTimeMillis();
         for (int i = 0; i < values.length; i++) {
            String key = Integer.toHexString(values[i]);
            if (index.get(key) != values[i]) {
               System.out.println("mismatch: " + values[i] + " != " + index.get(key));
            }
         }
         System.out.println("time=" + (System.currentTimeMillis() - start));

         System.out.println("verifying " + LOOPS + " values.");
         start = System.currentTimeMillis();
         verify(values, index.navigate(""), values.length);
         System.out.println("time=" + (System.currentTimeMillis() - start));

         System.out.println("deleting " + LOOPS + " values.");
         start = System.currentTimeMillis();
         for (int i = values.length - 1; i >= 0; i--) {
            String key = Integer.toHexString(values[i]);
            index.remove(key);

            /*
            try
         {
           index.get(key);
           System.out.println("key " + key + " not removed!");
         }
            catch (UnknownKeyException ex)
         {
         }
   
            verify(values, index.navigate(""), i);
            */
         }
         System.out.println("time=" + (System.currentTimeMillis() - start));

         verify(values, index.navigate(""), 0);
      } finally {
         index.close();
      }
   }

   public static void main2(String[] args)
   throws Exception
   {
      File f = new File("_index.dat");
      f.delete();
      BinaryTreeIndex index = new BinaryTreeIndex(new FileStorage(f.getPath(),
                                                                  BinaryTreeIndex.calcRecordLength(4)),
                                                  4);

      Random r = new Random(1000);
      final int LOOPS = 5;

      int values[] = new int[LOOPS];
      for (int i = 0; i < LOOPS; i++) {
         values[i] = (r.nextInt() & 0xfff) | 0x1000;
      }

      for (int i = 0; i < values.length; i++) {
         System.out.println("loop " + i);
         String key = Integer.toHexString(values[i]);
         index.put(key, values[i]);
      }

      for (int i = 0; i < values.length; i++) {
         String key = Integer.toHexString(values[i]);
         if (index.get(key) != values[i]) {
            System.out.println("mismatch: " + values[i] + " != " + index.get(key));
         }
      }

      for (IndexNavigator nav = index.navigate(""); nav.isValid(); nav.next()) {
         System.out.println("key '" + nav.getKey() + "' value " + nav.getRecordId());
      }

      for (int i = 0; i < values.length; i++) {
         String key = Integer.toHexString(values[i]);
         index.remove(key);
         try {
            index.get(key);
            System.out.println("key " + key + " not removed!");
         } catch (UnknownKeyException ex) {
            
         }
      }
   }
}
