///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id$
//
// Copyright (C) 2000 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

package org.cooldevtools.workflow;
import org.cooldevtools.utility.LinkedList;

public class BasicQueue
  implements Queue
{
  public BasicQueue()
    {
      m_closed = false;
      m_immediate = false;
      m_queue = new LinkedList();
    }

  /**
     Close the queue.  All waiting workers will receive a
     QueueClosedException when the last object has been
     removed from the queue.
  */
  public synchronized void close()
    {
      m_closed = true;
      notify();
    }

  /**
     Close the queue.  All waiting workers will receive a
     QueueClosedException immediately even if there are
     objects available in the queue.
  */
  public synchronized void closeImmediately()
    {
      m_immediate = true;
      notify();
    }

  /**
     Put an object into the queue.  The next waiting worker will
     be given the object.
  */
  public synchronized void put(Object item)
    {
      m_queue.push(item);
      notify();
    }

  /**
     Get the next available object from the queue.
     Throws QueueClosedException if the queue has been closed and no
     items remain.
  */
  public synchronized Object get()
    throws QueueClosedException
    {
      while (true)
	{
	  boolean empty = m_queue.isEmpty();
	  if (m_immediate || m_closed && empty)
	    {
	      notify();
	      throw new QueueClosedException();
	    }

	  if (!empty)
	    {
	      notify();
	      return m_queue.popLast();
	    }

	  try
	    {
	      wait();
	    }
	  catch (InterruptedException e)
	    {
	      e.printStackTrace();
	    }
	}
    }

  private boolean    m_closed;
  private boolean    m_immediate;
  private LinkedList m_queue;
}
